<?php
/**
 * Реализация класса, отвечающего за вывод CAPTCHA на страницах сайта.
 *
 * Конструктор класса устанавливает значения переменных класса в соответсвии с настройками системы. 
 *
 * @author Азамат Олисаев <azamat@binn.ru>
 * @version 4.0
 * @package SB_SiteCore
 * @copyright Copyright (c) 2009, OOO "СИБИЭС Групп"
 */

/**
 * Класс, отвечающий за вывод CAPTCHA.
 *
 * @author Азамат Олисаев <azamat@binn.ru>
 * @version 4.0
 * @package SB_SiteCore
 * @copyright Copyright (c) 2009, OOO "СИБИЭС Групп"
 */

class sbCaptcha
{
    /**
     * Тип фона CAPTCHA.
     *
     * Возможные значения:
     * <ul>
     * <li>img - изображения на фоне
     * <li>string - фон из букв
     * <li>color - сплошной цвет
     * </ul>
     *  
     * @var string
     */
    public $mTypeCaptcha = 'string';

    /**
     * Цвет фона CAPTCHA
     * 
     * Из данного массива в случайном порядке будет выбираться цвет фона CAPTCHA.
     * Формат цветов - #RRGGBB.
     * 
     * @var array
     */
    public $mColorFon = array('#F8922E', '#995B1C');
    
    /**
     * Использовать шум в виде линий (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    public $mShumLines = false;
    
    /**
     * Использовать шум в виде точек (TRUE) или нет (FALSE)
     *
     * @var bool
     */
    public $mShumPoints = false;
    
    /**
     * Массив цветов для шумов
     * 
     * Из данного массива в случайном порядке будут выбираться цвета для шумов.
     * Формат цветов - #RRGGBB.
     *
     * @var array
     */
    public $mLinesColor = array('#A71D21', '#C47524');    

    /**
     * Ширина изображения CAPTCHA
     *
     * @var int
     */
    public $mWidth = 140;
    
    /**
     * Высота изображения CAPTCHA
     *
     * @var int
     */
    public $mHeight = 80;

    /**
     * Размер шрифта кода CAPTCHA
     * 
     * Значение интервала "От".
     * 
     * @var int
     */
    public $mTextSizeFrom = 30;
    
    /**
     * Размер шрифта кода CAPTCHA
     * 
     * Значение интервала "До".
     * 
     * @var int
     */
    public $mTextSizeTo = 60;
    
    /**
     * Длина кода CAPTCHA
     * 
     * Значение интервала "От".
     * 
     * @var int
     */
    public static $mCodeLengthFrom = 10000;
    
    /**
     * Длина кода CAPTCHA
     * 
     * Значение интервала "До".
     * 
     * @var int
     */
    public static $mCodeLengthTo = 100000;
    
    /**
     * Массив цветов кода CAPTCHA
     *
     * Из данного массива в случайном порядке будут выбираться цвета для цифр кода.
     * Формат цветов - #RRGGBB.
     * 
     * @var array
     */
    public $mTextColor = array('#A71D21', '#C47524');
    
    /**
     * Массив цветов фонового текста
     * 
     * Из данного массива в случайном порядке будут выбираться цвета для шумовых символов на фоне.
     * Формат цветов - #RRGGBB.
     * 
     * @var array
     */
    public $mTextFonColor = array('#A71D21', '#C47524');
    
    /**
     * Имя файла шрифта кода CAPTCHA
     * 
     * Все файлы должны располагаться в папке /cms/lang/<язык>/fonts/.
     *
     * @var string
     */
    public $mTextFont = 'rubbfont.ttf';
    
    /**
     * URL изображения CAPTCHA.
     * 
     * Если указана директория с изображениями для CAPTCHA, то изображения будут браться в случайном порядке. 
     * 
     * @var string
     */
    public $mImageUrl = '';

    /**
     * Угол наклона символа CAPTCHA
     *
     * Значение интервала "От".
     * 
     * @var int
     */    
    public $mTextAngleFrom = -20;

    /**
     * Угол наклона символа CAPTCHA
     * 
     * Значение интервала "До".
     * 
     * @var int
     */    
    public $mTextAngleTo = 20;

    /**
     * Идентификатор ресурса изображения
     *
     * @var resource
     */
    private $mImageHandle = null;
    
    /**
     * Сгенерированный код CAPTCHA
     *
     * @var string
     */
    private $mCode;
    
    /**
     * Конструктор класса
     *
     * Считывает настройки CAPTCHA на сайте для текущего домена.
     * 
     * @param string $code Код КАПЧИ.
     */
    public function __construct($code)
    {
        if(($setting = sbPlugins::getSetting('sb_captcha_type')) != false)
        {
            $this->mTypeCaptcha = $setting;
        }
        
    	if(($setting = sbPlugins::getSetting('sb_captcha_img')) != false)
        {
            $this->mImageUrl = sb_str_replace(SB_DOMAIN, '', $setting);
        }
        
        if(($setting = sbPlugins::getSetting('sb_captcha_height')) != false)
        {
            $this->mHeight = $setting;
        }
        
        if(($setting = sbPlugins::getSetting('sb_captcha_width')) != false)
        {
            $this->mWidth = $setting;
        }
        
        if(($setting = sbPlugins::getSetting('sb_captcha_fon_shum_colors')) != false)
        {
            $setting = explode(' ', $setting);
            $this->mLinesColor = $setting;
        }
        
        if(($setting = sbPlugins::getSetting('sb_captcha_code_colors')) != false)
        {
            $setting = explode(' ', $setting);
            $this->mTextColor = $setting;
        }
        
        if(($setting = sbPlugins::getSetting('sb_captcha_code_fon_colors')) != false)
        {
            $setting = explode(' ', $setting);
            $this->mTextFonColor = $setting;
        }
        
        if(($setting = sbPlugins::getSetting('sb_captcha_fsize_from')) != false)
        {
            $this->mTextSizeFrom = $setting;
        }
        
        if(($setting = sbPlugins::getSetting('sb_captcha_fsize_to')) != false)
        {
            $this->mTextSizeTo = $setting;
        }
        
        if(($setting = sbPlugins::getSetting('sb_captcha_fonts')) != false)
        {
            $this->mTextFont = $setting;
        }
        
        if(($setting = sbPlugins::getSetting('sb_captcha_fon_color')) != false)
        {
        	$setting = explode(' ', $setting);
            $this->mColorFon = $setting;
        }
        
        if(($setting = sbPlugins::getSetting('sb_captcha_shum_lines')) != false)
        {
            $this->mShumLines = $setting;
        }
        
        if(($setting = sbPlugins::getSetting('sb_captcha_shum_points')) != false)
        {
            $this->mShumPoints = $setting;
        }
        
        $this->mCode = $code;
    }

    /**
     * Генерация кода КАПЧИ
     *
     * @return string Код КАПЧИ.
     */
    static function getCode()
    {
    	if(($setting = sbPlugins::getSetting('sb_captcha_code_length_from')) != false)
        {
            self::$mCodeLengthFrom = $setting;
        }
        
        if(($setting = sbPlugins::getSetting('sb_captcha_code_length_to')) != false)
        {
            self::$mCodeLengthTo = $setting;
        } 
        
    	$code = rand(self::$mCodeLengthFrom, self::$mCodeLengthTo);

    	sbProgStartSession();
        
        if (!isset($_SESSION['sb_captcha']) || !is_array($_SESSION['sb_captcha']))
        {
        	$_SESSION['sb_captcha'] = array();
        }
        
        $time = time().uniqid();
        $_SESSION['sb_captcha'][$time] = $code;
        
        return $time;
    }
    
    /**
     * Функция, выдающая идентификатор случайного цвета из массива
     *
     * @param array $colors Массив цветов.
     * 
     * @return resource Идентификатор цвета.
     */
    private function RandColor($colors)
    {
    	if (!is_resource($this->mImageHandle))
    		return false;

    	if (!is_array($colors) || count($colors) < 1)
    		return false;
    		
    	$col_ind = rand(0, count($colors) - 1);
        $col = $this->Color($colors[$col_ind]);

    	$color = imagecolorexact($this->mImageHandle, $col[0], $col[1], $col[2]);
        if($color == -1) 
        {
	        if(imagecolorstotal($this->mImageHandle) >= 255) 
	        {
	            $color = imagecolorclosest($this->mImageHandle, $col[0], $col[1], $col[2]);
            } 
            else 
            {
                $color = imagecolorallocate($this->mImageHandle, $col[0], $col[1], $col[2]);
            }
        }
        
        return $color;
    }

    /**
     * Функция, возвращающая массив шестнадцатиричных значений красного, зеленого и синего спектра переданного ей цвета.
     *
     * @param string $rgb Цвет в 16-ричной системе счисления.
     * 
     * @return array Массив, где  array[0] - красный, array[1] - зеленый, array[2] - синий.
     */
    private function Color($rgb)
    {
        for($i = 0, $t = 1; $i < 3; $i++, $t += 2)
        {
            $arr[$i] = sb_substr($rgb, $t, 2);
        }
        
        $rgb = array(hexdec($arr[0]), hexdec($arr[1]), hexdec($arr[2]));
        
        return $rgb;
    }

    /**
     * Функция, осуществляющая вывод кода CAPTCHA
     *
     * @return string Сгенерированный код CAPTCHA.
     */
    private function SetCode()
    {
    	if (!is_resource($this->mImageHandle))
    		return false;

    	$font = preg_replace('/[^A-Z\.a-z0-9\-\_]+/i', '', $this->mTextFont);
        if (!@file_exists(SB_CMS_LANG_PATH.'/fonts/'.$font) || !@is_file(SB_CMS_LANG_PATH.'/fonts/'.$font))
        {
        	sb_add_system_message(sprintf(SB_CAPTCHA_FONT_ERR, $font), SB_MSG_WARNING);
            return false;
        }   	
    	
    	if (!$this->mCode)
    	{
    		sb_add_system_message(SB_CAPTCHA_CODE_ERR, SB_MSG_WARNING);
    		return false;
    	}
    	
    	$x = 10;
        for($i = 0; $i < sb_strlen($this->mCode); $i++)
        {
            $size = rand($this->mTextSizeFrom, $this->mTextSizeTo);
            $angle = rand($this->mTextAngleFrom, $this->mTextAngleTo);
            $color = $this->RandColor($this->mTextColor);
            
            if ($color === false || $color === -1)
            {
            	sb_add_system_message(SB_CAPTCHA_COLOR_ERR, SB_MSG_WARNING);
            	return false;
            }
            
            if (!@imagettftext($this->mImageHandle, $size, $angle, $x, round(rand($size, $this->mHeight)), $color, SB_CMS_LANG_PATH.'/fonts/'.$font, sb_substr($this->mCode, $i, 1)))
            {
            	sb_add_system_message(SB_CAPTCHA_COLOR_ERR, SB_MSG_WARNING);
            	return false;
            }
            $x += ($this->mWidth - 10) / sb_strlen($this->mCode);
        }
    }
    
    /**
     * Функция, возвращающая изображение CAPTCHA в виде строки
     *
     * @return string Код изображения.
     */
    public function GetTuring()
    {
    	if (!function_exists('imagecreatetruecolor') || !function_exists('imagettftext'))
    	{
    		sb_add_system_message(SB_CAPTCHA_GD_ERR, SB_MSG_WARNING);
    		return false;
    	}
    	
    	$this->mImageHandle = @imagecreatetruecolor($this->mWidth, $this->mHeight);
        if (!$this->mImageHandle)
        {
        	sb_add_system_message(SB_CAPTCHA_COLOR_ERR, SB_MSG_WARNING);
            return false;
        }
            	
        $fonColor = $this->RandColor($this->mColorFon);
        @imagefill($this->mImageHandle, 0, 0, $fonColor);
        
        switch($this->mTypeCaptcha)
        {
            case 'img':
                if(trim($this->mImageUrl) == '')
                {
                    sb_add_system_message(sprintf(SB_CAPTCHA_IMG_URL_ERR, $this->mImageUrl), SB_MSG_WARNING);
                    break;
                }

                $GLOBALS['sbVfs']->mLocal = true;
        		if (sb_stripos($this->mImageUrl, 'http://') !== false)
                {
                	$this->mImageUrl = sb_str_replace(array('http://', 'www.', SB_COOKIE_DOMAIN), array('', '', ''), $this->mImageUrl);
                }
                	
                $this->mImageUrl = str_replace('//', '/', SB_BASEDIR.'/'.$this->mImageUrl);
                $this->mImageUrl = rtrim($this->mImageUrl, '\\/');
                
                $img_file = '';
                
                if($GLOBALS['sbVfs']->is_dir($this->mImageUrl))
                {
                	$dir_handle = $GLOBALS['sbVfs']->opendir($this->mImageUrl);
                    if (!$dir_handle)
                    {
                    	sb_add_system_message(sprintf(SB_CAPTCHA_IMG_DIR_ERR, $this->mImageUrl), SB_MSG_WARNING);
                    	$GLOBALS['sbVfs']->mLocal = false;
                    	break;
                    }
                    
                    $img_list = array();
                    while (($file = $GLOBALS['sbVfs']->readdir($dir_handle)) != false)
                    {
                        $ext = strtolower(sb_substr($file, -4));
                        if($ext == '.jpg' || $ext == '.gif')
                        {
                            $img_list[] = $file;
                        }
                    }

                    if (count($img_list) == 0)
                    {
                    	sb_add_system_message(sprintf(SB_CAPTCHA_IMG_EMPTY_DIR_ERR, $this->mImageUrl), SB_MSG_WARNING);
                    	$GLOBALS['sbVfs']->mLocal = false;
                    	break;
                    }
                    	
                    $img_file = $this->mImageUrl.'/'.$img_list[rand(0, count($img_list) - 1)];
                }
                else
                {
                	$img_file = $this->mImageUrl;
                }
                	
                if (!$GLOBALS['sbVfs']->is_file($img_file))
                {
                	sb_add_system_message(sprintf(SB_CAPTCHA_IMG_URL_ERR, $img_file), SB_MSG_WARNING);
                	$GLOBALS['sbVfs']->mLocal = false;
                	break;	
                }
                
                $ext = strtolower(sb_substr($img_file, -4));
                if ($ext == '.jpg')
                   	$im_handle = $GLOBALS['sbVfs']->imagecreatefromjpeg($img_file);
                else
                   	$im_handle = $GLOBALS['sbVfs']->imagecreatefromgif($img_file);                   

                $im_size = $GLOBALS['sbVfs']->getimagesize($img_file);
                
                $GLOBALS['sbVfs']->mLocal = false;
                
				if (!$im_handle)
				{
					sb_add_system_message(sprintf(SB_CAPTCHA_IMG_ERR, $img_file), SB_MSG_WARNING);
					break;
				}
				
                $ratio = $im_size[0] / $this->mWidth;
                $w_dest = round($im_size[0] / $ratio);
                $h_dest = round($im_size[1] / $ratio);
                
                if (!@imagecopyresampled($this->mImageHandle, $im_handle, 0, 0, 0, 0, $w_dest, $h_dest, $im_size[0], $im_size[1]))
                {
                	sb_add_system_message(sprintf(SB_CAPTCHA_IMG_ERR, $img_file), SB_MSG_WARNING);            
                }
                
                @imagedestroy($im_handle);
            	break;
            	
            case 'string':
                $chars = array('A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z');
                $font = preg_replace('/[^A-Z\.a-z0-9\-\_]+/i', '', $this->mTextFont);
        		if (!@file_exists(SB_CMS_LANG_PATH.'/fonts/'.$font) || !@is_file(SB_CMS_LANG_PATH.'/fonts/'.$font))
        		{
        			sb_add_system_message(sprintf(SB_CAPTCHA_FONT_ERR, $font), SB_MSG_WARNING);
        			break;
        		}
                            
                for($i = 0; $i < 15; $i++)
                {
                	$size = rand($this->mTextSizeFrom / 2, $this->mTextSizeTo / 2);
                	
                    $ypos = rand(-20, $this->mHeight);
                    $xpos = rand(-20, $this->mWidth);

                    $color = $this->RandColor($this->mTextFonColor);
	                if (!$color)
		            {
		            	sb_add_system_message(SB_CAPTCHA_COLOR_ERR, SB_MSG_WARNING);
		            	return false;
		            }
            
                    if (!@imagettftext($this->mImageHandle, $size, 0, $xpos, $ypos, $color, SB_CMS_LANG_PATH.'/fonts/'.$font, $chars[rand(0, 22)]))
                    	break;
                }
            	break;
        }
        
        $this->SetCode();
        
        if (!isset($fonColor))
        {
        	$fonColor = $this->RandColor($this->mColorFon);
        	if (!$fonColor)
            {
            	sb_add_system_message(SB_CAPTCHA_COLOR_ERR, SB_MSG_WARNING);
            	return false;
            }
        }
        	
        if($this->mShumLines)
        {
            for($i = 0; $i < $this->mWidth; $i += 3)
	        {
	            $color = $this->RandColor($this->mLinesColor);
	        	if (!$color)
	            {
	            	sb_add_system_message(SB_CAPTCHA_COLOR_ERR, SB_MSG_WARNING);
	            	return false;
	            }
	            @imageellipse($this->mImageHandle, $this->mWidth - $i, $this->mHeight / 2 , $this->mWidth, $this->mHeight * 2, $color);
	        }
        }
        
        if($this->mShumPoints)
        {
            for($i = 0; $i < 2000; $i++)
            {
                $color = $this->RandColor($this->mLinesColor);
            	if (!$color)
	            {
	            	sb_add_system_message(SB_CAPTCHA_COLOR_ERR, SB_MSG_WARNING);
	            	return false;
	            }
                @imagesetpixel($this->mImageHandle, rand(0, $this->mWidth), rand(0, $this->mHeight), $color);        
            }
        }

        ob_start();
        $GLOBALS['sbVfs']->mLocal = true;
        
        if (function_exists('imagejpeg'))
        {
        	$GLOBALS['sbVfs']->imagejpeg($this->mImageHandle, null, 100);
        }
        elseif (function_exists('imagegif'))
        {
        	$GLOBALS['sbVfs']->imagegif($this->mImageHandle, null);
        }
    	elseif (function_exists('imagepng'))
        {
        	$GLOBALS['sbVfs']->imagepng($this->mImageHandle, null, 8);
        }
        
        $data = ob_get_contents();
        $GLOBALS['sbVfs']->mLocal = false;
        ob_end_clean();
        
        return $data;
    }
}
?>