<?php
/**
 * Реализация класса, использующегося классом sbRSA и отвечающего за генерацию открытого и закрытого ключей.
 *
 * Данный класс использует при работе класс sbMath.
 *
 * @see sbMath
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Auth
 * @copyright Copyright (c) 2008, OOO "СИБИЭС Групп"
 */

/**
 * @ignore
 */
require_once SB_CMS_LIB_PATH.'/rsa/sbMath.inc.php';

/**
 * Класс, отвечающий за генерацию открытого и закрытого ключей для алгоритма RSA
 *
 * Данный класс использует при работе класс sbMath.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Auth
 * @copyright Copyright (c) 2008, OOO "СИБИЭС Групп"
 */
class sbKeyPair
{
    /**
     * Ссылка на объект класса, работающего с большими целыми числами.
     *
     * @var object sbMath
     */
    private $_math_obj;

    /**
     * Длина ключей
     *
     * @var int
     */
    private $_key_len;

    /**
     * Открытый ключ
     *
     * @var object sbKey
     */
    private $_public_key;

    /**
     * Закрытый ключ
     *
     * @var object sbKey
     */
    private $_private_key;

    /**
     * Атрибуты пары ключей RSA [version, n, e, d, p, q, dmp1, dmq1, iqmp] как ассоциативный массив
     *
     * @var array
     */
    private $_attrs;

    /**
     * Возвращает названия атрибутов пары ключей RSA
     *
     * @return array  Массив с названиями атрибутов пары ключей RSA
     */
    private function _get_attr_names() 
    {
        return array('version', 'n', 'e', 'd', 'p', 'q', 'dmp1', 'dmq1', 'iqmp');
    }

    /**
     * Конструктор класса.
     *
     * @param int $key_len Битовая длина пары ключей, которые будут сгенерированы в конструкторе
     * @param bool $error Содержит TRUE, если в результате создания класса возникли ошибки.
     * 
     * @return sbKeyPair
     */
    public function __construct($key_len, &$error)
    {
        $error = true;
        
        $this->_math_obj = new sbMath($error);
        
        if ($error)
            return;

        // генерируем пару ключей
        if (!$this->generate($key_len)) 
        {
            return;
        }
        
        $error = false;
    }
    
    /**
     * Генерирует новую пару ключей длиной $key_len.
     *
     * @param int $key_len  Битовая длина пары ключей, которые будут сгенерированы.
     * 
     * @return bool TRUE при удачной генерации, FALSE при ошибке.
     */
    public function generate($key_len = null)
    {
        if (is_null($key_len)) 
        {
            $key_len = $this->_key_len;
            if (is_null($key_len)) 
            {
                return false;
            }
        }

        if ($key_len < 8) 
        {
            $key_len = 8;
        }
        $this->_key_len = $key_len;

        // устанавливаем [e] в 0x10001 (65537)
        $e = $this->_math_obj->bin2int("\x01\x00\x01");

        // генерируем [p], [q] и [n]
        $p_len = intval(($key_len + 1) / 2);
        $q_len = $key_len - $p_len;
        $p1 = $q1 = 0;
        do 
        {
            // генерируем простое число [$p] длиной [$p_len], отвечающее следующим условиям:
            // GCD($e, $p - 1) = 1
            do 
            {
                $p = $this->_math_obj->getPrime($p_len);
                $p1 = $this->_math_obj->dec($p);
                $tmp = $this->_math_obj->GCD($e, $p1);
            } 
            while (!$this->_math_obj->isOne($tmp));
            
            // генерируем простое число [$q] длиной [$q_len], отвечающее следующим условиям:
            // GCD($e, $q - 1) = 1
            // $q != $p
            do 
            {
                $q = $this->_math_obj->getPrime($q_len);
                $q1 = $this->_math_obj->dec($q);
                $tmp = $this->_math_obj->GCD($e, $q1);
            } 
            while (!$this->_math_obj->isOne($tmp) && !$this->_math_obj->cmpAbs($q, $p));
            
            // если (p < q), то меняем их местами
            if ($this->_math_obj->cmpAbs($p, $q) < 0) 
            {
                $tmp = $p;
                $p = $q;
                $q = $tmp;
                $tmp = $p1;
                $p1 = $q1;
                $q1 = $tmp;
            }
            
            $n = $this->_math_obj->mul($p, $q);
        } 
        while ($this->_math_obj->bitLen($n) != $key_len);

        // d = 1/e mod (p - 1) * (q - 1)
        $pq = $this->_math_obj->mul($p1, $q1);
        $d = $this->_math_obj->invmod($e, $pq);

        // dmp1 = d mod (p - 1)
        $dmp1 = $this->_math_obj->mod($d, $p1);

        // dmq1 = d mod (q - 1)
        $dmq1 = $this->_math_obj->mod($d, $q1);

        // iqmp = 1/q mod p
        $iqmp = $this->_math_obj->invmod($q, $p);

        // сохраняем атрибуты пары ключей RSA
        $this->_attrs = array(
            'version' => "\x00",
            'n' => $this->_math_obj->int2bin($n),
            'e' => $this->_math_obj->int2bin($e),
            'd' => $this->_math_obj->int2bin($d),
            'p' => $this->_math_obj->int2bin($p),
            'q' => $this->_math_obj->int2bin($q),
            'dmp1' => $this->_math_obj->int2bin($dmp1),
            'dmq1' => $this->_math_obj->int2bin($dmq1),
            'iqmp' => $this->_math_obj->int2bin($iqmp),
        );

        $n = $this->_attrs['n'];
        $e = $this->_attrs['e'];
        $d = $this->_attrs['d'];

        // пытаемся создать объект для открытого ключа
        $obj = new sbKey($n, $e, 'public', $error);
        if ($error) 
        {
            return false;
        }
        $this->_public_key = $obj;

        // пытаемся создать объект для закрытого ключа
        $obj = new sbKey($n, $d, 'private', $error);
        if ($error) 
        {
            return false;
        }
        $this->_private_key = $obj;

        return true;
    }

    /**
     * Возвращает открытый ключ
     *
     * @return object Открытый ключ, объект класса sbKey.
     */
    public function getPublicKey()
    {
        return $this->_public_key;
    }

    /**
     * Возвращает закрытый ключ
     *
     * @return object Закрытый ключ, объект класса sbKey.
     */
    public function getPrivateKey()
    {
        return $this->_private_key;
    }

    /**
     * Возвращает битовую длину пары ключей
     *
     * @return int Битовая длина пары ключей.
     */
    public function getKeyLength()
    {
        return $this->_key_len;
    }
}

/**
 * Класс, содержащий открытый или закрытый ключ
 *
 * Данный класс использует при работе класс sbMath.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Auth
 * @copyright Copyright (c) 2008, OOO "СИБИЭС Групп"
 */
class sbKey
{
    /**
     * Ссылка на объект класса, работающего с большими целыми числами.
     *
     * @var object sbMath
     */
    private $_math_obj;

    /**
     * Размерность
     *
     * @var string
     */
    private $_modulus;

    /**
     * Экспонента
     *
     * @var string
     */
    private $_exp;

    /**
     * Тип ключа
     * 
     * private - закрытый или public - открытый
     *
     * @var string
     */
    private $_key_type;

    /**
     * Битовая длина ключа
     *
     * @var int
     */
    private $_key_len;

    /**
     * Конструктор класса.
     *
     * @param string $modulus Размерность ключа.
     * @param string $exp Экспонента ключа.
     * @param string $key_type Тип ключа (public или private).
     * @param bool $error Содержит TRUE, если в результате создания класса возникли ошибки.
     * 
     * @return sbKey
     */
    function sbKey($modulus, $exp, $key_type, &$error)
    {
        $error = true;
        
        $this->_math_obj = new sbMath($error);
        
        if ($error)
            return;

        $this->_modulus = $modulus;
        $this->_exp = $exp;

        if (!in_array($key_type, array('private', 'public'))) 
        {
            return;
        }
        $this->_key_type = $key_type;

        $mod_num = $this->_math_obj->bin2int($this->_modulus);
        $exp_num = $this->_math_obj->bin2int($this->_exp);

        if ($this->_math_obj->cmpAbs($mod_num, $exp_num) <= 0) 
        {
            return;
        }

        $this->_key_len = $this->_math_obj->bitLen($mod_num);
        
        $error = false;
    }

    /**
     * Возвращает битовую блину ключа
     *
     * @return int Битовая длина ключа.
     */
    public function getKeyLength()
    {
        return $this->_key_len;
    }

    /**
     * Возвращает размерность ключа, как бинарную строку
     *
     * @return string Размерность ключа, как бинарная строка.
     */
    public function getModulus()
    {
        return $this->_modulus;
    }

    /**
     * Возвращает экспоненту ключа, как бинарную строку
     *
     * @return string Экспонента ключа, как бинарная строка.
     */
    public function getExponent()
    {
        return $this->_exp;
    }

    /**
     * Возвращает тип ключа (public, private)
     *
     * @return string Тип ключа (public, private)
     */
    public function getKeyType()
    {
        return $this->_key_type;
    }

    /**
     * Возвращает строковое представление ключа
     *
     * @return string Ключ как строка.
     */
    public function toString()
    {
        return base64_encode(
            serialize(array(
                $this->_modulus,
                $this->_exp,
                $this->_key_type,
            ))
        );
    }

    /**
     * Возвращает объект sbKey из строкового представления ключа
     *
     * @param $key_str RSA ключ.
     * @return object Объект класса sbKey.
     */
    static function fromString($key_str)
    {
        list($modulus, $exponent, $key_type) = unserialize(base64_decode($key_str));
        $obj = new sbKey($modulus, $exponent, $key_type, $error);
        
        if ($error)
            return false;
            
        return $obj;
    }
    
    /**
     * Проверяет валидность ключа
     *
     * @param object $key Объект класса sbKey для проверки.
     * @return bool TRUE, если ключ валидный, FALSE в ином случае.
     */
    static function isValid($key)
    {
        return (is_object($key) && strtolower(get_class($key)) === strtolower(__CLASS__));
    }
}

?>