<?php
/**
 * Реализация класса, использующегося классом sbRSA и отвечающего за математические преобразования алгоритма.
 *
 * Данный класс использует при работе библиотеку BCMath.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Auth
 * @copyright Copyright (c) 2008, OOO "СИБИЭС Групп"
 */

/**
 * Класс, отвечающий за математические преобразования алгоритма RSA
 *
 * Данный класс использует при работе библиотеку BCMath.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Auth
 * @copyright Copyright (c) 2008, OOO "СИБИЭС Групп"
 */
class sbMath
{
    /**
     * Конструктор класса
     * 
     * Проверяет, загруженно ли расширение BCMath.
     * 
     * @param bool $error Содержит TRUE, если в результате создания класса возникли ошибки.
     * 
     * @return sbMath
     */
    public function __construct(&$error)
    {
        $error = true;
        
        if (!extension_loaded('bcmath')) 
        {
            return;
        }
        
        $error = false;
    }
    
    private function _millerTest($num, $base)
    {
        if (!bccomp($num, '1')) 
        {
            return false;
        }
        $tmp = bcsub($num, '1');

        $zero_bits = 0;
        while (!bccomp(bcmod($tmp, '2'), '0')) 
        {
            $zero_bits++;
            $tmp = bcdiv($tmp, '2');
        }

        $tmp = $this->powmod($base, $tmp, $num);
        if (!bccomp($tmp, '1')) 
        {
            return true;
        }

        while ($zero_bits--) 
        {
            if (!bccomp(bcadd($tmp, '1'), $num)) 
            {
                return true;
            }
            $tmp = $this->powmod($tmp, '2', $num);
        }

        return false;
    }

    /**
     * Преобразует бинарное представление целого числа к нормальному.
     * 
     * @param string $str Бинарное представление целого числа.
     * 
     * @return string Целое число.
     */
    public function bin2int($str)
    {
        $result = '0';
        $n = strlen($str);
        do 
        {
            $result = bcadd(bcmul($result, '256'), ord($str{--$n}));
        } while ($n > 0);
        
        return $result;
    }

    /**
     * Преобразует целое число к бинарному представлению.
     *
     * @param string $num Целое число.
     * 
     * @return string Бинарное представление.
     */
    public function int2bin($num)
    {
        $result = '';
        do 
        {
            $result .= chr(bcmod($num, '256'));
            $num = bcdiv($num, '256');
        } 
        while (bccomp($num, '0'));
        
        return $result;
    }

    /**
     * Вычисляет $num1 * $num2
     *
     * @param string $num1
     * @param string $num2
     * 
     * @return string
     */
    public function mul($num1, $num2)
    {
        return bcmul($num1, $num2);
    }

    /**
     * Вычисляет $num1 % $num2
     *
     * @param string $num1
     * @param string $num2
     * 
     * @return string
     */
    public function mod($num1, $num2)
    {
        return bcmod($num1, $num2);
    }

    /**
     * Сравнивает abs($num1) с abs($num2).
     *
     * @param string $num1
     * @param string $num2
     * 
     * @return int -1, если abs($num1) < abs($num2); 0, если abs($num1) == abs($num2); 1, если abs($num1) > abs($num2)
     */
    public function cmpAbs($num1, $num2)
    {
        return bccomp($num1, $num2);
    }

    /**
     * Проверяет, является ли $num простым числом.
     *
     * @param string $num
     * 
     * @return bool TRUE, если $num сильное псевдопростое число, FALSE в ином случае.
     */
    private function _isPrime($num)
    {
        static $primes = null;
        static $primes_cnt = 0;
        
        if (is_null($primes)) 
        {
            // генерируем все случайные числа до 10000
            $primes = array();
            for ($i = 0; $i < 10000; $i++) 
            {
                $primes[] = $i;
            }
            $primes[0] = $primes[1] = 0;
            for ($i = 2; $i < 100; $i++) 
            {
                while (!$primes[$i]) 
                {
                    $i++;
                }
                $j = $i;
                for ($j += $i; $j < 10000; $j += $i) 
                {
                    $primes[$j] = 0;
                }
            }
            $j = 0;
            for ($i = 0; $i < 10000; $i++) 
            {
                if ($primes[$i]) 
                {
                    $primes[$j++] = $primes[$i];
                }
            }
            $primes_cnt = $j;
        }

        // пытаемся разделить число на малые простые числа
        for ($i = 0; $i < $primes_cnt; $i++) 
        {
            if (bccomp($num, $primes[$i]) <= 0) 
            {
                // число простое
                return true;
            }
            if (!bccomp(bcmod($num, $primes[$i]), '0')) 
            {
                // число делится на $primes[$i]
                return false;
            }
        }

        for ($i = 0; $i < 7; $i++) 
        {
            if (!$this->_millerTest($num, $primes[$i])) 
            {
                // $num не простое
                return false;
            }
        }
        
        // $num сильное псевдопростое
        return true;
    }

    /**
     * Генерирует простое число длиной $bits_cnt.
     *
     * @param int $bits_cnt Длина генерируемого просто числа.
     */
    public function getPrime($bits_cnt)
    {
        $bytes_n = intval($bits_cnt / 8);
        $bits_n = $bits_cnt % 8;
        do 
        {
            $str = '';
            for ($i = 0; $i < $bytes_n; $i++) 
            {
                $str .= chr($this->getRnd() & 0xff);
            }
            $n = $this->getRnd() & 0xff;
            $n |= 0x80;
            $n >>= 8 - $bits_n;
            $str .= chr($n);
            $num = $this->bin2int($str);

            if (!bccomp(bcmod($num, '2'), '0')) 
            {
                $num = bcadd($num, '1');
            }
            while (!$this->_isPrime($num)) 
            {
                $num = bcadd($num, '2');
            }
        } 
        while ($this->bitLen($num) != $bits_cnt);
        
        return $num;
    }

    /**
     * Возвращает случайное число.
     *
     * @return int Случайное число.
     */
    public function getRnd()
    {
        $a = explode(' ',microtime());
        
        return(int)($a[0]*1000000);    
    }
    
    /**
     * Вычисляет $num - 1
     *
     * @param string $num
     * 
     * @return string
     */
    public function dec($num)
    {
        return bcsub($num, '1');
    }

    /**
     * Сравнивает $num с 1.
     *
     * @param string $num
     * 
     * @return bool TRUE, если $num равно 1, FALSE в ином случае.
     */
    public function isOne($num)
    {
        return !bccomp($num, '1');
    }

    /**
     * Находи наибольший общий делитель $num1 и $num2
     *
     * @param string $num1
     * @param string $num2
     * 
     * @return string
     */
    public function GCD($num1, $num2)
    {
        do 
        {
            $tmp = bcmod($num1, $num2);
            $num1 = $num2;
            $num2 = $tmp;
        } 
        while (bccomp($num2, '0'));
        
        return $num1;
    }

    /**
     * Находит обратное число для $num размерности $mod.
     *
     * @param string $num
     * @param string $mod
     * 
     * @return string
     */
    public function invmod($num, $mod)
    {
        $x = '1';
        $y = '0';
        $num1 = $mod;
        do 
        {
            $tmp = bcmod($num, $num1);
            $q = bcdiv($num, $num1);
            $num = $num1;
            $num1 = $tmp;

            $tmp = bcsub($x, bcmul($y, $q));
            $x = $y;
            $y = $tmp;
        } 
        while (bccomp($num1, '0'));
        
        if (bccomp($x, '0') < 0) 
        {
            $x = bcadd($x, $mod);
        }
        
        return $x;
    }

    /**
     * Воозвращает битовую длину числа $num
     *
     * @param string $num
     * 
     * @return int
     */
    public function bitLen($num)
    {
        $tmp = $this->int2bin($num);
        $bit_len = strlen($tmp) * 8;
        $tmp = ord($tmp{strlen($tmp) - 1});
        
        if (!$tmp) 
        {
            $bit_len -= 8;
        }
        else 
        {
            while (!($tmp & 0x80)) 
            {
                $bit_len--;
                $tmp <<= 1;
            }
        }
        
        return $bit_len;
    }

    /**
     * Вычисляет побитовую разницу между $num1 и $num2, начиная с $start_pos позиции числа $num1
     *
     * @param string $num1
     * @param string $num2
     * @param int $start_pos
     * 
     * @return string
     */
    public function bitOr($num1, $num2, $start_pos)
    {
        $start_byte = intval($start_pos / 8);
        $start_bit = $start_pos % 8;
        $tmp1 = $this->int2bin($num1);

        $num2 = bcmul($num2, 1 << $start_bit);
        $tmp2 = $this->int2bin($num2);
        
        if ($start_byte < strlen($tmp1)) 
        {
            $tmp2 |= substr($tmp1, $start_byte);
            $tmp1 = substr($tmp1, 0, $start_byte) . $tmp2;
        }
        else 
        {
            $tmp1 = str_pad($tmp1, $start_byte, "\0") . $tmp2;
        }
        
        return $this->bin2int($tmp1);
    }

    /**
     * Возвращает часть числа $num, начиная с бита $start длиной $length
     *
     * @param string $num
     * @param int start
     * @param int length
     * 
     * @return string
     */
    public function subint($num, $start, $length)
    {
        $start_byte = intval($start / 8);
        $start_bit = $start % 8;
        $byte_length = intval($length / 8);
        $bit_length = $length % 8;
        if ($bit_length) 
        {
            $byte_length++;
        }
        $num = bcdiv($num, 1 << $start_bit);
        $tmp = substr($this->int2bin($num), $start_byte, $byte_length);
        $tmp = str_pad($tmp, $byte_length, "\0");
        $tmp = substr_replace($tmp, $tmp{$byte_length - 1} & chr(0xff >> (8 - $bit_length)), $byte_length - 1, 1);
        
        return $this->bin2int($tmp);
    }
    
    /**
     * Вычисляет pow($num, $pow) (mod $mod)
     *
     * @param string $num
     * @param string $pow
     * @param string $mod
     * 
     * @return string
     */
    public function powmod($num, $pow, $mod)
    {
        return bcpowmod($num, $pow, $mod);
    }
}
?>