<?php
/**
 * Реализация класса, использующегося для кодирования и декодирования сообщений по алгоритму RSA.
 *
 * Данный класс использует при работе классы sbKey и sbMath.
 *
 * @see sbMath
 * @see SbKey
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Auth
 * @copyright Copyright (c) 2008, OOO "СИБИЭС Групп"
 */

/**
 * @ignore
 */
require_once SB_CMS_LIB_PATH.'/rsa/sbKey.inc.php';

/**
 * @ignore
 */
require_once SB_CMS_LIB_PATH.'/rsa/sbMath.inc.php';

/**
 * Класс, использующийся для кодирования и декодирования сообщений по алгоритму RSA.
 *
 * Данный класс использует при работе классы sbKey и sbMath.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Auth
 * @copyright Copyright (c) 2008, OOO "СИБИЭС Групп"
 */
class sbRSA
{
    /**
     * Ссылка на объект класса, работающего с большими целыми числами.
     *
     * @var object sbMath
     */
    private $_math_obj;
    
    /**
     * Открытый ключ, который используется методом encrypt()
     *
     * @var object Объект класса sbKey.
     */
    private $_public_key;

    /**
     * Закрытый ключ, который используется методом decrypt()
     *
     * @var object Объект класса sbKey.
     */
    private $_private_key;

    /**
     * Конструктор класса.
     *
     * @param object $public_key Открытый ключ (объект класса sbKey).
     * @param object $private_key Закрытый ключ (объект класса sbKey).
     * @param bool $error Содержит TRUE, если в результате создания класса возникли ошибки.
     * 
     * @return sbRSA
     */
    public function __construct($public_key = null, $private_key = null, &$error)
    {
        $error = true;
        
        $this->_math_obj = new sbMath($error);
        
        if ($error)
            return;
            
        $this->_public_key = $public_key;
        $this->_private_key = $private_key;
        
        $error = false;
    }

    /**
     * Кодирует $plain_data с помощью ключа $this->_public_key или $key.
     *
     * @param string $plain_data Текст, который следует закодировать.
     * @param object $key Ключ для кодирования (объект класса sbKey).
     * 
     * @return mixed Кодированный текст или FALSE при ошибке.
     */
    public function encrypt($plain_data, $key = null)
    {
        if (is_null($key)) 
        {
            $key = $this->_public_key;
        }
        
        if (!sbKey::isValid($key)) 
        {
            return false;
        }
        
        if ($key->getKeyType() != 'public') 
        {
            return false;
        }

        $enc_data = $this->_encryptBinary($plain_data, $key);
        if ($enc_data !== false) 
        {
            return base64_encode($enc_data);
        }

        return false;
    }

    /**
     * Кодирует $plain_data с помощью ключа $key.
     *
     * @param string $plain_data Текст, который следует закодировать.
     * @param object $key Ключ для кодирования (объект класса sbKey).
     * 
     * @return mixed Закодированный текст как бинарную строку или FALSE в случае ошибки.
     */
    private function _encryptBinary($plain_data, $key)
    {
        $plain_data .= "\x01";

        $plain_data = $this->_math_obj->bin2int($plain_data);
        $exp = $this->_math_obj->bin2int($key->getExponent());
        $modulus = $this->_math_obj->bin2int($key->getModulus());

        $data_len = $this->_math_obj->bitLen($plain_data);
        $chunk_len = $key->getKeyLength() - 1;
        $block_len = (int) ceil($chunk_len / 8);
        $curr_pos = 0;
        $enc_data = '';
        while ($curr_pos < $data_len) 
        {
            $tmp = $this->_math_obj->subint($plain_data, $curr_pos, $chunk_len);
            $enc_data .= str_pad(
                $this->_math_obj->int2bin($this->_math_obj->powmod($tmp, $exp, $modulus)),
                $block_len,
                "\0"
            );
            $curr_pos += $chunk_len;
        }
        return $enc_data;
    }
    
    /**
     * Декодирует $plain_data с помощью ключа $this->_private_key или $key.
     *
     * @param string $plain_data Текст, который следует декодировать.
     * @param object $key Ключ для декодирования (объект класса sbKey).
     * 
     * @return mixed Декодированный текст или FALSE при ошибке.
     */
    public function decrypt($enc_data, $key = null)
    {
        if (is_null($key)) 
        {
            $key = $this->_private_key;
        }
        
        if (!sbKey::isValid($key)) 
        {
            return false;
        }
        
        if ($key->getKeyType() != 'private') 
        {
            return false;
        }
        
        $enc_data = base64_decode($enc_data);
        return $this->_decryptBinary($enc_data, $key);
    }
    
    /**
     * Декодирует $plain_data с помощью ключа $key.
     *
     * @param string $plain_data Текст, который следует декодировать.
     * @param object $key Ключ для декодирования (объект класса sbKey).
     * 
     * @return mixed Декодированный текст как бинарную строку или FALSE в случае ошибки.
     */
    private function _decryptBinary($enc_data, $key)
    {
        $exp = $this->_math_obj->bin2int($key->getExponent());
        $modulus = $this->_math_obj->bin2int($key->getModulus());

        $data_len = strlen($enc_data);
        $chunk_len = $key->getKeyLength() - 1;
        $block_len = (int) ceil($chunk_len / 8);
        $curr_pos = 0;
        $bit_pos = 0;
        $plain_data = $this->_math_obj->bin2int("\0");
        while ($curr_pos < $data_len) 
        {
            $tmp = $this->_math_obj->bin2int(substr($enc_data, $curr_pos, $block_len));
            $tmp = $this->_math_obj->powmod($tmp, $exp, $modulus);
            $plain_data = $this->_math_obj->bitOr($plain_data, $tmp, $bit_pos);
            $bit_pos += $chunk_len;
            $curr_pos += $block_len;
        }
        $result = $this->_math_obj->int2bin($plain_data);

        $tail = ord($result{strlen($result) - 1});
        if ($tail != 1) 
        {
            return false;
        }

        return substr($result, 0, -1);
    }
    
    /**
     * Устанавливает открытый ключ
     *
     * @param object $key Открытый ключ (объект класса sbKey).
     */
    public function setPublicKey($key)
    {
        $this->_public_key = $key;
    }
    
    /**
     * Устанавливает закрытый ключ
     *
     * @param object $key Закрытый ключ (объект класса sbKey).
     */
    public function setPrivateKey($key)
    {
        $this->_private_key = $key;
    }
}
?>