<?php
/**
 * Кеширование SQL-запросов в БД типа key->value
 *
 * @author Serg
 */ 
class sbCacheSql
{
    static private $_Instance = null;
    static private $_noCached = array(
        'sb_system_log', 
        'sb_catchanges', 
        'sb_users_sessions',
        'sb_users', //Часто происходит изменение таблицы (время последней активности) - не имеет смысла кешировать
        'sb_cache_settings', 
        'sb_cron'
        );

    static public function getInstance($type)
    {
        if(self::$_Instance === null)
        {
            $config = require_once SB_CMS_KERNEL_PATH . '/config_sql_cache.inc.php';
            if(!isset($config['use_cache']) || $config['use_cache'] == '' || !isset($config[$config['use_cache']]))
            {
                self::$_Instance = new NoCacheSql();
                return self::$_Instance;
            }
            
            $className = $config['use_cache'];
            if(class_exists($className))
            {
                self::$_Instance = new $className($type, $config[$className]);
                self::$_Instance->setNoCached(self::$_noCached);
            }
            else
            {
                self::$_Instance = new NoCacheSql();
            }
        } 
        return self::$_Instance;
    }
}

interface ICacheSql
{
    public function getResult($query, $assoc);
    public function replaceCache($query, $data, $total, $assoc);
    public function addToCache($query, $data, $total, $replaceData, $assoc);
    public function clearCache($query, $assoc);
    public function setNoCached($tables);
}

/**
 * Кеширование результатов запросов с помощью memcache
 */
class sbMemcacheSql implements ICacheSql
{
    /**
     * Экземпляр класса Memcache
     * @var Memcache 
     */
    private $_memcache = null;
    
    /**
     * Ссылка на соединение с сервером Memcache
     * @var type 
     */
    private $_connection = null;
    
    /**
     * Массив таблиц, которые не кешируются
     * @var array
     */
    private $_noCached = array();
    
    /**
     * Тип адаптера базы данных. Необходим для парсинга текста запроса
     * @var string
     */
    private $_type = null;
    
    public function __construct($type, $config)
    {
        if(class_exists('Memcache'))
        {
            $this->_memcache = new Memcache();
            $this->_connection = $this->_memcache->pconnect($config['host'], $config['port']);
            $this->_type = $type;
        }
    }
    
    /**
     * Создает в кеше объект с тегами, который нужен для решения проблемы "состояния гонки"
     * @param array $data
     * @return void
     */
    public function setTags($data=array())
    {
        if(!$this->_connection)
        {
            return;
        }
        $this->_memcache->set('tags_'.SB_COOKIE_DOMAIN, $data);
    }
    
    /**
     * Устанавливает новые значения тегов для определения валидности кеша
     * 
     * @param string $query
     * @return void
     */
    public function clearCache($query, $assoc)
    {
        if(!$this->_connection)
        {
            return;
        }
        
        $table = array();
        $query = preg_replace('/\s{2,}/', ' ', $query);

        //определяем таблицы (теги) запроса
        switch ($this->_type)
        {
            case 'mysql':
                preg_match('/(?<=insert into|update|delete|truncate table)\s.*?(sb_\w+)\s*.*$/i', $query, $table);
                break;
        }
        if(isset($table[1]) && trim($table[1]) != '' && !in_array(trim($table[1]), $this->_noCached))
        {
            $tags = $this->getTags();
            $tags[$table[1]] = time();
            $this->setTags($tags);
        }
    }
    
    /**
     * Возвращает из кеша результат запроса, если он там есть и является валидным.
     * Если результат невалидный, то возвращается хэш запроса
     * если результата нет, то возвращается FALSE
     * 
     * @param string $query
     * @param boolean $assoc
     * @return boolean|string|array
     */
    public function getResult($query, $assoc)
    { 
        if(!$this->_connection)
        {
            return false;
        }
        
        $tablesInQuery = $this->getTables($query);
        
        if(empty($tablesInQuery))
        {
            return false;
        }
        
        $key = 'sql_'.SB_COOKIE_DOMAIN.'_'.md5($query.intval($assoc));
        $res = $this->_memcache->get($key);
        if(!$res)
        {
            return false;
        }
        
        //Проверяем валидность кеша
        $tags = $this->_memcache->get('tags_'.SB_COOKIE_DOMAIN);

        if(empty($tags))
        {
            //Если нет данных по тегам, то что-то произошло и мы удаляем записи из кеша,
            //т.к. они априори считаются невалидными
            $slabs = $this->_memcache->getExtendedStats('slabs');

            foreach ($slabs as $slab)
            {
                foreach ($slab as $slab_id => $slab_meta)
                {
                    $slab_id = intval($slab_id);
                    if ($slab_id != 0)
                    {
                        $cdump = $this->_memcache->getExtendedStats('cachedump', $slab_id);
                        foreach ($cdump as $val)
                        {
                            if (!is_array($val))
                                continue;

                            foreach ($val as $k => $v)
                            {
                                if(sb_stripos($k, 'sql_'.SB_COOKIE_DOMAIN) === 0)
                                {
                                    $this->_memcache->delete($k);
                                }
                            }
                        }
                    }
                }
            }
            return false;
        }
        
        foreach($tags as $tag=>$value)
        {
            if(isset($res[$tag]) && $res[$tag] != $value)
            {
                //Изменилось значение ключевого тега - кеш считается невалидным
                //Возвращаем ключ элемента кэша, как признак того, что необходимо
                //обновить кэш, а не создавать новый
                return $key;
            }
        }
        
        
        //Удаляем теги из результата
        foreach($tablesInQuery as $tag)
        {
            if(array_key_exists($tag, $res))
            {
                unset($res[$tag]);
            }
        }
        return $res;
    }
    
    
    /**
     * Добавляет результат запроса в кеш
     * 
     * @param string $query         Текст запроса
     * @param array $data           Результат запроса
     * @param int $total            Общее количество записей
     * @param boolean $replaceData  Обновлять существующий объект в кеше (true) или создавать новый (false)
     * @param boolean $assoc
     * @return boolean
     */
    public function addToCache($query, $data, $total, $replaceData=false, $assoc)
    {
        if(!$this->_connection)
        {
            return;
        }
        
        $tablesInQuery = $this->getTables($query);
        if(empty($tablesInQuery))
        {
            return false;
        }
        
        $key = 'sql_'.SB_COOKIE_DOMAIN.'_'.md5($query.intval($assoc));
        $tags = $this->_memcache->get('tags_'.SB_COOKIE_DOMAIN);
        $needUpdate = false;
        
        //Ставим блокировку на 10 секунд. В случае, если скрипт отвалится, блокировка "умрет" сама
        if(!$this->_memcache->add('lock_'.$key, '', MEMCACHE_COMPRESSED, 10))
        {
            //Если не удалось поставить блокировку, значит кто-то уже кеширует этот запрос. Не будем мешать.
            return false;
        }
        
        $data['total'] = $total ? $total : -1;
        foreach($tablesInQuery as $tag)
        {
            if(!isset($tags[$tag]))
            {
                $tags[$tag] = time();
                $needUpdate = true;
            }
            $data[$tag] = $tags[$tag];
        }
        
        if(!$replaceData)
        {
            $result = $this->_memcache->add($key, $data, MEMCACHE_COMPRESSED);
        }
        else
        {
            $result = $this->_memcache->replace($key, $data, MEMCACHE_COMPRESSED);
        }
        if($result && $needUpdate)
        {
            $this->_memcache->replace('tags_'.SB_COOKIE_DOMAIN, $tags);
        }
        //Снимаем блокировку
        $this->_memcache->delete('lock_'.$key);
    }
    
    /**
     * Обертка над addToCache для обновления существующего объекта в кеше
     * 
     * @param string $query         Текст запроса
     * @param array $data           Результат запроса
     * @param int $total            Общее количество записей
     * @param boolean $assoc
     */
    public function replaceCache($query, $data, $total, $assoc)
    {
        $this->addToCache($query, $data, $total, true, $assoc);
    }
    
    /**
     * Устанавливает таблицы, которые не кешируются
     * 
     * @param array $tables     Перечень таблиц
     */
    public function setNoCached($tables)
    {
        if(!is_array($tables))
        {
            $tables = array($tables);
        }
        $this->_noCached = $tables;
    }
    
    /**
     * Определяет, какие таблицы используются в запросе
     * 
     * @param string $query      Текст запроса
     * @return array             Список таблиц
     */
    private function getTables(&$query)
    {
        $query = preg_replace('/\s{2,}/', ' ', $query);
        
        $tables = array();
        switch ($this->_type)
        {
            case 'mysql':
                $tmp = array();
                preg_match('/from(.+?)(where|$)/i', $query, $tmp);
                if(isset($tmp[1]))
                {
                    preg_match_all('/(\s|^)sb_\w+(\s|$)/', trim($tmp[1]), $tables);
                }
                break;
        }
        
        $tablesInQuery = array();
        if(!empty($tables))
        {
            foreach($tables[0] as $item)
            {
                $tablesInQuery[] = trim($item);
            }
        }
        $tablesInQuery = array_unique($tablesInQuery);
        $tablesInQuery = array_diff($tablesInQuery, $this->_noCached);
        return $tablesInQuery;
    }
    
    
    /**
     * Возвращает из кеша объект с тегами
     * 
     * @return array|boolean
     */
    private function getTags()
    {
        if (!$this->_connection)
        {
            return false;
        }
        
        $tags = $this->_memcache->get('tags_'.SB_COOKIE_DOMAIN);
        return $tags;
    }
}

/**
 * Класс-заглушка, который подключается если не используется кеширование
 */
class NoCacheSql implements ICacheSql
{
    private $_connection = null;
    
    public function __construct()
    {
    }
    
    public function getResult($query, $assoc)
    {
        return false;
    }
    
    public function replaceCache($query, $data, $total, $assoc)
    {
        return false;
    }
    
    public function addToCache($query, $data, $total, $replaceData, $assoc)
    {
        return false;
    }
    
    public function clearCache($query, $assoc)
    {
        return false;
    }
    
    public function setNoCached($tables)
    {
        
    }
}

?>
