<?php
/**
 * Реализация класса, осуществляющего разбиение результатов SQL-запроса по страницам
 *
 * Данный класс использует библиотеку работы с СУБД mySQL.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_DB
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

/**
 * Класс, осуществляющий разбиение результатов SQL-запроса по страницам
 *
 * Пример использования класса:
 * <code>
 * // в эту переменную запишется общее кол-во найденных записей
 * $total = true;
 * // вызываем конструктор класса, передавая ему в качестве параметров уникальный идентификатор (для формирования имени GET-параметра),
 * // максимальное кол-во номеров страниц в списке, максимальное кол-во записей на одной странице, target-атрибут, устанавливаемый для всех
 * // сформированных ссылок
 * $pager = new sbDBPager('sl', 7, 50, 'content');
 * // переменная $result получает результат работы SQL-запроса
 * $result = $pager->init($total, 'SELECT sl_date, sl_user_id, sl_user_login, sl_user_ip, sl_message, sl_type FROM sb_system_log '.$order.' WHERE sl_domain=?', SB_COOKIE_DOMAIN);
 * // переменная $num_list получает HTML-код для вывода номеров страниц
 * $num_list = $pager->show();
 * </code>
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_DB
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbDBPager
{
    /**
     * Префикс, прибавляемый к GET-параметру текущей страницы
     *
     * Например, если указан префикс 'news', то текущую страницу можно получить через переменную
     * $_GET['page_news'].
     *
     * @var string
     */
    private $mPrefix = '';

    /**
     * Максимальное кол-во номеров страниц в списке
     *
     * @var int
     */
    private $mNumPages = 7;

    /**
     * Кол-во записей на странице
     *
     * @var int
     */
    private $mNumElems = 20;

    /**
     * Общее кол-во записей
     *
     * Определяется в методе init, либо задается пользователем.
     *
     * @var int
     */
    public $mNumElemsAll = 0;

    /**
     * Начиная с какой записи делать выборку
     *
     * @var int
     */
    public $mFrom = 0;

    /**
     * До какой записи делать выборку
     *
     * @var int
     */
    public $mTo = 0;

    /**
     * Номер текущей страницы
     *
     * @var int
     */
    public $mPage = 1;

    /**
     * Атрибут target у ссылок
     *
     * @var string
     */
    private $mTarget = '';

    /**
     * Макет дизайна для вывода ссылки "Начало"
     *
     * В макете дизайна доступны следующие псевдо-теги:
     * - {HREF} - заменяется на ссылку
     * - {TARGET} - заменяется на target, указанный в конструкторе класса
     * - {CUR_PAGE} - номер текущей страницы
     * - {ALL_PAGES} - общее кол-во страниц
     * - {ALL_ELEMS} - общее кол-во записей
     *
     * @var string
     */
    public $mBeginTemp;

    /**
     * Макет дизайна для вывода неактивной ссылки "Начало"
     *
     * В макете дизайна доступны следующие псевдо-теги:
     * - {HREF} - заменяется на ссылку
     * - {TARGET} - заменяется на target, указанный в конструкторе класса
     * - {CUR_PAGE} - номер текущей страницы
     * - {ALL_PAGES} - общее кол-во страниц
     * - {ALL_ELEMS} - общее кол-во записей
     *
     * @var string
     */
    public $mBeginTempDisabled;

    /**
     * Макет дизайна для вывода ссылки "Предыдущие"
     *
     * В макете дизайна доступны следующие псевдо-теги:
     * - {HREF} - заменяется на ссылку
     * - {TARGET} - заменяется на target, указанный в конструкторе класса
     * - {CUR_PAGE} - номер текущей страницы
     * - {ALL_PAGES} - общее кол-во страниц
     * - {ALL_ELEMS} - общее кол-во записей
     *
     * @var string
     */
    public $mPrevTemp;

    /**
     * Макет дизайна для вывода неактивной ссылки "Предыдущие"
     *
     * В макете дизайна доступны следующие псевдо-теги:
     * - {HREF} - заменяется на ссылку
     * - {TARGET} - заменяется на target, указанный в конструкторе класса
     * - {CUR_PAGE} - номер текущей страницы
     * - {ALL_PAGES} - общее кол-во страниц
     * - {ALL_ELEMS} - общее кол-во записей
     *
     * @var string
     */
    public $mPrevTempDisabled;

    /**
     * Макет дизайна для вывода ссылки на номер страницы
     *
     * В макете дизайна доступны следующие псевдо-теги:
     * - {HREF} - заменяется на ссылку
     * - {TARGET} - заменяется на target, указанный в конструкторе класса
     * - {NUMBER} - номер страницы
     * - {CUR_PAGE} - номер текущей страницы
     * - {ALL_PAGES} - общее кол-во страниц
     * - {ALL_ELEMS} - общее кол-во записей
     *
     * @var string
     */
    public $mNumberTemp = '&nbsp;<a href="{HREF}" target="{TARGET}">{NUMBER}</a>&nbsp;';

    /**
     * Макет дизайна для вывода ссылки на текущий номер страницы
     *
     * В макете дизайна доступны следующие псевдо-теги:
     * - {HREF} - заменяется на ссылку
     * - {TARGET} - заменяется на target, указанный в конструкторе класса
     * - {NUMBER} - номер страницы
     * - {ALL_PAGES} - общее кол-во страниц
     * - {ALL_ELEMS} - общее кол-во записей
     *
     * @var string
     */
    public $mCurNumberTemp = '&nbsp;<b>{NUMBER}</b>&nbsp;';

    /**
     * Макет дизайна для вывода ссылки "Следующие"
     *
     * В макете дизайна доступны следующие псевдо-теги:
     * - {HREF} - заменяется на ссылку
     * - {TARGET} - заменяется на target, указанный в конструкторе класса
     * - {CUR_PAGE} - номер текущей страницы
     * - {ALL_PAGES} - общее кол-во страниц
     * - {ALL_ELEMS} - общее кол-во записей
     *
     * @var string
     */
    public $mNextTemp;

    /**
     * Макет дизайна для вывода неактивной ссылки "Следующие"
     *
     * В макете дизайна доступны следующие псевдо-теги:
     * - {CUR_PAGE} - номер текущей страницы
     * - {ALL_PAGES} - общее кол-во страниц
     * - {ALL_ELEMS} - общее кол-во записей
     *
     * @var string
     */
    public $mNextTempDisabled;

    /**
     * Макет дизайна для вывода ссылки "Конец"
     *
     * В макете дизайна доступны следующие псевдо-теги:
     * - {HREF} - заменяется на ссылку
     * - {TARGET} - заменяется на target, указанный в конструкторе класса
     * - {CUR_PAGE} - номер текущей страницы
     * - {ALL_PAGES} - общее кол-во страниц
     * - {ALL_ELEMS} - общее кол-во записей
     *
     * @var string
     */
    public $mEndTemp;

    /**
     * Макет дизайна для вывода неактивной ссылки "Конец"
     *
     * В макете дизайна доступны следующие псевдо-теги:
     * - {CUR_PAGE} - номер текущей страницы
     * - {ALL_PAGES} - общее кол-во страниц
     * - {ALL_ELEMS} - общее кол-во записей
     *
     * @var string
     */
    public $mEndTempDisabled;

    /**
     * Массив со списком значений кол-ва элементов на странице
     *
     * @var array
     */
    public $mElemsPerPageArray = array(15, 30, 50, 100, 300);

    /**
     * Макет дизайна активной ссылки выбора кол-ва элементов на странице
     *
     * @var string
     */
    public $mElementPerPageTempActive = '&nbsp;<a href="{HREF}">{NUMBER}</a>&nbsp;';

    /**
     * Макет дизайна неактивной ссылки выбора кол-ва элементов на странице (текущее значение)
     *
     * @var string
     */
    public $mElementPerPageTempCurrent = '&nbsp;<b>{NUMBER}</b>&nbsp;';

    /**
     * Текущее значение количества элементов на странице
     *
     * @var int
     */
    public $CurrentElementsPerPage;

    /**
     * Макет дизайна для вывода разделителя номеров страниц.
     *
     * @var string
     */
    public $mDelimTemp = '...';

    /**
     * Макет дизайна для вывода списка ссылок
     *
     * В макете дизайна доступны следующие псевдо-теги:
     * - {BEGIN} - ссылка "Начало"
     * - {PREV} - ссылка "Предыдущие
     * - {LIST} - список номеров страниц
     * - {NEXT} - ссылка "Следующие"
     * - {END} - ссылка "Конец"
     * - {CUR_PAGE} - номер текущей страницы
     * - {ALL_PAGES} - общее кол-во страниц
     * - {ALL_ELEMS} - общее кол-во записей
     * - {ELEMS_PER_PAGE} - выбор количества элементов на странице
     *
     * @var string
     */
    public $mListTemp = '{BEGIN} {PREV} {LIST} {NEXT} {END}';

    /**
     * Конструктор класса
     *
     * В конструкторе класса определяется номер текущей страницы, если таковой был передан через GET-параметр.
     *
     * @param string $prefix Уникальный идентификатор (используется для формирования имени GET-параметра, например, если передано
     *                       значение 'news', то GET-параметр, через который передается номер текущей страницы, получит имя 'page_news').
     * @param int $num_pages Максимальное кол-во номеров страниц в списке.
     * @param int $num_elems Максимальное кол-во записей на одной странице.
     * @param int $num_cookie_name Имя cookie для установки кол-ва элементов на странице.
     *
     * @param string $target target-атрибут, устанавливаемый для всех сформированных ссылок.
     */
	function __construct($prefix, $num_pages=7, $num_elems=20, $target='', $num_cookie_name = '')
	{
        $this->mPrefix = $prefix;
        $this->mNumPages = $num_pages;

        if(isset($_REQUEST['num_'.$prefix]) && $num_cookie_name != '')
        {
        	if ((int)$_REQUEST['num_'.$prefix] > 0)
        	{
        		sb_setcookie($num_cookie_name, intval($_REQUEST['num_'.$prefix]));
        		$this->mNumElems = intval($_REQUEST['num_'.$prefix]);
        	}
        	else
        	{
        		sb_setcookie($num_cookie_name, '');
        		$this->mNumElems = $num_elems;
        	}
        }
        elseif(isset($_COOKIE[$num_cookie_name]) && (int)$_COOKIE[$num_cookie_name] > 0 && $num_cookie_name != '')
        {
        	$this->mNumElems = intval($_COOKIE[$num_cookie_name]);
        }
        else
        {
        	$this->mNumElems = $num_elems;
        }

        $this->mTarget = $target;

        $this->mPage = (isset($_GET['page_'.$prefix]) && $_GET['page_'.$prefix] != '' && $_GET['page_'.$prefix] != 0 ? max(1, intval($_GET['page_'.$prefix])) : 1);

        $this->mBeginTemp = '<a href="{HREF}" target="{TARGET}"><img src="'.SB_CMS_IMG_URL.'/begin.gif" width="14" height="15" border="0" alt="'.KERNEL_PAGER_BEGIN.'" align="absmiddle"></a>';
        $this->mBeginTempDisabled = '<img src="'.SB_CMS_IMG_URL.'/begin_gray.gif" width="14" height="15" border="0" alt="'.KERNEL_PAGER_BEGIN.'" align="absmiddle">';

        $this->mPrevTemp = '<a href="{HREF}" target="{TARGET}"><img src="'.SB_CMS_IMG_URL.'/previous.gif" width="14" height="15" border="0" alt="'.KERNEL_PAGER_PREV.'" align="absmiddle"></a>';
        $this->mPrevTempDisabled = '<img src="'.SB_CMS_IMG_URL.'/previous_gray.gif" width="14" height="15" border="0" alt="'.KERNEL_PAGER_PREV.'" align="absmiddle">';

        $this->mNextTemp = '<a href="{HREF}" target="{TARGET}"><img src="'.SB_CMS_IMG_URL.'/next.gif" width="14" height="15" border="0" alt="'.KERNEL_PAGER_NEXT.'" align="absmiddle"></a>';
        $this->mNextTempDisabled = '<img src="'.SB_CMS_IMG_URL.'/next_gray.gif" width="14" height="15" border="0" alt="'.KERNEL_PAGER_NEXT.'" align="absmiddle">';

        $this->mEndTemp = '<a href="{HREF}" target="{TARGET}"><img src="'.SB_CMS_IMG_URL.'/end.gif" width="14" height="15" border="0" alt="'.KERNEL_PAGER_END.'" align="absmiddle"></a>';
        $this->mEndTempDisabled = '<img src="'.SB_CMS_IMG_URL.'/end_gray.gif" width="14" height="15" border="0" alt="'.KERNEL_PAGER_END.'" align="absmiddle">';
	}

	/**
	 * Запускает переданный SQL-запрос
	 *
	 * В SQL-запросе не надо использовать структуру LIMIT. Параметру $total присваивается общее кол-во найденных записей.
	 *
	 * @see sbMySql::paramQueryPage()
     * @see sbMySqli::paramQueryPage()
     *
     * @param int $total Получает общее кол-во найденных записей.
     * @param string $query SQL-запрос.
     * @param mixed $arg1,... Значения макросов в запросе.
     * @param string $message Текст системного сообщения.
     * @param int $type Тип сообщения (SB_MSG_INFORMATION, SB_MSG_WARNING и т.д.).
     *
     * @return mixed Двумерный массив. FALSE в случае возникновения ошибки.
     */
    function init(&$total)
	{
	    if (!is_object($GLOBALS['sbSql']))
            return false;

        $args = func_get_args();
        if (!is_array($args) || trim($args[0]) == '')
        {
            return false;
        }

        $message = '';
        $type = SB_MSG_INFORMATION;

        // отбрасываем аргумент $total
        array_shift($args);

		$num = count($args) - 1;
		if ($num > 0)
		{
			/**
	         * Определяем кол-во макросов в запросе
	         */
	        $num_placeholders = $GLOBALS['sbSql']->placeholdersCount($args[0]);

	        /**
	         * Определяем, указано ли системное сообщение и его тип
	         */
	        $diff = $num - $num_placeholders;
			switch ($diff)
	        {
	            case 1:
	                $message = array_pop($args);
	                break;

	            case 2:
	                $type = array_pop($args);
	                $message = array_pop($args);
	                break;
	        }
		}

        if ($this->mTo != 0 && ($this->mNumElems > $this->mTo - $this->mFrom + 1 || $this->mNumElems == 0))
            $this->mNumElems = max(0, $this->mTo - $this->mFrom + 1);

        $from = ($this->mPage - 1) * $this->mNumElems + max(0, $this->mFrom - 1);
        $sql = $args[0];

        if ($this->mNumElems != 0)
        {
            $args[0] = $sql.' LIMIT '.$from.','.$this->mNumElems;
        }
        else
        {
        	$this->mNumElems = 2147483647;
            $args[0] = $sql.' LIMIT '.$from.', 9999999999';
        }

        $total = true;
        $result = $GLOBALS['sbSql']->paramQueryPage($total, $args);

		if ($this->mPage > ceil($total / $this->mNumElems))
        {
        	$this->mPage = ceil($total / $this->mNumElems);
        	$from = ($this->mPage - 1) * $this->mNumElems + max(0, $this->mFrom - 1);

	        if ($this->mNumElems != 0)
	        {
	            $args[0] = $sql.' LIMIT '.$from.','.$this->mNumElems;
	        }
	        else
	        {
	            $args[0] = $sql.' LIMIT '.$from.', 9999999999';
	        }

        	$total = true;
        	$result = $GLOBALS['sbSql']->paramQueryPage($total, $args);
        }

        if ($this->mFrom != 0)
        {
            $total = max(0, $total - $this->mFrom + 1);
        }

        if ($this->mTo != 0)
        {
            $total = min($total, $this->mTo - $this->mFrom + 1);
        }

        /**
         * Если запрос выполнен и указано системное сообщение, записываем системное сообщение в журнал
         */
        if ($result)
        {
            $this->mNumElemsAll = $total;

            if ($message != '')
                sb_add_system_message($message, $type);
        }

        $this->mFrom = $from;
        return $result;
	}

	/**
	 * Возвращает HTML-код для номеров страниц
	 *
	 * Для настройки данного вывода используйте соотв. переменные класса.
	 *
	 * @return strign HTML-код для номеров страниц
	 */
	function show()
	{
		if ($this->mNumElemsAll <= $this->mNumElems || $this->mNumElemsAll == 0 || $this->mNumElems == 0)
			return '';

		if (is_null($this->mDelimTemp))
		{
			$this->mDelimTemp = '...';
		}

		$get_str = ($_SERVER['QUERY_STRING'] != '' ? '?'.$_SERVER['QUERY_STRING'].'&' : '?');

		// добавляем вычищенные системой GET-параметры
		foreach ($_GET as $key => $value)
	    {
	    	if ($key == 'event')
	    	{
	    		$get_str .= 'event='.$value.'&';
	    	}
			elseif (!preg_match('/^p_f_(.*)/i', $key) && (preg_match('/^[A-Z0-9_\-]+_cid$/i', $key) || preg_match('/^[A-Z0-9_\-]+_temp_id$/i', $key) || preg_match('/^[A-Z0-9_\-]+_id$/i', $key) || preg_match('/^[A-Z0-9_\-]+_tag$/i', $key)))
	    	{
	    		$get_str .= $key.'='.intval($value).'&';
	    	}
			elseif(sbPlugins::getSetting('sb_static_urls') != 1 &&
	    	   ($key == 'pl_forum_cat' || $key == 'pl_forum_cat_sel' || $key == 'pl_forum_sub'))
	    	{
	    		$get_str .= $key.'='.intval($value).'&';
			}
			elseif((preg_match('/page_[0-9]+/', $key) || preg_match('/page_t_[0-9]+/', $key)) && (preg_match('/c_[0-9]+/', $this->mPrefix)))
			{
				$get_str .= $key.'='.intval($value).'&';
			}
			elseif((preg_match('/page_[0-9]+/', $key) || preg_match('/page_c_[0-9]+/', $key)) && (preg_match('/t_[0-9]+/', $this->mPrefix)))
			{
				$get_str .= $key.'='.intval($value).'&';
			}
		}

		$index_file = trim(sbPlugins::getSetting('sb_directory_index'), ' /\\');
		if ($index_file == '')
		{
		    $index_file = 'index.php';
		}

		$href = str_replace($index_file, '', (isset($GLOBALS['PHP_SELF']) ? $GLOBALS['PHP_SELF'] : $_SERVER['PHP_SELF']).$get_str);

		$all_pages = ceil($this->mNumElemsAll / $this->mNumElems);
		if ($this->mPage > $all_pages)
		{
			$this->mPage = $all_pages;
		}

        $num_list = '';

        if ($all_pages <= $this->mNumPages)
	    {
	        for ($i = 1; $i <= $all_pages; $i++)
	        {
	            if ($i == $this->mPage)
	            {
	               $num_list .= str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{ALL_PAGES}', '{ALL_ELEMS}'),
	                                           array($href.'page_'.$this->mPrefix.'='.$i, $this->mTarget, $i, $all_pages, $this->mNumElemsAll),
	                                           $this->mCurNumberTemp);
	            }
	            else
	            {
	               $num_list .= str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
	                                           array($href.'page_'.$this->mPrefix.'='.$i, $this->mTarget, $i, $this->mPage, $all_pages, $this->mNumElemsAll),
	                                           $this->mNumberTemp);
	            }
	        }
		}
		else
		{
		    if ($this->mPage < $this->mNumPages - 1)
		    {
                for ($i = 1; $i < $this->mNumPages; $i++)
                {
                   if ($i == $this->mPage)
                   {
                       $num_list .= str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                                   array($href.'page_'.$this->mPrefix.'='.$i, $this->mTarget, $i, $all_pages, $this->mNumElemsAll),
                                                   $this->mCurNumberTemp);
                   }
                   else
                   {
                       $num_list .= str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                                   array($href.'page_'.$this->mPrefix.'='.$i, $this->mTarget, $i, $this->mPage, $all_pages, $this->mNumElemsAll),
                                                   $this->mNumberTemp);
                   }
                }

                if ($all_pages == $this->mPage)
                {
                   $num_list .= $this->mDelimTemp.str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                                     array($href.'page_'.$this->mPrefix.'='.$this->mPage, $this->mTarget, $this->mPage, $all_pages, $this->mNumElemsAll),
                                                     $this->mCurNumberTemp);
                }
                else
                {
                   $num_list .= $this->mDelimTemp.str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                                     array($href.'page_'.$this->mPrefix.'='.$all_pages, $this->mTarget, $all_pages, $this->mPage, $all_pages, $this->mNumElemsAll),
                                                     $this->mNumberTemp);
                }
		    }
		    elseif ($this->mPage > $all_pages - $this->mNumPages + 2)
		    {
                if (1 == $this->mPage)
                {
                   $num_list .= str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                               array($href.'page_'.$this->mPrefix.'='.$this->mPage, $this->mTarget, $this->mPage, $all_pages, $this->mNumElemsAll),
                                               $this->mCurNumberTemp).$this->mDelimTemp;
                }
                else
                {
                   $num_list .= str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                               array($href.'page_'.$this->mPrefix.'=1', $this->mTarget, 1, $this->mPage, $all_pages, $this->mNumElemsAll),
                                               $this->mNumberTemp).$this->mDelimTemp;
                }

                for ($i = $all_pages - $this->mNumPages + 2; $i <= $all_pages; $i++)
                {
                   if ($i == $this->mPage)
                   {
                       $num_list .= str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                                   array($href.'page_'.$this->mPrefix.'='.$i, $this->mTarget, $i, $all_pages, $this->mNumElemsAll),
                                                   $this->mCurNumberTemp);
                   }
                   else
                   {
                       $num_list .= str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                                   array($href.'page_'.$this->mPrefix.'='.$i, $this->mTarget, $i, $this->mPage, $all_pages, $this->mNumElemsAll),
                                                   $this->mNumberTemp);
                   }
                }
		    }
            else
	        {
                if (1 == $this->mPage)
                {
                   $num_list .= str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                               array($href.'page_'.$this->mPrefix.'='.$this->mPage, $this->mTarget, $this->mPage, $all_pages, $this->mNumElemsAll),
                                               $this->mCurNumberTemp).$this->mDelimTemp;
                }
                else
                {
                   $num_list .= str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                               array($href.'page_'.$this->mPrefix.'=1', $this->mTarget, 1, $this->mPage, $all_pages, $this->mNumElemsAll),
                                               $this->mNumberTemp).$this->mDelimTemp;
                }

                for ($i = $this->mPage - $this->mNumPages + 4; $i <= $this->mPage + 1; $i++)
                {
                   if ($i == $this->mPage)
                   {
                       $num_list .= str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                                   array($href.'page_'.$this->mPrefix.'='.$i, $this->mTarget, $i, $all_pages, $this->mNumElemsAll),
                                                   $this->mCurNumberTemp);
                   }
                   else
                   {
                       $num_list .= str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                                   array($href.'page_'.$this->mPrefix.'='.$i, $this->mTarget, $i, $this->mPage, $all_pages, $this->mNumElemsAll),
                                                   $this->mNumberTemp);
                   }
                }
                if ($all_pages == $this->mPage)
                {
                   $num_list .= $this->mDelimTemp.str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                                     array($href.'page_'.$this->mPrefix.'='.$this->mPage, $this->mTarget, $this->mPage, $all_pages, $this->mNumElemsAll),
                                                     $this->mCurNumberTemp);
                }
                else
                {
                   $num_list .= $this->mDelimTemp.str_replace(array('{HREF}', '{TARGET}', '{NUMBER}', '{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                                     array($href.'page_'.$this->mPrefix.'='.$all_pages, $this->mTarget, $all_pages, $this->mPage, $all_pages, $this->mNumElemsAll),
                                                     $this->mNumberTemp);
                }
		    }
		}

        if ($this->mPage != 1)
	    {
	        $prev = str_replace(array('{HREF}', '{TARGET}', '{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                   array($href.'page_'.$this->mPrefix.'='.($this->mPage - 1), $this->mTarget, $this->mPage, $all_pages, $this->mNumElemsAll),
                                   $this->mPrevTemp);
            $begin = str_replace(array('{HREF}', '{TARGET}', '{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                    array($href.'page_'.$this->mPrefix.'=1', $this->mTarget, $this->mPage, $all_pages, $this->mNumElemsAll),
                                    $this->mBeginTemp);
	    }
	    else
	    {
	        $prev = str_replace(array('{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                   array($this->mPage, $all_pages, $this->mNumElemsAll),
                                   $this->mPrevTempDisabled);

            $begin = str_replace(array('{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                    array($this->mPage, $all_pages, $this->mNumElemsAll),
                                    $this->mBeginTempDisabled);
	    }

	    if ($this->mPage != $all_pages)
	    {
	        $next = str_replace(array('{HREF}', '{TARGET}', '{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                   array($href.'page_'.$this->mPrefix.'='.($this->mPage + 1), $this->mTarget, $this->mPage, $all_pages, $this->mNumElemsAll),
                                   $this->mNextTemp);
            $end = str_replace(array('{HREF}', '{TARGET}', '{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                    array($href.'page_'.$this->mPrefix.'='.$all_pages, $this->mTarget, $this->mPage, $all_pages, $this->mNumElemsAll),
                                    $this->mEndTemp);
	    }
	    else
	    {
	        $next = str_replace(array('{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                   array($this->mPage, $all_pages, $this->mNumElemsAll),
                                   $this->mNextTempDisabled);
            $end = str_replace(array('{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
                                    array($this->mPage, $all_pages, $this->mNumElemsAll),
                                    $this->mEndTempDisabled);
	    }


	    $result = str_replace(array('{BEGIN}', '{PREV}', '{LIST}', '{NEXT}', '{END}', '{CUR_PAGE}', '{ALL_PAGES}', '{ALL_ELEMS}'),
	                             array($begin, $prev, $num_list, $next, $end, $this->mPage, $all_pages, $this->mNumElemsAll),
	                             $this->mListTemp);
		return $result;
	}

    /**
     * Возвращает HTML-код для выбора количества элементов на странице
     *
     * @return string
     */
    public function showElemsPerPageList()
    {
        $elemsPerPageTemp = '';

        foreach($this->mElemsPerPageArray as $value)
        {
            if($this->CurrentElementsPerPage == $value)
            {
                $elemsPerPageTemp .= str_replace('{NUMBER}', $value, $this->mElementPerPageTempCurrent);
            }
            else
            {
                $elemsPerPageTemp .= str_replace(array('{HREF}', '{NUMBER}'), array('#', $value), $this->mElementPerPageTempActive);
            }
        }

        return $elemsPerPageTemp;
    }
}
?>