<?php
/**
 * Реализация классов sbSocket и sbDownload, использующихся для получения содержимого страницы или заголовков сервера
 * по URL страницы.
 *
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Sockets
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

/**
 * Класс, использующийся для получения содержимого страницы или заголовков сервера
 * по URL страницы
 *
 * Данный класс осуществляет работу через сокеты, соотв. расширение PHP должно быть доступно на сервере.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Sockets
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbSocket
{
	/**
     * URL страницы, содержимое которой мы хотим получить
     *
     * Устанавливается в конструкторе класса.
     *
     * @var string
     */
    public $mUrl = '';

    /**
     * Хост страницы
     *
     * Получаем после распарсивания URL страницы.
     *
     * @var string
     */
    private $mHost = '';

    /**
     * Протокол
     *
     * Получаем после распарсивания URL страницы.
     *
     * @var string
     */
    private $mProtocol = 'http';

    /**
     * Порт
     *
     * Получаем после распарсивания URL страницы.
     *
     * @var string
     */
    private $mPort = '80';

    /**
     * URI
     *
     * Получаем после распарсивания URL страницы.
     *
     * @var string
     */
    private $mUri = '';

    /**
     * Хранит URL страницы, на которую был редирект
     *
     * Если в процессе загрузки был редирект, то подгружается страница, на которую был редирект, а
     * в эту переменную класса записывается URL реально подгруженной страницы.
     *
     * @var string
     */
    public $mRedirectUrl = '';

    /**
     * Хранит заголовки, полученные при загрузке
     *
     * @var string
     */
    public $mHeader = '';

    /**
     * Хранит текст, полученный при загрузке
     *
     * @var string
     */
    public $mBody = '';

    /**
     * Строка, добавляемая к URL страницы.
     *
     * @var string
     */
    public $mQueryStr = '';

	/**
	 * Дескриптор сокета
	 *
	 * @var resource $fp
	 */
	public $mFp = null;

	/**
	 * Блокированый режим соединения.
	 *
	 * @var boolean $blocking
	 */
    public $mBlocking = true;

    /**
     * Постоянное соединение
     *
     * @var boolean $persistent
     */
    public $mPersistent = false;

    /**
     * IP-адрес для соединения
     *
     * @var string $addr
     */
    public $mAddr = '';

    /**
     * Таймаут ожидания соединения
     *
     * @var integer $timeout
     */
	public $mTimeout = null;

    /**
     * Конструктор класса
     *
     * @param string $url URL страницы, содержимое которой мы хотим получить
     */
    public function __construct($url)
    {
        $this->mRedirectUrl = '';
        $this->mUrl = $url;

        $this->scan_url();
    }

    /**
     * Используется для определения протокола, хоста, порта и пр. по URL страницы
     *
     */
    protected function scan_url()
    {
        $url = @parse_url($this->mUrl);
	    if (!$url)
	    {
	       	return;
	    }

	    if (!isset($url['scheme']))
	        $url['scheme'] = 'http';
	    if (!isset($url['host']))
	        $url['host'] = SB_COOKIE_DOMAIN;
	    if (!isset($url['path']))
	       	$url['path'] = '/';
	    if (!isset($url['port']))
	        $url['port'] = ($url['scheme'] == 'https') ? 443 : ($url['scheme'] == 'ftp' ? 21 : 80);

        $this->mProtocol = $url['scheme'];
        $this->mHost = $url['host'];
        $this->mPort = $url['port'];
        $this->mUri = $url['path'].(isset($url['query']) ? '?'.$url['query'] : '');
    }

    /**
     * Выделяет из ответа сервера заголовки и содержимое
     *
     * @param string $response Ответ сервера.
     */
	protected function set_header_body($response)
	{
        $this->mHeader = '';
        $this->mBody = '';

        $regex = '/(.*?)\r*\n[\r\n]*?\n+\r*(.*)/s';
        $m = array();

        if (!preg_match($regex, $response, $m))
        {
            $this->mBody = $response;
        }
        else
        {
            $this->mHeader = $m[1];
            $this->mBody = $m[2];

            while (sb_strpos($this->mBody, 'HTTP/') === 0 && preg_match($regex, $this->mBody, $m))
            {
                $this->mHeader = $m[1];
                $this->mBody = $m[2];
            }
        }
    }

    /**
     * Получает содержимое указанной страницы
     *
     * @return string Содержимое страницы. FALSE, если страницу открыть не удалось
     */
    public function download($redir_count = 0)
    {
    	static $crlf = "\r\n";

    	if ($redir_count > 5)
    		return false;

    	if (trim($this->mUri) == '')
    		return false;

		//	открываем сокет
		$socket = $this->connect(($this->mProtocol == 'https' ? 'ssl://' : '').$this->mHost, $this->mPort);

		if ($socket === false)
        {
			//	если произошла ошибка
			return false;
        }

        // собираем запрос
        @fwrite($socket, 'GET '.$this->mUri.' HTTP/1.0'.$crlf.'Host: '.$this->mHost.$crlf.$crlf);
        @stream_set_timeout($socket, 15);

        // считываем содержимое страницы
        $response = '';
        while(!@feof($socket))
        {
            $response .= @fread($socket, 4096);
        }
        @fclose($socket);

        // разделяем заголовок сервера и тело страницы
        $this->set_header_body($response);

        // распарсиваем заголовок сервера
        $headers = array();
        $lines = explode($crlf, $this->mHeader);

        foreach($lines as $line)
        {
            if(($pos = strpos($line, ':')) !== false)
            {
                $headers[strtolower(trim(substr($line, 0, $pos)))] = trim(substr($line, $pos + 1));
            }
        }

        // была переадресация?
        if(isset($headers['location']) && $headers['location'] != $this->mUri)
        {
            $http = new sbSocket($headers['location']);
            $result = $http->download($redir_count + 1);

            if ($http->mRedirectUrl != '')
            {
                $this->mRedirectUrl = $http->mRedirectUrl;
            }
            else
            {
                $this->mRedirectUrl = $headers['location'];
            }

            $this->mHeader = $http->mHeader;
            $this->mBody = $http->mBody;
        }

        return $this->mBody;
    }

    /**
     * Устанавливает адрес для скачивания
     *
     * @param string $url URL для скачивания.
     */
    public function setURL($url)
    {
    	$this->mUrl = $url;
    	$this->mUri = '';
    	$this->mRedirectUrl = '';

        $this->scan_url();
    }

	/**
	 * Соединяется с сокетом. Если соединение уже открыто то закрывает и снова открывает соединение.
	 *
	 * @param string  $addr			IP адерс или домен.
	 * @param int $port 			Порт
	 * @param bool $persistent		Постоянное соединение (TRUE) или нет (FALSE)
	 * @param int $timeout			Таймаут ожидания соединения
	 * @param array $options		Опции для stream_context_create.
	 * @return mixed
	 */
	public function connect($addr, $port = 0, $persistent = null, $timeout = null, $options = null)
    {
		if(is_resource($this->mFp))
		{
			@fclose($this->mFp);
			$this->mFp = null;
		}

		if(!$addr)
		{
			return false;
		}
        elseif (strspn($addr, '.0123456789') == strlen($addr) || strstr($addr, '/') !== false)
        {
			$this->mAddr = $addr;
        }
        else
        {
			$this->mAddr = @gethostbyname($addr);
        }

		$this->mPort = $port % 65536;

        if ($persistent !== null)
        {
            $this->mPersistent = $persistent;
        }

		if ($timeout !== null)
		{
			$this->mTimeout = $timeout;
		}

		$openfunc = $this->mPersistent ? 'pfsockopen' : 'fsockopen';

        $errno = 0;
        $errstr = '';
		if ($options && function_exists('stream_context_create'))
		{
			if ($this->mTimeout)
            {
				$timeout = $this->mTimeout;
            }
			else
            {
				$timeout = 0;
			}
			$context = stream_context_create($options);

            if (function_exists('stream_socket_client'))
            {
				$flags = $this->mPersistent ? STREAM_CLIENT_PERSISTENT : STREAM_CLIENT_CONNECT;
				$addr = $this->mAddr.':'.$this->mPort;
				$fp = stream_socket_client($addr, $errno, $errstr, $timeout, $flags, $context);
            }
			else
			{
				$fp = @$openfunc($this->mAddr, $this->mPort, $errno, $errstr, $timeout, $context);
			}
        }
		else
        {
			if ($this->mTimeout)
            {
				$fp = @$openfunc($this->mAddr, $this->mPort, $errno, $errstr, $this->mTimeout);
            }
            else
            {
				$fp = @$openfunc($this->mAddr, $this->mPort, $errno, $errstr);
			}
        }

        if (!$fp)
        {
			sb_add_system_message($errstr, SB_MSG_WARNING);
			return false;
		}

		$this->mFp = $fp;

		$this->setBlocking($this->mBlocking);
		return $this->mFp;
    }

    /**
     * Закрывает соединение
     *
     * @return unknown
     */
	public function disconnect()
	{
		if (!is_resource($this->mFp))
        {
			return false;
		}

		@fclose($this->mFp);
		$this->mFp = null;
		return true;
	}

	/**
	 * Блокирует или разблокироваывает соединение
	 *
	 * @param int $mode 1 Блокирует 0 разблокировывает соединение
	 * @return boolean Удачно (TRUE) не удачно (FALSE)
	 */
    public function setBlocking($mode)
    {
        if (!is_resource($this->mFp))
        {
			return false;
        }

		$this->mBlocking = $mode;
		socket_set_blocking($this->mFp, $this->mBlocking);
		return true;
	}

 	/**
     * Отправляет GET или POST запрос на сервер
     *
     * @param string $method - тип запроса POST или GET
     * @param array $getdata GET данные
     * @param array $postdata POST данные
     * @param boolean $return_data возвращать (true) или не возвращать (false) ответ сервера
     * @param array $custom_headers дополнительные заголовки
     *
     * @return boolean Удачно (TRUE) не удачно (FALSE)
     */
    public function request($method='GET', $getdata = array(), $postdata = array(), $return_data = true, $custom_headers = array())
	{
		$method = strtoupper($method);
		if ($method != 'GET' && $method != 'POST')
			return false;

		$ret = '';
	    $getdata_str = count($getdata) ? '?' : '';
	    $postdata_str = '';

	    foreach ($getdata as $k => $v)
	    {
	        $getdata_str .= $k .'='. $v;
	    }

	    foreach ($postdata as $k => $v)
	    {
	        $postdata_str .= $k .'='. $v .'&';
	    }

	    $crlf = "\r\n";
	    $req = $method .' '. $this -> mUri . $getdata_str .' HTTP/1.1' . $crlf;
	    $req .= 'Host: '. $this->mHost . $crlf;
	    $req .= 'User-Agent: Mozilla/5.0 Firefox/3.6.12' . $crlf;
	    $req .= 'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8' . $crlf;
	    $req .= 'Accept-Language: en-us,en;q=0.5' . $crlf;
	    $req .= 'Accept-Encoding: deflate' . $crlf;
	    $req .= 'Accept-Charset: utf-8;q=0.7,*;q=0.7' . $crlf;
	    foreach ($custom_headers as $v)
        	$req .= $v. $crlf;

	    if ($method == 'POST' && !empty($postdata_str))
	    {
	        $postdata_str = substr($postdata_str, 0, -1);
	        $req .= 'Content-Type: application/x-www-form-urlencoded' . $crlf;
	        $req .= 'Content-Length: '. strlen($postdata_str) . $crlf . $crlf;
	        $req .= $postdata_str;
	    }
	    else
	    {
	    	$req .= $crlf;
	    }

	    $this->connect(($this->mProtocol == 'https' ? 'ssl://' : '').$this->mHost, $this->mPort);

	    if(!$this->mFp)
		{
			return false;
		}

		@fputs($this->mFp, $req);
	    if($return_data)
	    {
			while (($line = @fgets($this->mFp)) != false)
	    	{
	    		$ret .= $line;
	    	}
	    	$ret = substr($ret, strpos($ret, "\r\n\r\n") + 4);
	    	@fclose($this->mFp);
	    	return $ret;
	    }
	    else
	    {
	    	return true;
	    }
	}
}

/**
 * Класс, использующийся для получения содержимого страницы или заголовков сервера
 * по URL страницы
 *
 * Данный класс пытается получить страницу через расширение PHP cURL. Если это расширение не подключено, то класс пытается
 * использовать сокеты, если же и сокеты не доступны, то работа ведется через fopen.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Sockets
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbDownload extends sbSocket
{
    /**
     * Конструктор класса
     *
     * @param string $url URL страницы, содержимое которой мы хотим получить
     */
    public function __construct($url)
    {
        $this->mRedirectUrl = '';
        $this->mUrl = $url;
    }

	private function curl_redir_exec(&$ch)
	{
	    static $curl_loops = 0;

	    if ($curl_loops >= 5)
	    {
	    	$curl_loops = 0;
	        return false;
	    }

	    $response = @curl_exec($ch);
        if ($response === false)
        {
            $curl_loops = 0;
        	return false;
        }

	    // разделяем заголовок сервера и тело страницы
        $this->set_header_body($response);

        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
	    if ($http_code == 301 || $http_code == 302)
	    {
	    	$matches = array();
	        preg_match('/Location:(.*?)\n/', $this->mHeader, $matches);
	    	$url = @parse_url(trim(array_pop($matches)));
	        if (!$url)
	    	{
	        	$curl_loops = 0;
	        	return $this->mBody;
	        }

	    	$last_url = parse_url(curl_getinfo($ch, CURLINFO_EFFECTIVE_URL));
	    	if (!isset($url['scheme']))
	        	$url['scheme'] = isset($last_url['scheme']) ? $last_url['scheme'] : 'http';
	    	if (!isset($url['host']))
	        	$url['host'] = isset($last_url['host']) ? $last_url['host'] : SB_COOKIE_DOMAIN;
	    	if (!isset($url['path']))
	        	$url['path'] = isset($last_url['path']) ? $last_url['path'] : '/';
	        if (!isset($url['port']))
	        	$url['port'] = isset($last_url['port']) ? $last_url['port'] : 80;

		$path = explode('/', $url['path']);
		foreach ($path as $key => $value)
		{
			$path[$key] = urlencode(urldecode($value));
		}
		$url['path'] = implode('/', $path);

	    	$new_url = $url['scheme'].'://'.$url['host'].($url['port'] != 80 ? ':'.$url['port'] : '').$url['path'].(isset($url['query']) ? '?'.$url['query'] : '');

	    	if (trim($this->mQueryStr) != '')
	    	{
	    		$new_url = $new_url.(strpos($new_url, '?') !== false ? '&' : '?').$this->mQueryStr;
	    	}

	    	$this->mRedirectUrl = $new_url;
	    	curl_setopt($ch, CURLOPT_URL, $new_url);
	    	$curl_loops++;

	    	return $this->curl_redir_exec($ch);
	    }
	    else
	    {
			$curl_loops=0;
			return $this->mBody;
	    }
	}

    /**
     * Получает содержимое указанной страницы
     *
     * @return string Содержимое страницы. FALSE, если страницу открыть не удалось
     */
    public function download($redir_count = 0)
    {
    	if (trim($this->mUrl) == '')
    		return false;

    	if (trim($this->mQueryStr) != '')
    	{
    		$this->mUrl = $this->mUrl.(strpos($this->mUrl, '?') !== false ? '&' : '?').$this->mQueryStr;
    	}

        $result = false;

        if (function_exists('curl_init'))
        {
            //открываем файл через cURL
            $ch = @curl_init($this->mUrl);
            if ($ch !== false)
            {
            	@curl_setopt($ch, CURLOPT_HEADER, true);
	    		@curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	    		@curl_setopt($ch, CURLOPT_TIMEOUT, 60);
	    		@curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
	    		@curl_setopt($ch, CURLOPT_USERAGENT, 'S.Builder Bot');
	    		@curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
				@curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);

                /* !!! НЕ работает при редиректе в nginx, решил всегда использовать кастомный редирект
                if (ini_get('safe_mode') != 1 && strtoupper(ini_get('safe_mode')) != 'ON' && ini_get('open_basedir') == '')
                {
                	@curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
                	@curl_setopt($ch, CURLOPT_MAXREDIRS, 5);
                	$result = @curl_exec($ch);

                	if ($result !== false)
                	{
                        // разделяем заголовок сервера и тело страницы
			        	$this->set_header_body($result);

                        $effective_url = curl_getinfo($ch, CURLINFO_EFFECTIVE_URL);
                        if ($effective_url != $this->mUrl)
                        {
                            $this->mRedirectUrl = $effective_url;
                        }

			        	$result = $this->mBody;

                        $lines = explode("\r\n", $this->mHeader);
                	}
                }
                else
                {*/
                	$result = $this->curl_redir_exec($ch);
                //}

                @curl_close($ch);
            }
        }
        else
        {
	    	// если через cURL открыть не удалось, то пытаемся открыть через сокеты
	        $this->scan_url();
	        $result = parent::download();

	        if ($result === false)
	        {
	            $result = @file_get_contents($this->mUrl);
	            $this->mHeader = '';
	            $this->mBody = $result;
	        }
        }

        return $result;
    }
    
    /**
     * Загружает содержимое удаленного файла в указанную директорию
     * 
     * @param $localPath Папка в которую необходимо сохранить файл
     * @return Mixed Имя файла если файл загружен успешно, false если нет
     */
    public function copyRemoteFile($localPath)
    {
        $pUrl = parse_url($this->mUrl);
        $pUrl = explode('/', $pUrl['path']);
        $name = $pUrl[count($pUrl)-1];
        
        $name = explode('.', $name);
        if(!isset($name[1])) //файл приходит без расширения
        {
            $type = 'bin';
            
            //если пришло изображение, то пытаемся определить его тип
            $info = getimagesize($this->mUrl);
            if(isset($info['mime']))
            {
                switch ($info['mime'])
                {
                    case 'image/jpeg':
                        $type = 'jpg';
                        break;
                    case 'image/png':
                        $type = 'jpg';
                        break;
                    default:
                        $type = 'gif';
                        break;
                }
            }
            
            $name[1] = $type;
        }

        $name[count($name)-2] .= '_'.uniqid(); //в динамически формируемых файлах имя не уникально
        $name = implode('.', $name);
        
        $localPath = $localPath.'/'.$name;
        
        $download = $this->download();

        $mLocal = $GLOBALS['sbVfs']->mLocal;
        $GLOBALS['sbVfs']->mLocal = true;
        $GLOBALS['sbVfs']->fopen(SB_BASEDIR.$localPath, 'w');
        $GLOBALS['sbVfs']->fwrite($download);
        
        $GLOBALS['sbVfs']->mLocal = $mLocal;
        
        return $name;
    }
}
?>