<?php
/**
 * Реализация класса, отвечающего за создание или восстановления дампа базы данных
 *
 * @author Сергей Болотаев <sergey@binn.ru>
 * @version 4.0
 * @package SB_DB
 * @copyright Copyright (c) 2008, OOO "СИБИЭС Групп"
 */

define('SB_DUMPER_DUMP_SQL','sql');
define('SB_DUMPER_DUMP_XML','xml');
define('SB_DUMPER_DUMP_CSV','csv');
define('SB_DUMPER_DUMP_TABLE','table');
define('SB_DUMPER_DUMP_FULL','full');

/**
 * Класс, отвечающий за создание или восстановления дампа базы данных
 *
 * @author Сергей Болотаев <sergey@binn.ru>
 * @version 4.0
 * @package SB_DB
 * @copyright Copyright (c) 2008, OOO "СИБИЭС Групп"
 */
class sbDumper
{
	/**
    * Указывает, доступна ли библиотека zlib на сервере
    *
    * TRUE - библиотека доступна,  FALSE - библиотека не доступна.
    *
    * @var bool
    */
	public $zlibAvailable = false;

	/**
	 * Указывает, была ли ошибка во время упаковки дампа
	 *
	 * TRUE - ошибка была, FALSE - ошибки не было.
	 *
	 * @var bool
	 */
	public $packError = false;

	/**
    * Указывает, была ли ошибка во время снятия или записи дампа
    *
    * TRUE - ошибка была, текст ошибки записывается в переменную $errorStr,
    * FALSE - ошибки не было.
    *
    * @var bool
    */
	public $errorFlag = false;

	/**
    * В случае возникновения ошибки содержит текст ошибки
    *
    * @var string
    */
	public $errorStr = '';

	/**
	* Активирует перезагрузку дампера, работает, если в этом нуждается сам дампер
	*
	* @var bool
	*/
	private $reloadDumper = false;

	/**
	* Время окончания работы скрипта
	*
	* @var integer
	*/
	private $timeEnd = 0;

	/**
	* Если нужно выполнять без ограничений времени.
	*
	* @var bool
	*/
	public $unlimitTime = false;

	/**
	* Кол-во записей таблицы БД, которое считывается за одно обращение к методу getResultPage()
	*
	* @var integer
	*/
	private $limit = 100;

	/**
	 * Позиция в файле, с которой необходимо продолжить разбор SQL-инструкций при перезагрузке дампера
	 *
	 * @var int
	 */
	private $position = 0;

   /**
	* Создавать или нет дамп отдельных столбцов
	*
	* TRUE - создается дамп отдельных столбцов, FALSE - в таблице дампятся все столбцы.
	*
	* @var boolean
	*/
	private $createColumnDump = false;

	/**
	* Перечень столбцов с которых нужно снять дамп
	*
	* @var string
	*/
	private $columns = '';

	public $replaceInfo = array();

    private $files = array();

	/**
    * Конструктор класса
    *
    * В конструкторе класса происходит проверка на доступность библиотеки zlib на сервере,
    * а также перезагрузка дампера.
    *
    * @param int $limit Кол-во записей таблицы БД, которое считывается за одно обращение к методу getResultPage().
    * @param boolean $reload Если TRUE, то происходит перезагрузка дампера.
    */
    public function __construct($limit = 100, $reload = false)
    {
		$this->reloadDumper = $reload;

		//	Устанавливаем время завершения работы скрипта, если нужно.
		if(!$this->unlimitTime)
		{
			$this->timeEnd = @ini_get('max_execution_time');
	        if ($this->timeEnd > 0)
	        {
				$this->timeEnd = time() + $this->timeEnd - 5;
	        }
	        else
	        {
	            $this->timeEnd = time() + 30;
	        }
		}

        // Устанавливается лимит результатов
        $this->limit = max(1, $limit);

        // Проверяется доступность библиотеки Zlib на сервере
        $this->zlibAvailable = (@extension_loaded('zlib') && (version_compare('5.2.0', phpversion()) == -1 || version_compare('5.2.0', phpversion()) == 0));

        // Перезагрузка Dumper'a
        if ($this->isNeedReload())
        {
         	if(isset($_SESSION['PL_DUMPER_RELOAD_ACTION']) && $_SESSION['PL_DUMPER_RELOAD_ACTION'] == 'SQL_IMPORT_RELOAD')
         	{
         	    // Перезагрузка дампера для загрузки дампа из файла
                if (isset($_SESSION['PL_DUMPER_SQLIMPORT_POSITION']) && isset($_SESSION['PL_DUMPER_SQLIMPORT_FILEPATH']))
                {
                    $filePath = $_SESSION['PL_DUMPER_SQLIMPORT_FILEPATH'];
                    $this->position = intval($_SESSION['PL_DUMPER_SQLIMPORT_POSITION']);

                    if(isset($_SESSION['PL_DUMPER_SQLIMPORT_REPLACEINFO']))
                    {
                    	$this->replaceInfo = $_SESSION['PL_DUMPER_SQLIMPORT_REPLACEINFO'];
                    	unset($_SESSION['PL_DUMPER_SQLIMPORT_REPLACEINFO']);
                    }
                    unset($_SESSION['PL_DUMPER_SQLIMPORT_POSITION']);
                    unset($_SESSION['PL_DUMPER_SQLIMPORT_FILEPATH']);
                    unset($_SESSION['PL_DUMPER_RELOAD_ACTION']);

                    $this->loadDumpFile($filePath);
                }
                else
                {
                    $this->reloadDumper = false;
                    unset($_SESSION['PL_DUMPER_RELOAD_ACTION']);
                }
         	}
         	elseif (isset($_SESSION['PL_DUMPER_RELOAD_ACTION']) && $_SESSION['PL_DUMPER_RELOAD_ACTION'] == 'EXPORT_RELOAD')
         	{
         	    // Перезагрузка дампера для создания дампа базы данных
         		if(isset($_SESSION['PL_DUMPER_EXPORT_LIMIT']) && isset($_SESSION['PL_DUMPER_EXPORT_FILE']) &&
         		   isset($_SESSION['PL_DUMPER_EXPORT_PACK']) && isset($_SESSION['PL_DUMPER_EXPORT_PACKFILE']) &&
         		   isset($_SESSION['PL_DUMPER_EXPORT_METHOD']) && isset($_SESSION['PL_DUMPER_EXPORT_TABLE']) &&
         		   isset($_SESSION['PL_DUMPER_EXPORT_FORMAT']) && isset($_SESSION['PL_DUMPER_RELOAD_ACTION']) &&
         		   isset($_SESSION['PL_DUMPER_EXPORT_CREATECOLUMNDUMP']) && isset($_SESSION['PL_DUMPER_EXPORT_COLUMNS']) &&
         		   isset($_SESSION['PL_DUMPER_EXPORT_RELOADBEFOREPACK']) && isset($_SESSION['PL_DUMPER_ORDER_STR']))
         		{
                    $this->limit        = $_SESSION['PL_DUMPER_EXPORT_LIMIT'];    // Устанавливаем старый лимит результатов
                    $fileName           = $_SESSION['PL_DUMPER_EXPORT_FILE'];     // Имя файла куда записывается дамп
                    $pack               = $_SESSION['PL_DUMPER_EXPORT_PACK'];     // Упаковывать или нет дамп
                    $packFileName       = $_SESSION['PL_DUMPER_EXPORT_PACKFILE']; // Имя упакованного дампа
                    $dumpMode           = $_SESSION['PL_DUMPER_EXPORT_METHOD'];   // Что дампим? Всю базу или таблицу
                    $tableName          = $_SESSION['PL_DUMPER_EXPORT_TABLE'];    // Таблица с которой надо продолжать
                    $orderStr           = $_SESSION['PL_DUMPER_ORDER_STR'];       // ORDER BY выражение
                    $dumpFormat         = $_SESSION['PL_DUMPER_EXPORT_FORMAT'];   // Формат дампа SQL CSV XML
                    $createColumnDump   = $_SESSION['PL_DUMPER_EXPORT_CREATECOLUMNDUMP'];
	                $columns            = $_SESSION['PL_DUMPER_EXPORT_COLUMNS'];
	                $reloadBeforePack   = $_SESSION['PL_DUMPER_EXPORT_RELOADBEFOREPACK'];

	                unset($_SESSION['PL_DUMPER_RELOAD_ACTION']);

                    $this->createDump($dumpMode, $dumpFormat, $fileName, $pack, $packFileName, $tableName, $createColumnDump, $columns, $reloadBeforePack, $orderStr);   // Собственно дампим
         		}
         		else
         		{
         			$this->reloadDumper = false;
         		}
         	}
         	elseif(isset($_SESSION['PL_DUMPER_RELOAD_ACTION']) && $_SESSION['PL_DUMPER_RELOAD_ACTION'] == 'EXPORT_PACK')
         	{
         	    // Перезагрузка дампера для упаковки дампа
                $fileName = $_SESSION['PL_DUMPER_EXPORT_FILE'];          // Имя файла куда записывается дамп
                $packFileName = $_SESSION['PL_DUMPER_EXPORT_PACKFILE'];  // Имя упакованного дампа

                unset($_SESSION['PL_DUMPER_EXPORT_FILE']);
                unset($_SESSION['PL_DUMPER_EXPORT_PACKFILE']);
                unset($_SESSION['PL_DUMPER_RELOAD_ACTION']);

                $this->reloadDumper = false;
                if ($this->packFile($packFileName, $fileName))
                {
                    if ($fileName != '' && $GLOBALS['sbVfs']->is_file($fileName))
                    {
                        $GLOBALS['sbVfs']->delete($fileName);
                    }
                }
         	}
         	else
         	{
         		$this->reloadDumper = false;
         	}
        }
    }

   /**
    * Проверяет, нуждается ли дампер в перезагрузке
    *
    * @return boolean TRUE - дампер требует перезагрузки, FALSE - дампер не нуждается в перезагрузке.
    */
	public function isNeedReload()
    {
        return $this->reloadDumper;
    }

   /**
    * Осуществляет упаковку дампа в zip-архив.
    *
    * @param string $packFileName Имя файла архива, который будет создан.
    * @param strign $fileName Имя файла с дампом, который будет запакован.
    *
    * @return boolean TRUE, если упаковка прошла успешно, FALSE в ином случае.
    */
	private function packFile($packFileName, $fileName)
    {
        // запрещаем выход на более верхний уровень
        $packFileName = preg_replace('/[^0-9a-zA-Z_\-\.\[\]\/]+/', '', $packFileName);
        $packFileName = preg_replace('/\.\.+/', '', $packFileName);
        $packFileName = trim(str_replace('\\', '/', $packFileName), '/');

        if ($packFileName == '')
        {
        	$this->errorStr = sprintf(SB_DUMPER_ERROR_ZIP_OPEN, '/');
            $this->errorFlag = true;
            $this->packError = true;

            return false;
        }

        $fileName = preg_replace('/[^0-9a-zA-Z_\-\.\[\]\/]+/', '', $fileName);
        $fileName = preg_replace('/\.\.+/', '', $fileName);
        $fileName = trim(str_replace('\\', '/', $fileName), '/');

        if ($fileName == '' || !$GLOBALS['sbVfs']->exists($fileName) || !$GLOBALS['sbVfs']->is_file($fileName))
        {
        	$this->errorStr = sprintf(SB_DUMPER_ERROR_OPEN_IN_FILE, '/'.$fileName);
			$this->errorFlag = true;
        	return false;
        }

        if (!$this->zlibAvailable)
        {
            $this->errorStr = SB_DUMPER_ERROR_ZLIB;
            $this->packError = true;
            $this->errorFlag = true;

            return false;
        }

        $mask = sbPlugins::getSetting('sb_file_rights');
        $mask = 777 - $mask;
        $oldmask = umask(octdec($mask));

        if (!sb_gzip(SB_BASEDIR.'/'.$fileName, SB_BASEDIR.'/'.$packFileName))
        {
        	$this->errorStr = sprintf(SB_DUMPER_ERROR_ZIP_ADD, '/'.$packFileName, '/'.$fileName);
            $this->errorFlag = true;
            $this->packError = true;

            umask($oldmask);
            return false;
        }

        umask($oldmask);
        return true;
    }

	/**
    * Возвращает массив записей таблицы БД, в массиве $limit записей
    *
    * Каждый такой массив считается страницей.
    *
    * @param string $tableName Имя таблицы, из которой возвращаются результаты.
    * @param int $pageId Номер страницы.
    * @param string $orderStr ORDER BY выражение, используемое при снятии дампа.
    *
    * @return array Массив записей таблицы БД.
    *
    */
	private function getResultPage($tableName, $pageId, $orderStr)
    {
        if($pageId < 1)
            $pageId = 1;

        $result = sql_query('SELECT * FROM ?#'.($orderStr != '' ? ' ORDER BY '.$orderStr : '').' LIMIT '.intval(($pageId - 1) * $this->limit).', '.intval($this->limit), $tableName);
        return $result;
    }

	 /**
     * Создает дамп таблицы из текущей установленной базы данных
     *
     * @param string $tableName Имя таблицы, котору надо дампить.
     * @param strign $dumpFormat Формат файла, в котором будет создан дамп: SB_DUMPER_DUMP_SQL, SB_DUMPER_DUMP_XML, SB_DUMPER_DUMP_CSV.
     * @param string $orderStr ORDER BY выражение, используемое при снятии дампа.
     *
     * @return boolean TRUE, если дамп удалось записать, FALSE в ином случае.
     */
    private function createTableDump($tableName, $dumpFormat, $orderStr = '')
	{
		$result_str = '';
		$result = array();

        // Вытаскиваем названия полей из таблицы
        $result = $GLOBALS['sbSql']->getFields($tableName);
        if (!$result)
        {
            return false;
        }

        $tmpcol = explode(',', $this->columns);

        for($i = 0; $i < count($tmpcol); $i++)
        {
        	$tmpcol[$i] = sb_strtolower(trim($tmpcol[$i]));
        }

        $columns = array();
        foreach ($result as $value)
        {
        	if ($this->createColumnDump)
        	{
        		if (array_search($value, $tmpcol) !== false)
        		{
        			$columns[] = $value;
        		}
        	}
        	else
        	{
        		$columns[] = $value;
        	}

        }

        $count = count($columns);

        if($count == 0)
        {
        	return false;
        }

        $col_str = '`'.$columns[0].'`';
        for($i = 1; $i < $count; $i++)
        {
            $col_str .= ' , `'.$columns[$i].'`';
        }

        // Вытаскиваем кол-во записей в таблице и формируем кол-во страниц
        $result = sql_query('SELECT COUNT(*) FROM ?#', $tableName);
        if (!$result)
            return false;

        list($results_count) = $result[0];

        $totalPages = ceil($results_count / $this->limit);

        // Если идет перезагрузка скрипта
        if($this->isNeedReload())
        {
            $page = intval($_SESSION['PL_DUMPER_EXPORT_PAGE']);  // Устанавливаем страницу с результатами на которой остановились
        }
        else
        {
			$page = 1;
		}

		$this->reloadDumper = false;
        switch($dumpFormat)
        {
            case SB_DUMPER_DUMP_SQL:
                if($page == 1)
                {
                    //	Вытаскиваем SQL код создания таблицы
					$result = sql_query('SHOW CREATE TABLE ?#', $tableName);
					if (!$result)
						return false;

					if ($tableName == 'sb_system_log' ||
                        $tableName == 'sb_users_sessions' ||
                        $tableName == 'sb_cache' ||
                        $tableName == 'sb_404_log' || 
						$tableName == 'sb_search_words' ||
						sb_strpos($tableName, 'sb_search_pages_') === 0)
                    {
						$result[0][1] = str_replace('CREATE TABLE', 'CREATE TABLE IF NOT EXISTS', $result[0][1]);
						$result_str .= $result[0][1].";\n\n\n";
                    }
					else
                    {
						$result_str = 'DROP TABLE IF EXISTS '.$tableName.";\n\n";
						$result_str .= $result[0][1].";\n\n\n";
                    }

                    if (!$GLOBALS['sbVfs']->fwrite($result_str))
                        return false;

                    if ($tableName == 'sb_system_log' ||
                        $tableName == 'sb_users_sessions' ||
                        $tableName == 'sb_cache' ||
                        $tableName == 'sb_404_log' || 
						$tableName == 'sb_search_words' ||
						sb_strpos($tableName, 'sb_search_pages_') === 0)
                    	return true;
                }

                $mb = 1024 * 1024;

				for($i = $page; $i <= $totalPages; $i++)
				{
					//	Есть ли у нас еще время?
					if(!$this->unlimitTime && time() > $this->timeEnd)
					{
                        $_SESSION['PL_DUMPER_EXPORT_TABLE'] = $tableName;        // Записываем текущую таблицу
                        $_SESSION['PL_DUMPER_EXPORT_PAGE'] = $i;                 // Записываем страницу с результатом
                        $_SESSION['PL_DUMPER_RELOAD_ACTION'] = 'EXPORT_RELOAD';  // Устанавливаем режим перезагрузки дампера

                        $this->reloadDumper = true; // Даем понять пользователю что дампер требует перезагрузки
                        return true;
                    }

                    $table_content = $this->getResultPage($tableName, $i, $orderStr);

                    if(is_array($table_content) && count($table_content) > 0)
                    {
                    	$result_str = 'INSERT INTO `'.$tableName.'` ('.$col_str.") VALUES\n";

                        // Заполняем таблицу через INSERT
                        foreach ($table_content as $value)
                        {
                            $query_str = '(';

                            for($j = 0; $j < $count; $j++)
                            {
                            	if ($j != 0)
                            		$query_str .= ' , ';

                                if($value[$j] !== '' && !is_null($value[$j]))
                                {
                                    $query_str .= $GLOBALS['sbSql']->escape($value[$j]);
                                }
                                elseif($value[$j] === '')
                                {
                                    $query_str .= '\'\'';
                                }
                                else
                                {
                                    $query_str .= 'NULL';
                                }
                            }

                            $query_str .= " ),\n";

                            if (sb_strlen($result_str) * 2 + sb_strlen($query_str) * 2 > $mb)
                            {
                            	$result_str = sb_substr($result_str, 0, -2).";\n\n";
                        		if (!$GLOBALS['sbVfs']->fwrite($result_str))
                            		return false;

                            	$result_str = 'INSERT INTO `'.$tableName.'` ('.$col_str.") VALUES\n".$query_str;
                            }
                            else
                            {
                            	$result_str .= $query_str;
                            }
                        }

                        $result_str = sb_substr($result_str, 0, -2).";\n\n";
                        if (!$GLOBALS['sbVfs']->fwrite($result_str))
                            return false;

                        unset($result_str);
                    }
                }
            	break;

            case SB_DUMPER_DUMP_XML:
                for($i = $page; $i <= $totalPages; $i++)
                {
                    // Есть ли у нас еще время?
                    if(!$this->unlimitTime && time() > $this->timeEnd)
                    {
                        $_SESSION['PL_DUMPER_EXPORT_TABLE'] = $tableName;        // Записываем текущую таблицу
                        $_SESSION['PL_DUMPER_EXPORT_PAGE'] = $i;                 // Записываем страницу с результатом
                        $_SESSION['PL_DUMPER_RELOAD_ACTION'] = 'EXPORT_RELOAD';  // Устанавливаем режим перезагрузки дампера

                        $this->reloadDumper = true; // Даем понять пользователю что дампер требует перезагрузки
                        return true;
                    }

                    $table_content = $this->getResultPage($tableName, $i, $orderStr);

                    if(is_array($table_content))
                    {
                    	 $result_str = '';

                        foreach ($table_content as $value)
                        {
                            $result_str .= "\t<".$tableName.">\n";
                            for ($j = 0; $j < $count; $j++)
                            {
                                if (is_null($value[$j]))
                                    $result_str .= "\t\t<".$columns[$j].'>NULL</'.$columns[$j].">\n";
                                else
                                    $result_str .= "\t\t<".$columns[$j].'>'.sb_htmlspecialchars($value[$j]).'</'.$columns[$j].">\n";
                            }

                            $result_str .= "\t</".$tableName.">\n\n";
                        }

                        if (!$GLOBALS['sbVfs']->fwrite($result_str))
                            return false;

                        unset($result_str);
                    }
                }
            	break;

            case SB_DUMPER_DUMP_CSV:
                for($i = $page; $i <= $totalPages; $i++)
                {
                    // Есть ли у нас еще время?
                    if(!$this->unlimitTime && time() > $this->timeEnd)
                    {
                        $_SESSION['PL_DUMPER_EXPORT_TABLE'] = $tableName;        // Записываем текущую таблицу
                        $_SESSION['PL_DUMPER_EXPORT_PAGE'] = $i;                 // Записываем страницу с результатом
                        $_SESSION['PL_DUMPER_RELOAD_ACTION'] = 'EXPORT_RELOAD';  // Устанавливаем режим перезагрузки дампера

                        $this->reloadDumper = true; // Даем понять пользователю что дампер требует перезагрузки
                        return true;
                    }

                    $table_content = $this->getResultPage($tableName, $i, $orderStr);

                    if(is_array($table_content))
                    {
                    	$result_str = '';

                        foreach ($table_content as $value)
                        {
                            for($j = 0; $j < $count; $j++)
                            {
                                if($value[$j] !== '' && !is_null($value[$j]))
                                {
                                    $result_str .= '"'.str_replace('"', '""""', $value[$j]).'";';
                                }
                                elseif($value[$j] === '')
                                {
                                    $result_str .= ';';
                                }
                                else
                                {
                                    $result_str .= 'NULL;';
                                }
                            }

                            $result_str = sb_substr($result_str, 0, -1)."\n";
                        }

                        if (!$GLOBALS['sbVfs']->fwrite($result_str))
                            return false;

                        unset($result_str);
                    }
                }
            	break;
        }

        return true;
    }

	 /**
     * Создает дамп всей текущей установленной базы данных
     *
     * @param string $dumpFormat Формат файла в котором будет создан дамп: SB_DUMPER_DUMP_SQL, SB_DUMPER_DUMP_XML, SB_DUMPER_DUMP_CSV.
     *
     * @return boolean TRUE, если дамп удалось записать, FALSE в ином случае.
     */
	private function createFullDump($dumpFormat)
    {
        $tables = $GLOBALS['sbSql']->showTables();

        $tables_count = count($tables);
        $startIndex = 0;

        // Если происходит перезагрузка дампера
        if($this->isNeedReload())
        {
            // То ищем таблицу на которой остановились
            for($i = 0; $i < $tables_count; $i++)
            {
                if($tables[$i] == $_SESSION['PL_DUMPER_EXPORT_TABLE'])
                {
                    $startIndex = $i;
                    break;
                }
            }

            unset($_SESSION['PL_DUMPER_EXPORT_TABLE']);
        }

        for($i = $startIndex; $i < $tables_count; $i++)
        {
            if (!$this->createTableDump($tables[$i], $dumpFormat))
            {
                return false;
            }

            if ($this->isNeedReload())
            {
                return true;  // Если требуется перезагрузка выходим
            }
        }

        return true;
    }

	 /**
     * Создает дамп всей базы или таблицы из текущей установленной базы данных
     *
     * @param string $dumpMode Режим снятия дампа (полный дамп всей базы данных или таблицы): SB_DUMPER_DUMP_FULL, SB_DUMPER_DUMP_TABLE.
     * @param string $dumpFormat Формат файла в котором будет создан дамп: SB_DUMPER_DUMP_SQL, SB_DUMPER_DUMP_XML, SB_DUMPER_DUMP_CSV.
     * @param string $fileName Имя и путь к файлу, в который будет сохранен дамп.
     * @param string $pack Упаковывать дамп или нет.
     * @param string $packFileName Имя и путь к файлу архива, в который будет упакован дамп.
     * @param string $tableName Имя таблицы, дамп которой надо снять, указывается только для режима SB_DUMPER_DUMP_TABLE.
     * @param string $createColumnDump Если переменная равна TRUE, то создается дамп отдельных столбцов, если FALSE, то в таблице дампятся все столбцы.
     * 								   Указываеться только для режима SB_DUMPER_DUMP_TABLE.
     * @param string $columns Перечень столбцов через запятую, с которых нужно снять дамп. Указывается только для режима SB_DUMPER_DUMP_TABLE при
     * 						  включенном режиме $createColumnDump.
     * @param string $reloadBeforePack Указывает, надо ли перезагружать дампер перед запаковкой файла с дампом.
     * @param string $orderStr ORDER BY выражение, используемое при снятии дампа. Указывается только для режима SB_DUMPER_DUMP_TABLE.
     *
     * @return boolean TRUE, если дамп удалось записать, FALSE в ином случае.
     */
	public function createDump($dumpMode, $dumpFormat, $fileName, $pack = false, $packFileName = 'dump.zip', $tableName = '', $createColumnDump = false, $columns = '', $reloadBeforePack = true, $orderStr = '')
    {
		$result_str = '';

        if($this->isNeedReload())
        {
            // Открываем файл для дозаписи
            if (!$GLOBALS['sbVfs']->fopen($fileName, 'a'))
            {
				$this->reloadDumper = false;
                $this->errorFlag = true;
                $this->errorStr = sprintf(SB_DUMPER_ERROR_OPEN_OUT_FILE, '/'.$fileName);

                return false;
            }
        }
        else
        {
            if (!$GLOBALS['sbVfs']->fopen($fileName, 'wb+'))
            {
				$this->errorFlag = true;
                $this->errorStr = sprintf(SB_DUMPER_ERROR_OPEN_OUT_FILE, '/'.$fileName);

                return false;
            }

            $_SESSION['PL_DUMPER_EXPORT_LIMIT'] = $this->limit;	     // ========
            $_SESSION['PL_DUMPER_EXPORT_FILE'] = $fileName;          //
            $_SESSION['PL_DUMPER_EXPORT_PACK'] = $pack;              //
            $_SESSION['PL_DUMPER_EXPORT_PACKFILE'] = $packFileName;  //            Сохраняем параметры функции
            $_SESSION['PL_DUMPER_EXPORT_METHOD'] = $dumpMode;        //
            $_SESSION['PL_DUMPER_EXPORT_TABLE'] = $tableName;		 //
            $_SESSION['PL_DUMPER_ORDER_STR'] = $orderStr;        //
            $_SESSION['PL_DUMPER_EXPORT_FORMAT'] = $dumpFormat;      // ========
    		$_SESSION['PL_DUMPER_EXPORT_CREATECOLUMNDUMP'] = $createColumnDump;
    		$_SESSION['PL_DUMPER_EXPORT_COLUMNS'] = $columns;
    		$_SESSION['PL_DUMPER_EXPORT_RELOADBEFOREPACK'] = $reloadBeforePack;

    		$result_str = '';

            if ($dumpFormat == SB_DUMPER_DUMP_SQL)
            {
           	   	$result_str = '/* Domains: ';

           	   	foreach ($GLOBALS['sb_domains'] as $key => $value)
   				{
   					$result_str .= $key;
   					if (SB_PORT != '80')
   						$result_str .= ':'.SB_PORT;

   					$result_str .= ',';
    			}

    			$result_str = substr($result_str, 0, -1);
   				$result_str .= " */ \n\n\n/* SBUILDER DUMPER */\n\n\nSET NAMES ".SB_DB_CHARSET.";\n\n\n";
            }
            elseif ($dumpFormat == SB_DUMPER_DUMP_XML)
            {
                $result_str = '<?xml version="1.0" encoding="'.SB_CHARSET.'" ?>'."\n\n\n".'<db>'."\n\n";
            }
        }

        if ($result_str != '' && !$GLOBALS['sbVfs']->fwrite($result_str))
        {
            $this->errorFlag = true;
            $this->errorStr = sprintf(SB_DUMPER_ERROR_WRITE_OUT_FILE, '/'.$fileName);

			$GLOBALS['sbVfs']->fclose();
			return false;
		}

		if ($dumpMode == SB_DUMPER_DUMP_FULL)
        {
            if (!$this->createFullDump($dumpFormat))
            {
                $this->errorFlag = true;
                $this->errorStr = sprintf(SB_DUMPER_ERROR_WRITE_OUT_FILE, '/'.$fileName);

                $GLOBALS['sbVfs']->fclose();
                return false;
            }

            if ($this->isNeedReload())
            {
                $GLOBALS['sbVfs']->fclose();
                return true;
            }
        }

        if ($dumpMode == SB_DUMPER_DUMP_TABLE)
        {
        	$this->createColumnDump = $createColumnDump;
        	$this->columns = $columns;

            if (!$this->createTableDump($tableName, $dumpFormat, $orderStr))
            {
                $this->errorFlag = true;
                $this->errorStr = sprintf(SB_DUMPER_ERROR_WRITE_OUT_FILE, '/'.$fileName);

                $GLOBALS['sbVfs']->fclose();
                return false;
            }

            if ($this->isNeedReload())
            {
				$GLOBALS['sbVfs']->fclose();
                return true;
            }
        }

        if ($dumpFormat == SB_DUMPER_DUMP_XML)
        {
			$GLOBALS['sbVfs']->fwrite('</db>');
		}
		$GLOBALS['sbVfs']->fclose();

        // Если дампер завершил свою работу без требования перезагрузки то очищаем сессию
        unset($_SESSION['PL_DUMPER_EXPORT_LIMIT']);
        unset($_SESSION['PL_DUMPER_EXPORT_PACK']);
        unset($_SESSION['PL_DUMPER_EXPORT_TABLE']);
        unset($_SESSION['PL_DUMPER_ORDER_STR']);
        unset($_SESSION['PL_DUMPER_EXPORT_FORMAT']);
    	unset($_SESSION['PL_DUMPER_EXPORT_CREATECOLUMNDUMP']);
        unset($_SESSION['PL_DUMPER_EXPORT_COLUMNS']);
    	unset($_SESSION['PL_DUMPER_EXPORT_RELOADBEFOREPACK']);

        if ($pack && $reloadBeforePack)
        {
            // Перед упаковкой перезагружаем дампер
            $_SESSION['PL_DUMPER_RELOAD_ACTION'] = 'EXPORT_PACK';
            $this->reloadDumper = true;
        }
        elseif ($pack && $reloadBeforePack == false)
        {
        	if ($this->packFile($packFileName, $fileName))
            {
                if ($fileName != '' && $GLOBALS['sbVfs']->exists($fileName) && $GLOBALS['sbVfs']->is_file($fileName))
                {
                    $GLOBALS['sbVfs']->delete($fileName);
                }
            }
            else
            {
            	return false;
            }
        }

        return true;
    }

    /**
     * Создает копию файлов системы для точки восстановления
     * @param boolean $pack         Упаковывать в GZ-архив (TRUE) или нет (FALSE)
     * @param string $packFileName  Имя архива для $pack==TRUE или директория хранения копии при $pack==FALSE
     * @return boolean
     */
    public function createDumpCMS($pack=false, $packFileName='')
    {
        $dest_dir = $pack? '/cms/tmp/cms/' : $packFileName.'/cms/';
        $dir_contents = $GLOBALS['sbVfs']->scandir('/cms');

        foreach ($dir_contents as $dir)
        {
            if($dir == '.' || $dir == '..' || $dir == 'backup' || $dir == 'tmp')
            {
                continue;
            }

            if ($GLOBALS['sbVfs']->is_dir('/cms/' . $dir))
            {
                if (!$GLOBALS['sbVfs']->copy('/cms/' . $dir, $dest_dir, true))
                {
                    $this->errorFlag = true;
                    $this->errorStr = SB_DUMPER_ERROR_BACKUP_CMS;
                    return false;
                }
            }
            else
            {
                if (!$GLOBALS['sbVfs']->copy('/cms/'.$dir, $dest_dir.$dir, true))
                {
                    $this->errorFlag = true;
                    $this->errorStr = SB_DUMPER_ERROR_BACKUP_CMS;
                    return false;
                }
            }
        }

        $GLOBALS['sbVfs']->delete($dest_dir.'kernel/config.inc.php');
        $GLOBALS['sbVfs']->delete($dest_dir.'kernel/key.php');

        if($pack)
        {
            $result = $this->packDir($dest_dir, $packFileName);
            $GLOBALS['sbVfs']->delete($dest_dir);
            if($result === false)
            {
                $this->errorFlag = true;
                $this->errorStr = SB_DUMPER_ERROR_ZIP_CMS_CREATE;
                return false;
            }
        }
    }

    /**
     * Проверка на соответствие доменов в дампе с доменами в системе
     *
     * @param string $filePath - имя файла с дампом, откуда будет взята информация о доменах.
     *
     * @return mixed TRUE, если домены в дампе полностью соотв. доменам в системе; FALSE в случае ошибки; массив
     * 				 доменов, присутствующих в дампе, но отсутствующих в системе.
     */
    public function checkDomains($filePath)
    {
    	$gzip = (strtolower(substr($filePath, -4)) == '.zip');
    	if ($gzip && !$this->zlibAvailable)
        {
            $this->errorFlag = true;
            $this->errorStr = SB_DUMPER_ERROR_ZLIB;

            return false;
        }

    	if ($gzip)
    	{
    		if (function_exists('gzopen64'))
    			$gz = @gzopen64(SB_BASEDIR.'/'.$filePath, 'rb');
    		else
    			$gz = @gzopen(SB_BASEDIR.'/'.$filePath, 'rb');

    		if (!$gz)
    		{
    			$this->errorFlag = true;
            	$this->errorStr = sprintf(SB_DUMPER_ERROR_OPEN_IN_FILE, '/'.$filePath);

            	return false;
    		}
    	}
    	else if(!$GLOBALS['sbVfs']->fopen($filePath, 'rb'))
        {
            $this->errorFlag = true;
            $this->errorStr = sprintf(SB_DUMPER_ERROR_OPEN_IN_FILE, '/'.$filePath);

            return false;
        }

        if ($gzip)
        {
        	$str = trim(@gzgets($gz, 4096));
        	@gzclose($gz);
        }
        else
        {
        	$str = trim($GLOBALS['sbVfs']->fgets());
        	$GLOBALS['sbVfs']->fclose();
        }

        $strPos = sb_strpos($str, '/* Domains: ');

        if($strPos === false)
        {
        	$this->errorFlag = true;
            $this->errorStr = sprintf(SB_DUMPER_ERROR_GET_DOMAINS, '/'.$filePath);

        	return false;
        }

        $str = trim(str_replace(array('/* Domains: ', ' */'), array('', ''), $str));
        $domains = explode(',', $str);
        $different_domains = array();
        $equal_domains = array();

   		for($i = 0; $i < count($domains); $i++)
   		{
   			$domains[$i] = trim($domains[$i]);
   			$pos = strpos($domains[$i], ':');
   			if ($pos !== false)
   			{
   				$domain = substr($domains[$i], 0, $pos);
   			}
   			else
   			{
   				$domain = $domains[$i];
   			}

   			if (!array_key_exists($domain, $GLOBALS['sb_domains']))
   			{
   				$different_domains[] = $domains[$i];
   			}
   			else
   			{
   				$equal_domains[] = $domains[$i];
   			}
   		}

   		if(count($different_domains) == 0 || count($equal_domains) == count($GLOBALS['sb_domains']))
    		return true;
    	else
    		return $domains;
    }

 	/**
 	 * Возвращает кодировку дампа
     *
     * @param string $filePath - имя файла с дампом, откуда будет взята информация о кодировке.
     *
     * @return string Кодировка дампа.
     */
    public function getCharset($filePath)
    {
    	$gzip = (strtolower(substr($filePath, -4)) == '.zip');
    	if ($gzip && !$this->zlibAvailable)
        {
            $this->errorFlag = true;
            $this->errorStr = SB_DUMPER_ERROR_ZLIB;

            return false;
        }

    	if ($gzip)
    	{
    		if (function_exists('gzopen64'))
    			$gz = @gzopen64(SB_BASEDIR.'/'.$filePath, 'rb');
    		else
    			$gz = @gzopen(SB_BASEDIR.'/'.$filePath, 'rb');

    		if (!$gz)
    		{
    			$this->errorFlag = true;
            	$this->errorStr = sprintf(SB_DUMPER_ERROR_OPEN_IN_FILE, '/'.$filePath);

            	return false;
    		}
    	}
    	else if(!$GLOBALS['sbVfs']->fopen($filePath, 'rb'))
        {
            $this->errorFlag = true;
            $this->errorStr = sprintf(SB_DUMPER_ERROR_OPEN_IN_FILE, '/'.$filePath);

            return false;
        }

        $i = 0;
        $eof = $gzip ? @gzeof($gz) : $GLOBALS['sbVfs']->feof();
        while ($i < 7 && !$eof)
        {
	        if ($gzip)
	        {
	        	$str = trim(@gzgets($gz, 4096));
	        }
	        else
	        {
	        	$str = trim($GLOBALS['sbVfs']->fgets());
	        }

	        $eof = $gzip ? @gzeof($gz) : $GLOBALS['sbVfs']->feof();
	        $i++;
        }

    	if ($gzip)
        {
        	@gzclose($gz);
        }
        else
        {
        	$GLOBALS['sbVfs']->fclose();
        }

        $strPos = sb_strpos($str, 'SET NAMES ');

        if($strPos === false)
        {
        	return SB_DB_CHARSET;
        }

        return trim(str_replace(array('SET NAMES', ';'), array('', ''), $str));
    }

    private function replaceSQL($sql)
    {
    	if(!is_array($this->replaceInfo) || count($this->replaceInfo) == 0)
    	{
    		return $sql;
    	}
    	else
    	{
    		$keys = array_keys($this->replaceInfo);
    		$values = array_values($this->replaceInfo);

    		// обработка сериализованных массивов
    		$matches = array();
    		$result = preg_match_all('#\s\'(a:\d+:\{.*?\})\'\s#s', $sql, $matches);
    		if (!$result)
    		{
    			$result = preg_match_all('#\s\'(s:\d+:\\\\".*?\\\\";)\'\s#s', $sql, $matches);
    		}

    		if ($result)
    		{
    			$search = array();
    			$replace = array();
    			$replace_tmp = array();

    			$i = 0;
    			foreach($matches[1] as $match)
    			{
    				if (!in_array($match, $search))
    				{
    					$ar = @unserialize(str_replace(array('\\\'', '\\"', '\r', '\n', '\\r', '\\n'), array('\'', '"', "\r", "\n", "\r", "\n"), $match));

    					if ($ar !== false)
    					{
    						$ar = sb_array_replace($keys, $values, $ar);
    						$search[] = $match;
    						$replace[] = trim($GLOBALS['sbSql']->escape(serialize($ar)), "'");
    						$replace_tmp[] = '!!**sb_tmp_replacer_'.$i.'**!!';

    						$i++;
    					}
    				}
    			}

    			$sql = str_replace($search, $replace_tmp, $sql);
    			$sql = sb_str_replace($keys, $values, $sql);

    			return str_replace($replace_tmp, $replace, $sql);
    		}

    		return sb_str_replace($keys, $values, $sql);
    	}
    }

	 /**
     * Загружает дамп из SQL файла.
     *
     * Если передан zip архив, то он должен содержать файл с названием dump.sql, в котором храниться дамп.
     *
     * @param string $filePath Имя и путь к файлу, содержащему дамп.
     *
     * @return boolean TRUE, если дамп был успешно загружен, FALSE в случае ошибки.
     */
	public function loadDumpFile($filePath)
    {
    	$filePath = preg_replace('/[^0-9a-zA-Z_\-\.\[\]\/\s]+/', '', $filePath);
        $filePath = preg_replace('/\.\.+/', '', $filePath);
        $filePath = trim(str_replace('\\', '/', $filePath), '/');

        if ($filePath == '')
        {
        	$this->errorFlag = true;
            $this->errorStr = sprintf(SB_DUMPER_ERROR_OPEN_IN_FILE, '/');

            return false;
        }

        $charset = $this->getCharset($filePath);
        if (strtolower($charset) != SB_DB_CHARSET)
        {
        	$this->replaceInfo[$charset] = SB_DB_CHARSET;
        }

        $gzip = (strtolower(substr($filePath, -4)) == '.zip');
    	if ($gzip && !$this->zlibAvailable)
        {
            $this->errorFlag = true;
            $this->errorStr = SB_DUMPER_ERROR_ZLIB;

            return false;
        }

        if ($gzip)
        {
        	if (function_exists('gzopen64'))
    			$gz = @gzopen64(SB_BASEDIR.'/'.$filePath, 'rb');
    		else
    			$gz = @gzopen(SB_BASEDIR.'/'.$filePath, 'rb');

        	if (!$gz)
    		{
    			$this->errorFlag = true;
            	$this->errorStr = sprintf(SB_DUMPER_ERROR_OPEN_IN_FILE, '/'.$filePath);

            	return false;
    		}
    	}
    	else if(!$GLOBALS['sbVfs']->fopen($filePath, 'rb'))
        {
            $this->errorFlag = true;
            $this->errorStr = sprintf(SB_DUMPER_ERROR_OPEN_IN_FILE, '/'.$filePath);

            return false;
        }

        $sql = $insert = '';
        $execute = 0;

        if ($this->isNeedReload())
        {
        	if ($gzip)
        		@gzseek($gz, $this->position);
        	else
            	$GLOBALS['sbVfs']->fseek($this->position);

            $sql = $_SESSION['PL_DUMPER_SQLIMPORT_SQL'];
            $insert = $_SESSION['PL_DUMPER_SQLIMPORT_INSERT'];
            $execute = $_SESSION['PL_DUMPER_SQLIMPORT_EXECUTE'];

            unset($_SESSION['PL_DUMPER_SQLIMPORT_SQL']);
            unset($_SESSION['PL_DUMPER_SQLIMPORT_INSERT']);
            unset($_SESSION['PL_DUMPER_SQLIMPORT_EXECUTE']);

            $this->reloadDumper = false;
        }

        $eof = $gzip ? @gzeof($gz) : $GLOBALS['sbVfs']->feof();
        while (!$eof)
        {
            if (!$this->unlimitTime && time() > $this->timeEnd)
            {
                $_SESSION['PL_DUMPER_SQLIMPORT_SQL'] = $sql;
                $_SESSION['PL_DUMPER_SQLIMPORT_INSERT'] = $insert;
                $_SESSION['PL_DUMPER_SQLIMPORT_EXECUTE'] = $execute;
                $_SESSION['PL_DUMPER_SQLIMPORT_FILEPATH'] = $filePath;
                $_SESSION['PL_DUMPER_SQLIMPORT_POSITION'] = $gzip ? gztell($gz) : $GLOBALS['sbVfs']->ftell();
                $_SESSION['PL_DUMPER_RELOAD_ACTION'] = 'SQL_IMPORT_RELOAD';
                $_SESSION['PL_DUMPER_SQLIMPORT_REPLACEINFO'] = $this->replaceInfo;

                $this->reloadDumper = true;

                if ($gzip)
        			@gzclose($gz);
        		else
        			$GLOBALS['sbVfs']->fclose();
                return true;
            }

	        if ($gzip)
	        {
	        	$str = trim(@gzgets($gz, 1024 * 1024));
	        }
	        else
	        {
	        	$str = trim($GLOBALS['sbVfs']->fgets());
	        }

	        $eof = $gzip ? @gzeof($gz) : $GLOBALS['sbVfs']->feof();

            if (empty($str) || preg_match('/^(#|--|\/\*)/'.SB_PREG_MOD, $str))
            {
                continue;
            }

            $tmp = $this->replaceSQL($str);
            if (!empty($tmp))
            	$str = $tmp;

            $m = array();
            if (!$insert && preg_match('/^(INSERT INTO `?([^` ]+)`? .*?VALUES)(.*)$/i'.SB_PREG_MOD, $str, $m))
            {
                $insert = $m[1] . ' ';
                $sql .= $m[3];
            }
            else
            {
                $sql .= $str;
            }
            if ($sql)
            {
                if (sb_substr($str, -1) == ';')
                {
                    $sql = sb_substr($insert . $sql, 0, -1);
                    $insert = '';
                    $execute = 1;
                }

                if ($execute)
                {
                	sql_query($sql);

                    if ($GLOBALS['sbSql']->mErrno != 0)
                    {
                        $this->errorFlag = true;
                        $this->errorStr = $GLOBALS['sbSql']->mError;

                        if ($gzip)
        					@gzclose($gz);
        				else
        					$GLOBALS['sbVfs']->fclose();
                        return false;
                    }

                    $sql = '';
                    $execute = 0;
                }
            }
        }

        if ($gzip)
        	@gzclose($gz);
        else
        	$GLOBALS['sbVfs']->fclose();

        if (SB_PORT != '80')
        {
        	$res = sql_query('UPDATE sb_categs SET cat_title="'.SB_COOKIE_DOMAIN.'" WHERE cat_title="'.SB_COOKIE_DOMAIN.':'.SB_PORT.'"');
        }

        // Если дампер завершил свою работу без требования перезагрузки то очищаем сессию
        unset($_SESSION['PL_DUMPER_SQLIMPORT_SQL']);
        unset($_SESSION['PL_DUMPER_SQLIMPORT_INSERT']);
        unset($_SESSION['PL_DUMPER_SQLIMPORT_EXECUTE']);
        unset($_SESSION['PL_DUMPER_SQLIMPORT_FILEPATH']);
        unset($_SESSION['PL_DUMPER_SQLIMPORT_POSITION']);
        unset($_SESSION['PL_DUMPER_RELOAD_ACTION']);
        unset($_SESSION['PL_DUMPER_SQLIMPORT_REPLACEINFO']);

        return true;
    }

    public function restoreDumpCMS($cmsDumpPath)
    {
        if($GLOBALS['sbVfs']->is_file('cms/backup/' . $cmsDumpPath))
        {
            if (!class_exists('ZipArchive'))
            {
                $this->errorFlag = true;
                $this->errorStr = SB_DUMPER_ERROR_ZIP_CMS_EXTRACT;
                return false;
            }

            $zip = new ZipArchive();
            if($zip->open(SB_BASEDIR . '/cms/backup/' . $cmsDumpPath) !== true)
            {
                $this->errorFlag = true;
                $this->errorStr = SB_DUMPER_ERROR_ZIP_CMS_EXTRACT;
                return false;
            }

            $zip->extractTo(SB_BASEDIR);
            $zip->close();
        }
        elseif ($GLOBALS['sbVfs']->is_dir('cms/backup/' . $cmsDumpPath))
        {
            if(!$GLOBALS['sbVfs']->copy('cms/backup/' . $cmsDumpPath . '/cms', '/', true))
            {
                $this->errorFlag = true;
                $this->errorStr = SB_DUMPER_ERROR_CMS_RESTORE;
                return false;
            }
        }
    }

    private function packDir($pathDir, $fileName)
    {
        if(!class_exists('ZipArchive'))
        {
            return false;
        }

        $zip = new ZipArchive();
        if ($zip->open(SB_BASEDIR . '/' . $fileName, ZipArchive::CREATE) !== true)
        {
            return false;
        }

        if(version_compare('5.1.2', phpversion()) !== -1 && version_compare('5.1.2', phpversion()) !== 0)
        {
            $cms = new RecursiveIteratorIterator(new RecursiveDirectoryIterator(SB_BASEDIR. $pathDir));
            while($cms->valid()){
                if(!$cms->isDot())
                {
                    if($GLOBALS['sbVfs']->exists($pathDir.$cms->getSubPathName()))
                    {
                        $zip->addFile(SB_BASEDIR.$pathDir.$cms->getSubPathName(), 'cms/'.$cms->getSubPathName());
                    }
                }
                $cms->next();
            }
        }
        else
        {
            $this->getRecursiveFiles($pathDir);
            if(!empty($this->files))
            {
                foreach($this->files as $file)
                {
                    $pathInfo = pathinfo($file);
                    $dir = sb_str_replace($pathDir, '', $pathInfo['dirname']);
                    $zip->addFile(SB_BASEDIR.$file, 'cms/'.$dir.'/'.$pathInfo['basename']);
                }
            }
        }
        if(!$zip->close())
        {
            return false;
        }
    }

    private function getRecursiveFiles($pathDir)
    {
        $items = $GLOBALS['sbVfs']->scandir($pathDir);
        foreach($items as $item)
        {
            if($item == '.' || $item == '..')
            {
                continue;
            }

            $fullPath = $pathDir.$item;
            if($GLOBALS['sbVfs']->is_dir($fullPath))
            {
                $this->getRecursiveFiles($fullPath.'/');
            }
            else
            {
                if(!in_array($fullPath, $this->files) && $GLOBALS['sbVfs']->is_file($fullPath))
                {
                    $this->files[] = $fullPath;
                }
            }
        }
    }
}
?>