<?php
/**
 * Реализация класса для определения мобильных устройств
 * Для определения устройств используется сервис Яндекс-детектор
 * @author Артур Фурса
 * @param $cacheTime Int Время кеширования результатов в секундах (по умолчанию две недели)
 * @see http://api.yandex.ru/detector/
 */
class sbMobile{
    const DETECTOR_LINK = 'http://phd.yandex.net/detect';
    
    /**
     * @var Время валидности кеша 
     */
    private $_cacheTime;
    
    /**
     * @var Получать подробную информацию об устройстве
     */
    private $_info = false;
    
    /**
     * @var Содержимое заголовков запроса
     */
    private $_headers;
    
    /**
     * @var Хеш заголовков запроса
     */
    private $_hash;
    
    /**
     * @param $cacheTime Int Время кеширования результатов в секундах (по умолчанию две недели)
     */
    public function __construct($cacheTime = 1209600)
    {
        $this->_cacheTime = $cacheTime;
    }
    
    /**
     * Возвращает информацию о мобильном устройстве или false если устройство не определено
     * @param $info Boolean Возвращать подробную информацию об устройстве (по умолчанию false)
     * @return String|Boolean JSON-строка
     * @see http://api.yandex.ru/detector/doc/dg/concepts/detector-response.xml
     */
    public function detect($info=false)
    {
        //Определяем заголовки запроса для формирования хеша и последующей передачи на детектор
        $headers = '';
        $excl = array('HTTP_COOKIE', 'HTTP_HOST', 'HTTP_CONNECTION', 'HTTP_IF_MODIFIED_SINCE', 'HTTP_IF_NONE_MATCH', 'HTTP_CACHE_CONTROL', 'HTTP_ACCEPT_ENCODING');
        
        foreach ($_SERVER as $key => $value) {
           if (strpos($key, 'HTTP_') === 0 && !in_array($key, $excl)) {
               $key = strtolower(strtr(substr($key, 5), '_', '-'));
               $headers .= $key . ': ' . $value . "\r\n";
           }
        }
        
        $this->_headers = $headers;
        $this->_hash = md5($headers);
        $this->_info = $info;
        
        //удаляем старые записи с вероятностью 10%
        $needClear = (rand(1, 10) == 10);
        if ($needClear)
        {
            $time = time() - $this->_cacheTime;

            sql_query('DELETE FROM sb_mobile WHERE date < ?', $time);
        }
        
        return $this->getFromCache();
    }
    
    /**
     * Получаем данные из кеша если они там есть. 
     * В противном случае выполняется запрос на сервис Яндекса
     */
    protected function getFromCache()
    {
        //кука с данными присутствует только у мобильных
        if(isset($_COOKIE['sb_mobile_info']) && trim($_COOKIE['sb_mobile_info']) != '')
        {
            return $this->_info ? base64_decode($_COOKIE['sb_mobile_info']) : true;
        }
        
        if(!$this->_info)
        {
            $res = sql_query('SELECT mobile FROM sb_mobile WHERE hash = ?', $this->_hash);
            if($res)
            {
                sql_query('UPDATE sb_mobile SET counter = counter + 1 WHERE hash = ?', $this->_hash);

                return $res[0][0] == '1' ? true : false;
            }
            
            return $this->getDetectorData();
        }
        
        //если требуется подробная информация об устройстве и в куках она не найдена, то идем в гости к яндексу
        return $this->getDetectorData();
    }
    
    /**
     * Отправляем запрос на яндекс-детектор
     */
    private function getDetectorData()
    {
        $opts = array(
            'http'=>array(
               'method'=>"GET",
               'header'=> $this->_headers,
             )
            );
            
        $obj = simplexml_load_string(file_get_contents(self::DETECTOR_LINK, false, stream_context_create($opts)));
        $data = '';
        if($obj)
        {
            $data = json_encode($obj);
        }
        $mobDevice = isset($obj -> name) ? 1 : 0;

        //сохраняем в бд результат запроса к детектору, чтобы снизить кол-во запросов на внешние сервисы
        sql_query('INSERT INTO sb_mobile 
            (id, hash, date, mobile, counter) 
            VALUES ("", "'.$this->_hash.'", "'.time().'", "'.$mobDevice.'", 0)');
        
        //если тип устройства мобильный, то сохраняем его данные в кук для последующего использования
        if($mobDevice)
        {
            sb_setcookie('sb_mobile_info', base64_encode($data), time()+$this->_cacheTime);
        }

        return $this->_info ? $data : $mobDevice;
    }
}
