<?php
/**
 * Реализация класса работы с СУБД MS SQL и подключение к базе
 *
 * Данный класс работает с СУБД MS SQL, используя расширение PHP MS SQL (sqlsrv_connect(), sqlsrv_query() и т.д.).
 * После обявления класса вызывается конструктор класса и идет подключение к указанной в настройках системе (config.inc.php) БД.
 * В случае успешного подключения, ссылка на класс сохраняется в глобальной переменной $GLOBALS['sbSql'], иначе
 * скрипт прерывает свою работу и выводит соотв. ошибку.
 *
 * @see config.inc.php
 * @see $GLOBALS['sbSql']
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_DB
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

/**
 * Константа, использующаяся в качестве пустого значения заменителя макроса
 *
 */
define('SB_SQL_SKIP', log(0));

/**
 * Класс работы с СУБД MS SQL
 *
 * Данный класс работает с СУБД MS SQL, используя расширение PHP MS SQL (sqlsrv_connect(), sqlsrv_query() и т.д.).
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_DB
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbMSSql implements sbISql
{
    /**
     * Номер ошибки MS SQL
     *
     * В случае возникновения ошибки в SQL-запросе или при соединении с БД,
     * в эту переменную записывается номер ошибки. Ошибка попадает в системный журнал.
     *
     * @var integer
     */
    public $mErrno    = 0;

    /**
     * Текст ошибки MS SQL
     *
     * В случае возникновения ошибки в SQL-запросе или при соединении с БД,
     * в эту переменную записывается текст ошибки. Ошибка попадает в системный журнал.
     *
     * @var string
     */
    public $mError    = '';

    /**
     * Отображать или нет ошибки MS SQL
     *
     * Если TRUE, то ошибки будут выводится в браузер пользователя, если FALSE - то не будут.
     * В любом случае ошибки всегда записываются в системный журнал.
     *
     * @var bool
     */
    public $mShowErrors = false;

    /**
     * Идентификатор активного соединения с MS SQL
     *
     * Хранит идентификатор последнего соединения с БД, возвращенный функцией sqlsrv_connect() в методе connect().
     *
     * @var resource
     */
    public $mLinkId   = false;

    /**
     * Идентифиактор последнего запроса к MS SQL
     *
     * Хранит идентификатор последнего запроса к БД, возвращенный функцией sqlsrv_query() в различных методах класса.
     *
     * @var resource
     */
    public $mQueryId  = false;

    /**
     * Тип запроса
     *
     * Хранит тип последнего выполненного запроса к БД. Возможные значения -  'DELETE', 'UPDATE', 'INSERT' или 'SELECT'.
     *
     * @var string
     */
    private $mQueryType = '';

    /**
     * Значение автоинкриментного поля после последнего запроса INSERT
     *
     * @var int
     */
    private $mQueryInsertId = 0;

    /**
     * Кол-во записей, затронутых запросом
     *
     * Хранит кол-во записей, возвращенных SELECT-ом, либо кол-во записей, измененных UPDATE-ом, INSERT-ом или DELETE-ом в
     * последнем запросе к БД.
     *
     * @var int
     */
    private $mQueryNumRows = 0;

    /**
     * Массив значений для "заменителей"
     *
     * @var array
     */
    private $_mPlaceholderArgs;

    /**
     * Значение для текущего "заменителя" не найдено
     *
     * TRUE - не найдено, FALSE - найдено
     *
     * @var bool
     */
    private $_mPlaceholderNoValueFound = false;

    /**
     * Версия СУБД MS SQL.
     *
     * @var string
     */
    private $mVersionInfo = '';

    /**
     * Общее кол-во SQL-запросов
     *
     * Хранит общее кол-во запросов к базе данных при формировании страницы.
     *
     * @var int
     */
    private $mAllQueryNum = 0;

    /**
     * Конструктор класса
     *
     * В конструкторе класса идет подключение к БД по переданным в конструктор реквизитам доступа к БД.
     *
     * @param string $database Имя базы данных, к которой производится подключение
     * @param strign $host Хост базы данных
     * @param strign $user Пользователь, под которым производится подключение
     * @param strign $password Пароль пользователя
     *
     * @see connect()
     */
    function __construct($database = '', $host = '', $user = '', $password = '')
    {
        $this->connect($database, $host, $user, $password);
    }

    /**
     * Деструктор класса
     *
     * Закрывает активное соединение с БД.
     * @see disconnect()
     */
    public function __destruct()
    {
        $this->disconnect();
    }

    /**
     * Запись ошибок
     *
     * Записывает код и текст ошибки в соотв. переменные класса и в системный журнал.
     * Если включен вывод ошибок, то выводит код и текст ошибки в браузер пользователя.
     *
     * @param string $query SQL-запрос, инициировавший ошибку
     */
    private function halt($query = '')
    {
    	$errors = @sqlsrv_errors(SQLSRV_ERR_ERRORS);
        if ( is_null($errors) )
            return;

        $this->mError = (SB_CHARSET != 'WINDOWS-1251' ? iconv('WINDOWS-1251', SB_CHARSET, $errors[0]['message']) : $errors[0]['message']);
        $this->mErrno = $errors[0]['code'];

        if ($this->mShowErrors)
        {
        	printf("<b>MS SQL Error</b>: %s (%s)<br>\n", $this->mErrno, $this->mError);
        }
        elseif (function_exists('sb_add_system_message') && stripos($query, 'sb_system_log') === false)
        {
        	// записываем ошибку в системный журнал
            $message = '<span style="color:red;">'.KERNEL_ERROR_MYSQL.'</span>:<br />['.$this->mErrno.'] '.$this->mError.($query != '' ? '<br /><br /><span style="color:red;">'.KERNEL_ERROR_SQL_QUERY.'</span>:<br />'.sb_htmlentities($query) : '');

        	$aCallstack = debug_backtrace(false);

	        $backtrace = '';
	        if (count($aCallstack) > 0)
	        {
	        	$aCallstack = array_reverse($aCallstack);
	        	$backtrace = '<br /><br /><span style="color:red;">'.KERNEL_ERROR_BACKTRACE.'</span>:<br /><br /><div align="center"><table class="form" width="95%" cellspacing="1" cellpadding="3">
	        		<tr><th class="debug">№</th><th class="debug">'.KERNEL_ERROR_BACKTRACE_FUNCTION.'</th><th class="debug">'.KERNEL_ERROR_BACKTRACE_FILE.'</th><th class="debug">'.KERNEL_ERROR_BACKTRACE_LINE.'</th></tr>';

	        	$i = 1;
	        	foreach($aCallstack as $aCall)
				{
					if ($aCall['function'] == 'sb_php_error_reporting' || $aCall['function'] == 'sb_php_shutdown')
						continue;

					if (!isset($aCall['file']))
						$aCall['file'] = KERNEL_ERROR_BACKTRACE_PHP_KERNEL;

					if (!isset($aCall['line']))
						$aCall['line'] = '';

					$backtrace .= '<tr'.($i % 2 != 0 ? ' class="even_debug"' : ' class="odd_debug"').'><td>'.($i++).'</td><td>'.$aCall['function'].'</td><td>'.sb_str_replace(SB_BASEDIR, '', sb_str_replace('\\', '/', $aCall['file'])).'</td><td>'.$aCall['line'].'</td></tr>';
				}

				$backtrace .= '</table></div><br />';
	        }

            sb_add_system_message($message.$backtrace, SB_MSG_SQL_ERROR);
        }

    }

    /**
     * Возвращает код последней ошибки или FALSE, если ошибки не было
     *
     * @return int Код последней ошибки.
     */
    private function getError()
    {
    	$errors = @sqlsrv_errors(SQLSRV_ERR_ERRORS);

    	if (is_null($errors))
    		return false;

    	return $errors[0]['code'];
    }

    /**
     * Осуществляет подключение к БД
     *
     * @param string $database Имя базы данных, к которой производится подключение
     * @param strign $host Хост базы данных
     * @param strign $user Пользователь, под которым производится подключение
     * @param strign $password Пароль пользователя
     *
     * @return resource Идентификатор соединения или FALSE в случае возникновения ошибки
     */
    public function connect( $database, $host, $user, $password = '' )
    {
        $this->mLinkId = false;
        /**
         * Если не задана база данных, хост или пользователь,
         * то выводим ошибку и возвращаем FALSE
         */
        if ( '' == $database || '' == $host || '' == $user )
        {
            return false;
        }

        /**
         * Пытаемся установить соединение с БД и, в случае успеха, записываем идентификатор соединения
         * в переменную класса $mLinkId
         */
        $params = array('CharacterSet' => SB_CHARSET,
        	'Database' => $database,
        	'UID' => (!defined('SB_USE_ENCRYPT') || SB_USE_ENCRYPT ? sb_decrypt($user, SB_HASH) : $user),
        	'PWD' => (!defined('SB_USE_ENCRYPT') || SB_USE_ENCRYPT ? sb_decrypt($password, SB_HASH) : $password));

        $this->mLinkId = @sqlsrv_connect($host, $params);

        /**
         * Если не удалось установить соединение, выводим ошибку и возвращаем FALSE
         */
        if ( $this->getError() !== false || !is_resource($this->mLinkId) )
        {
            $this->halt();
            $this->mLinkId = false;
            return false;
        }

    	$res = @sqlsrv_server_info($this->mLinkId);
        if ($res)
        {
            $this->mVersionInfo = $res['SQLServerVersion'];
    	}
    	else
    	{
    		$this->halt();
            $this->mLinkId = false;
            return false;
    	}

        $res = @sqlsrv_query('SET ANSI_NULL_DFLT_ON ON', $this->mLinkId);
		$this->free($res);

        /**
         * Возвращаем идентификатор соединения
         */
        return $this->mLinkId;
    }

    /**
     * Выбор рабочей БД
     *
     * @param string $database
     * @return bool TRUE - если ошибок не было и FALSE в случае возникновения ошибки
     */
    public function changeDB($database)
    {
        if ( !is_resource($this->mLinkId) || empty($database) )
        {
            return false;
        }

        /**
         * Если не удалось выбрать базу данных, выводим ошибку и возвращаем FALSE
         */
        $res = @sqlsrv_query($this->mLinkId, 'USE '.$database);
        if (!$res)
        {
            $this->halt();
            return false;
        }

        $this->free($res);
        return true;
    }

    /**
     * Закрывает соединение с БД
     *
     * Закрывает соединение с базой данных. Если в качестве параметра передан идентификатор соединения с БД,
     * то закрывается это соединение, иначе соединие, на которое указывает переменная класса mLinkId.
     *
     * @see mLinkId
     * @param resource $link_id
     */
    public function disconnect($link_id = false)
    {
        if ( is_resource($link_id) )
        {
            @sqlsrv_close($link_id);
        }
        else
        {
            if ( is_resource($this->mLinkId) )
                @sqlsrv_close($this->mLinkId);

            $this->mLinkId = false;
        }
    }

    /**
     * Освобождает память, выделенную под результат запроса
     *
     * В качестве идентификатора запроса используется либо переданный в функцию идентификатор,
     * либо, если переданный идентификатор не является ресурсом, значение переменной класса mQueryId.
     *
     * @see mQueryId
     *
     * @param resource $query_id
     *
     */
    public function free($query_id = false)
    {
        if (is_resource($query_id))
        {
            @sqlsrv_free_stmt($query_id);
        }
        else
        {
            if ( is_resource($this->mQueryId) )
            {
                @sqlsrv_free_stmt($this->mQueryId);
            }

            $this->mQueryInsertId = 0;
            $this->mQueryNumRows = 0;
            $this->mQueryId = false;
        }
    }

    /**
     * Возвращает значение, сгенерированное для AUTO_INCREMENT поля предыдущим запросом INSERT
     *
     * @return int Значение AUTO_INCREMENT поля или 0, если предыдущий запрос INSERT был неудачным.
     */
    public function getInsertId()
    {
        return $this->mQueryInsertId;
    }

    /**
     * Возвращает кол-во записей, затронутых предыдущим SQL-запросом.
     *
     * Для SELECT-запроса это кол-во выбранных из БД записей, для запросов INSERT, UPDATE и DELETE - кол-во записей,
     * затронутых данными запросами.
     *
     * @return int Кол-во записей или 0, если предыдущий запрос был неудачным.
     */
    public function getAffectedRows()
    {
        return $this->mQueryNumRows;
    }

    /**
     * Дублирует запись в таблице
     *
     * @param int $id Уникальный идентификатор дублируемой записи (обычно значение AUTO_INCREMENT поля)
     * @param string $id_field Название поля таблицы, в котором хранятся значения $id
     * @param string $table Название таблицы, запись которой мы хотим продублировать
     * @param string $null_fields Названия полей, которым следует присвоить NULL-значения.
     * @param array  $fields_values Поля, которым следует присвоить новые значения. key-название поля, value-новое значение.
     *
     * @return int Значение AUTO_INCREMENT поля новой записи или 0 в случае возникновения ошибки
     */
    public function duplicateRow($id, $id_field, $table, $null_fields='', $fields_values = array())
    {
        if ( !is_resource($this->mLinkId) )
        {
            return false;
        }

        $id = intval($id);
        $id_field = preg_replace('/[^a-z0-9_]+/i', '', $id_field);
        $table = preg_replace('/[^a-z0-9_]+/i', '', $table);

        /**
         * Вытаскиваем все поля дублируемой записи
         */
        $result = @sqlsrv_query($this->mLinkId, 'SELECT * FROM '.$table.' WHERE '.$id_field.'=\''.$id.'\'', array(), array('Scrollable' => SQLSRV_CURSOR_STATIC));
        if ( !is_resource($result) )
        {
            $this->halt('SELECT * FROM '.$table.' WHERE '.$id_field.'=\''.$id.'\'');
            return false;
        }

        $fields = @sqlsrv_fetch_array($result, SQLSRV_FETCH_ASSOC);
        $this->free($result);

        /**
         * $column_str Содержит список названий полей
         * $values_str Содержит список значений полей
         */
        $columns_str = '';
        $values_str = '';

        if ($null_fields != '')
        	$null_fields = explode(',', $null_fields);
        else
        	$null_fields = array();

        foreach($fields as $field => $value)
        {
            if ($field != $id_field)
            {
				if(array_key_exists($field, $fields_values))
                {
					$values_str .= $this->escape($fields_values[$field]).', ';
                }
                elseif (!in_array($field, $null_fields) && !is_null($value))
                {
                    $values_str .= $this->escape($value).', ';
                }
                else
                {
                    $values_str .= 'NULL, ';
                }

        	    $columns_str .= $field.', ';
            }
        }
        $columns_str = sb_substr($columns_str, 0, -2);
        $values_str = sb_substr($values_str, 0, -2);

        /**
         * Пытаемся вставить новую запись в таблицу
         */
        $query_str = "INSERT INTO $table ($columns_str) VALUES ($values_str)";
        if (!$this->paramQuery($query_str))
        {
            $this->halt($query_str);
            return false;
        }

        return $this->getInsertId();
    }

    /**
     * Выполняет параметризованный запрос
     *
     * <b>Важно!</b> Всегда используйте данный метод, поскольку он реализует полную защиту от SQL-инъекций.
     * Метод query() был оставлен для совместимости со старыми версиями модулей и для выполнения запросов без параметров.
     * Возвращаемые значения аналогичны методу <i>query()</i>.
     *
     * В запросах можно использовать следующие макросы:
     * <ul>
     * <li><b>?</b> - Этот макрос используется в большинстве запросов. Вставляемое на его место значение
     *      обрамляется апострофами ('), при этом все служебные символы экранируются.
     * <code>
     * $GLOBLAS['sbSql']->paramQuery('UPDATE table SET title=?', 'Заголовок "новости"');
     * // MySQL: UPDATE table SET title='Заголовок \"новости\"';
     * </code>
     *
     * <li><b>?a</b> - Этот макрос удобно использовать для составления IN-выражений. Если в качестве параметра передан
     *      ассоциативный массив, то макрос заменяется набором пар ключ=значение. Это удобно использовать в UPDATE-запросах.
     * <code>
     * $ids = array(1, 2, 3);
     * $GLOBLAS['sbSql']->paramQuery('SELECT title FROM table WHERE id IN(?a)', $ids);
     * // SELECT title FROM table WHERE id IN(1, 2, 3);
     *
     * $row = array('id' => 1, 'title' => 'Заголовок "новости"');
     * $GLOBLAS['sbSql']->paramQuery('UPDATE table SET ?a', $row);
     * // MySQL: UPDATE table SET `id`='1', `title`='Заголовок \"новости\"';
     * </code>
     * <li><b>?#</b> - Данный макрос заставляет СУБД воспринимать значение как идентификатор. Он также может принимать
     *      в качестве значения не только строку, но и массив (список значений). Это очень удобно для формирования INSERT-запросов.
     * <code>
     * $GLOBLAS['sbSql']->paramQuery('SELECT ?# FROM table', 'date');
     * // MySQL: SELECT `date` FROM table
     *
     * $row = array('id' => 1, 'title' => 'Заголовок "новости"');
     * $GLOBLAS['sbSql']->paramQuery('INSERT INTO table(?#) VALUES(?a)', array_keys($row), array_values($row));
     * // MySQL: INSERT INTO table (`id`, `title`) VALUES ('1', 'Заголовок \"новости\"');
     * </code>
     * <li><b>?d</b> - Этот макрос преобразует переданный параметр в целое число, которое вставляется без обрамления апострофами.
     *      В случае ошибки конвертирования вставляется 0.
     * <code>
     * $GLOBLAS['sbSql']->paramQuery('SELECT title FROM table WHERE id=?d', 1);
     * // MySQL: SELECT title FROM table WHERE id=1;
     * </code>
     * <li><b>?f</b> - Этот макрос преобразует переданный параметр в дробное число, которое вставляется без обрамления апострофами.
     *      В случае ошибки конвертирования вставляется 0.
     * <code>
     * $GLOBLAS['sbSql']->paramQuery('SELECT title FROM table WHERE id="?f"', 1,56);
     * // MySQL: SELECT title FROM table WHERE id="1.56";
     * </code>
     * <li><b>?n</b> - Подставляемое значение преобразуется в целое число. Если это число равно нулю, то вместо него
     *      подставляется NULL, иначе — оно само.
     * </ul>
     * Также возможно использование условных макроподстановок:
     * <code>
     * $GLOBLAS['sbSql']->paramQuery('SELECT * FROM table WHERE `id` = ?d { AND `date` > ? }',
     *                                $id, (isset($_POST['date']) ? $_POST['date'] : SB_SQL_SKIP) );
     * </code>
     * В данном примере блок <samp>AND `date` > ?</samp> будет пропущен, если через POST-форму не пришло значение date.
     *
     * @see query
     *
     * @param string $query Текст SQL-запрос
     * @param mixed $arg1,... Значения макросов в запросе
     *
     * @return mixed Двумерный массив для SELECT-запросов или идентификатор запроса. FALSE в случае
     *         возникновения ошибки.
     */
	public function paramQuery()
    {
        $args = func_get_args();
        $total = false;

        if (count($args) == 1 && is_array($args[0]))
        	return $this->query($args[0], $total);
        else
        	return $this->query($args, $total);
    }

    /**
     * Аналогичен методу paramQuery за исключением того, что в переменную $total записывается общее кол-во записей в таблице
     *
     * Удобно применять данный метод при использовании пейджирования результатов запроса.
     *
     * @see paramQuery
     *
     * @param int $total Получает общее кол-во найденных записей (независимо от использования LIMIT)
     * @param string $query Текст SQL-запрос
     * @param mixed $arg1,... Значения макросов в запросе
     *
     * @return mixed Двумерный массив для SELECT-запросов или идентификатор запроса. FALSE в случае
     *         возникновения ошибки.
     */
	public function paramQueryPage(&$total)
    {
        $args = func_get_args();
        array_shift($args);
        $total = true;

        if (count($args) == 1 && is_array($args[0]))
        	return $this->query($args[0], $total);
        else
        	return $this->query($args, $total);
    }

    /**
     * Аналогичен paramQuery
     *
     * @see query
     *
     * @param string $query Текст SQL-запрос
     * @param mixed $arg1,... Значения макросов в запросе
     *
     * @return mixed Ассоциативный массив для SELECT-запросов или результат работы функции mysqli_query(). FALSE в случае
     *         возникновения ошибки.
     */
    public function paramAssoc()
    {
        $args = func_get_args();
        $total = false;

        if (count($args) == 1 && is_array($args[0]))
        	return $this->query($args[0], $total, true);
        else
        	return $this->query($args, $total, true);
    }

    /**
     * Аналогичен paramQueryPage, но возвращает набор ассоциативных массивов
     *
     * @see paramQueryPage
     *
     * @param int $total Получает общее кол-во найденных записей (независимо от использования LIMIT)
     * @param string $query Текст SQL-запрос
     * @param mixed $arg1,... Значения макросов в запросе
     *
     * @return mixed Ассоциативный массив для SELECT-запросов или результат работы функции mysqli_query(). FALSE в случае
     *         возникновения ошибки.
     */
    public function paramAssocPage(&$total)
    {

        $args = func_get_args();
        array_shift($args);
        $total = true;

        if (count($args) == 1 && is_array($args[0]))
        	return $this->query($args[0], $total, true);
        else
        	return $this->query($args, $total, true);
    }

    /**
     * Экранирует переданную строку в соотв. с правилами СУБД
     *
     * @param string $str Строка для экранирования
     * @param $is_ident Название поля или таблицы (TRUE) или значение (FALSE)
     * @param $add_quotes Обрамляет результат в ` или '
     *
     * @return string Экранированная строка
     */
    public function escape($str, $is_ident=false, $add_quotes=true)
    {
        if (!$is_ident)
        {
            if ($add_quotes)
            {
            	return "'".sb_str_replace("'", "''", $str)."'";
            }
            else
            {
            	if (function_exists('mysql_escape_string'))
	            	$str = @mysql_escape_string($str);
	            else
	                $str = @addslashes($str);

	            return $str;
            }
        }
        else
        {
            if ($add_quotes)
	            return '"'.preg_replace('/[^a-z0-9_]+/i', '', $str).'"';
	        else
	            return preg_replace('/[^a-z0-9_\.]+/i', '', $str);
        }
    }

    /**
     * Используется в методах <i>paramQuery()</i> и <i>paramQueryPage()</i>.
     * Вместо текста SQL-запроса передается массив, первый элемент которого - текст SQL-запроса, остальные элементы -
     * значения макросов, использующихся в SQL-запросе. Принимает также параметр $total. Если значение этого параметра TRUE,
     * то в него записывается общее кол-во выбранных SQL-запросом (независимо от использования LIMIT).
     *
     * @param array $query Массив, первый элемент которого - текст SQL-запроса, остальные элементы -
     *                     значения макросов, использующихся в SQL-запросе
     * @param bool $total Если значение этого параметра TRUE, то в него записывается общее кол-во выбранных
     *                    SQL-запросом записей (независимо от использования LIMIT)
     * @param bool $assoc Метод возвращает набор ассоциативных массивов (true) для SELECT-запросов или двумерный массив (false)
     *
     * @return mixed Двумерный массив для SELECT-запросов или TRUE. FALSE в случае
     *         возникновения ошибки.
     */
    private function query($query, &$total, $assoc = false)
    {
        /**
         * Если текст запроса пустой, или не установлено соединение с базой, вернуть FALSE
         */
        if ( !is_array($query) || $query[0] == '' || !is_resource($this->mLinkId) )
        {
            return false;
        }

        /**
         * Освобождаем память, выделенную под предыдущий запрос, если таковой был
         */
        $this->free();

        /**
		 * Инкремент счетчика запросов
		 */
		$this->mAllQueryNum++;

        /**
         * Определяем тип запроса (SELECT, DELETE, INSERT, UPDATE) и записываем в переменную класса mQueryType.
         * Если тип запроса определить не удалось, то 'UNDEFINED'.
         */
        $m = null;
        if (preg_match('/^\s* \(?([a-zA-Z]*) \s+/six'.SB_PREG_MOD, $query[0], $m))
        {
            $this->mQueryType = sb_strtoupper($m[1]);
        }
        else
        {
            $this->mQueryType = 'UNDEFINED';
        }

        if ($this->mQueryType == 'LOCK' || $this->mQueryType == 'UNLOCK')
        {
        	return true;
        }

        $query[0] = sb_str_replace(' STRAIGHT_JOIN ', ' ', $query[0]);

        /**
         * Модифициурем запрос, если надо вытащить общее кол-во записей или есть LIMIT.
         */

        $offset = null;
        $count = null;
        if ($this->mQueryType == 'SELECT')
        {
        	$m = null;
	        if (preg_match('/\s+ LIMIT \s+ (\d+)(\s*,\s*)?(\d+)? \s*$/msix'.SB_PREG_MOD, $query[0], $m))
	        {
	        	$query[0] = preg_replace('/\s+ LIMIT \s+ (\d+)(\s*,\s*)?(\d+)? \s*$/msix'.SB_PREG_MOD, '', $query[0]);

	        	if (isset($m[3]))
	        	{
	        		$offset = intval($m[1]);
	        		$count = intval($m[3]);
	        		$order = '';

	        		if (!preg_match('/^(\s* \(?SELECT)(\s+ DISTINCT)? (.*?) (\s+ FROM \s) (.*) (\s ORDER \s+ BY \s+ .*)$/msix'.SB_PREG_MOD, $query[0], $m))
	        		{
	        			if (preg_match('/^(\s* \(?SELECT)(\s+ DISTINCT)? (.*?) (\s+ FROM \s) (.*)/msix'.SB_PREG_MOD, $query[0], $m))
	        			{
	        				$order = explode(',', $m[3]);
	        				$order = 'ORDER BY '.trim($order[0]);
	        			}
	        		}
	        		else
	        		{
	        			$order = $m[6];
	        		}

	        		if ($order != '')
	        		{
	        			$query[0] = 'SELECT TOP '.$count.' * FROM (
	        				SELECT'.$m[2].' ROW_NUMBER() OVER ('.$order.') AS RowNumber,'.($total ? ' TotalRows=Count(*) OVER(),' : '').'
	        				'.$m[3].' FROM '.$m[5].'
	        				) _tmpPageView
							WHERE RowNumber >= '.$offset;
	        		}
	        	}
	        	else
	        	{
	        		$count = intval($m[1]);
					$query[0] = preg_replace('/^\s* \(?SELECT(\s+ DISTINCT)?/msix'.SB_PREG_MOD, 'SELECT\\1 TOP '.$count, $query[0]);
	        	}
	        }
        }

        /**
         * Заменяем макросы в SQL-запросе реальными значениями
         */
    	if (count($query) > 1)
        {
        	$this->_expandPlaceholders($query);
        }

        /**
         * Замена для функции CONCAT
         */
        if (preg_match('/\s+ CONCAT \s* \( (.*?) \)\s+/msix'.SB_PREG_MOD, $query[0], $m))
        {
        	$m = explode(',', $m[1]);
        	$replace = ' '.implode('+', $m).' ';
        	$query[0] = preg_replace('/\s+ CONCAT \s* \( (.*?) \)\s+/msix'.SB_PREG_MOD, $replace, $query[0]);
        }

        /**
         * Выполняем запрос, если произошла ошибка, выводим ее и возвращаем FALSE
         */
        $time_start = microtime(true);
        $this->mQueryId = @sqlsrv_query($this->mLinkId, $query[0], array(), array('Scrollable' => SQLSRV_CURSOR_STATIC));
        $time_end = microtime(true);
        
        if (class_exists('sbProfiler'))
        {
        	sbProfiler::SQL($query[0]."\r\n", $time_end-$time_start);
        }
        
        if ( $this->mQueryId === false )
        {
            $this->halt($query[0]);
            return false;
        }

        if($this->mQueryType == 'SELECT' || $this->mQueryType == 'SHOW')
        {
            /**
             * Если тип запроса SELECT, то определяем кол-во записей, возвращенное запросом.
             * Если кол-во записей 0, то возвращаем FALSE.
             */
            $num_rows = @sqlsrv_num_rows($this->mQueryId);
            if (!$num_rows)
            {
                return false;
            }

            /**
             * Формируем двумерный массив с результатами запроса
             */
        	$ret = array();
        	$over_all = 0;
            if (!$assoc)
            {
                while (($row = @sqlsrv_fetch_array($this->mQueryId, SQLSRV_FETCH_NUMERIC)) != false)
                {
                    if (get_magic_quotes_runtime())
                    {
                        foreach ($row as $key => $value)
                        {
                            $row[$key] = stripslashes($value);
                        }
                    }

                    if (!is_null($offset))
                    {
                        array_shift($row);

                        if ($total)
                        {
                            $over_all = array_shift($row);
                        }
                    }

                    $ret[] = $row;
                }
            }
            else
            {
                while (($row = @sqlsrv_fetch_assoc($this->mQueryId, SQLSRV_FETCH_ASSOC)) != false)
                {
    	        if (get_magic_quotes_runtime())
    	        {
    	            foreach ($row as $key => $value)
    	            {
    	                $row[$key] = stripslashes($value);
    	            }
    	        }

    	        if (!is_null($offset))
    	        {
    	        	array_shift($row);

    	        	if ($total)
	    	        {
	    	        	$over_all = array_shift($row);
	    	        }
    	        }

    	        $ret[] = $row;
    	    }
            }

    	    /**
    	     * Для SELECT-запросов сразу освобождаем память, чтобы избежать переполнения при больших запросах
    	     */
    	    $this->free();
    	    $this->mQueryNumRows = $num_rows;

    	    if (empty($ret))
    	    {
    	        return false;
    	    }
    	    elseif ($total)
    	    {
    	    	if (is_null($offset))
    	    	{
    	    		$total = count($ret);
    	    	}
    	        else
    	        {
                    $total = $over_all;
    	        }
    	    }

    	    return $ret;
        }

    	if ($this->mQueryType == 'UPDATE' || $this->mQueryType == 'INSERT' || $this->mQueryType == 'DELETE')
        {
            /**
             * Для INSERT, UPDATE и DELETE запросов запоминаем кол-во затронутых записей
             */
            $this->mQueryNumRows = @sqlsrv_rows_affected ($this->mQueryId);
        }

        if ($this->mQueryType == 'INSERT')
        {
            /**
             * Для INSERT-запроса запоминаем сформированное значение для автоинкрементного поля
             */
	        $res = @sqlsrv_query($this->mLinkId, "SELECT @@IDENTITY AS 'Identity'", array(), array('Scrollable' => SQLSRV_CURSOR_STATIC));
		    if ($res)
		    {
		      	$ar = sqlsrv_fetch_array($res);
		        $this->mQueryInsertId = $ar[0];
		        $this->free($res);
		    }
        }

        return true;
    }

    /**
     * Заменяет макросы в запросе на реальные значения
     *
     * @param array $queryAndArgs Массив, первый элемент которого - текст SQL-запроса, остальные элементы -
     *                            значения макросов, использующихся в SQL-запросе
     */
    private function _expandPlaceholders(&$queryAndArgs)
    {
        if (!is_array($queryAndArgs))
        {
            $queryAndArgs = array($queryAndArgs);
        }

        $this->_mPlaceholderArgs = array_reverse($queryAndArgs);

        /**
         * Вытаскиваем запрос
         */
        $query = array_pop($this->_mPlaceholderArgs);

        /**
         * Заменяем макросы
         */
        $query = $this->_expandPlaceholdersFlow($query);

        $queryAndArgs = array($query);
    }

    /**
     * Функция замены макросов
     *
     * Перед использованием данной функции все значения макросов должны быть помещены в переменную
     * класса _mPlaceholderArgs. Данный метод может вызываться рекурсивно callback-методом <i>_expandPlaceholdersCallback()</i>.
     *
     * @param string $query Текст SQL-запроса
     *
     * @return string Запрос с реальными значениями вместо макросов
     */
    private function _expandPlaceholdersFlow($query)
    {
        $re = '{
            (?>
                (?>
                    "   (?> [^"\\\\]+|\\\\"|\\\\)*    "   |
                    \'  (?> [^\'\\\\]+|\\\\\'|\\\\)* \'   |
                    `   (?> [^`]+ | ``)*              `   |
                    /\* .*?                          \*/
                )
            )
              |
            (?>
                \{
                    ( (?> (?>[^{}]*)  |  (?R) )* )
                \}
            )
              |
            (?>
                (\?) ( [dsafn\#]? )
            )
        }sx'.SB_PREG_MOD;

        /**
         * Вызываем callback-функцию для замены макросов
         */
        $query = preg_replace_callback($re, array(&$this, '_expandPlaceholdersCallback'), $query);

        return $query;
    }

    /**
     * Возвращает кол-во макросов в запросе
     *
     * @param string $query Текст SQL-запроса
     *
     * @return int Кол-во найденных макросов
     */
    public function placeholdersCount($query)
    {
        $re = '{
            (?>
                (?>
                    "   (?> [^"\\\\]+|\\\\"|\\\\)*    "   |
                    \'  (?> [^\'\\\\]+|\\\\\'|\\\\)* \'   |
                    `   (?> [^`]+ | ``)*              `   |
                    /\* .*?                          \*/
                )
            )
              |
            (?>
                \{
                    ( (?> (?>[^{}]*)  |  (?R) )* )
                \}
            )
              |
            (?>
                (\?) ( [dsafn\#]? )
            )
        }sx'.SB_PREG_MOD;

        $ar = array();
        preg_match_all($re, $query, $ar);
        $count = 0;

        foreach ($ar[0] as $key => $value)
        {
            if (!empty($ar[2][$key]) || (isset($ar[1][$key]) && sb_strlen($ar[1][$key])))
                $count++;
        }

        return $count;
    }

    /**
     * Сallback-функция для замены макросов
     *
     * Вызывается в регулярном выражении метода <i>_expandPlaceholders()</i>.
     *
     * @param array $m Массив найденных значений регулярного выражения
     *
     * @return mixed Значение найденного макроса или текст ошибки ('SB_SQL_ERROR_NO_VALUE', 'SB_SQL_ERROR_VALUE_NOT_ARRAY'
     *               'SB_SQL_ERROR_ARRAY_VALUE_NOT_STRING', 'SB_SQL_ERROR_VALUE_NOT_SCALAR').
     */
    private function _expandPlaceholdersCallback($m)
    {
        if (!empty($m[2]))
        {
            /**
             * Содержит тип макроса
             */
            $type = $m[3];

            /**
             * Если значений для замены нет, возвращаем ошибку
             */
            if (!$this->_mPlaceholderArgs)
            {
                return 'SB_SQL_ERROR_NO_VALUE';
            }

            /**
             * Содержит значение для замены макроса
             */
            $value = array_pop($this->_mPlaceholderArgs);

            /**
             * Если значение равно SB_SQL_SKIP, то заменяем макрос на ''.
             */
            if ($value === SB_SQL_SKIP)
            {
                $this->_mPlaceholderNoValueFound = true;
                return '';
            }

            switch ($type)
            {
                /**
                 * Тип макроса
                 */
                case 'a':
                    /**
                     * ?a - массив значений или пар значений [имя поля] = значение
                     */
                    if (!is_array($value))
                    {
                        return 'SB_SQL_ERROR_VALUE_NOT_ARRAY';
                    }
                    $parts = array();
                    foreach ($value as $k=>$v)
                    {
                        $v = is_null($v) ? 'NULL' : $this->escape($v);
                        if (!is_int($k))
                        {
                            $k = $this->escape($k, true);
                            $parts[] = "$k=$v";
                        }
                        else
                        {
                            $parts[] = $v;
                        }
                    }

                    return join(', ', $parts);
                case "#":
                    /**
                     * ?# - название поля таблицы
                     */
                    if (!is_array($value))
                    {
                        return $this->escape($value, true);
                    }
                    $parts = array();
                    foreach ($value as $identifier)
                    {
                        if (!is_string($identifier))
                        {
                            return 'SB_SQL_ERROR_ARRAY_VALUE_NOT_STRING';
                        }
                        $parts[] = $this->escape($identifier, true);
                    }
                    return join(', ', $parts);
                case 'n':
                    /**
                     * ?n - целое число или NULL
                     */
                    return empty($value) ? 'NULL' : intval($value);
            }

            if ($value === null) return 'NULL';
            switch ($type)
            {
                /**
                 * Тип макроса
                 */
                case '':
                    /**
                     * ?
                     */
                    if (!is_scalar($value))
                    {
                        return 'SB_SQL_ERROR_VALUE_NOT_SCALAR';
                    }
                    return $this->escape($value);
                case 'd':
                    /**
                     * ?d
                     */
                    return intval($value);
                case 'f':
                    /**
                     * ?f
                     */
                    return str_replace(',', '.', floatval($value));
            }

            return $this->escape($value);
        }

        /**
         * Условный макрос
         */
        if (isset($m[1]) && sb_strlen($block=$m[1]))
        {
            $prev = @$this->_mPlaceholderNoValueFound;

            /**
             * Производим замену макросов внутри условного блока
             */
            $block = $this->_expandPlaceholdersFlow($block);

            $block = $this->_mPlaceholderNoValueFound ? '' : ' ' . $block . ' ';
            $this->_mPlaceholderNoValueFound = $prev;

            return $block;
        }

        return $m[0];
    }

    /**
     * Возвращает массив с названиями полей для указанной таблицы.
     *
     * @param string $table Имя таблицы
     *
     * @return array Массив с названиями полей
     */
    public function getFields($table)
    {
    	$return = false;
    	$res = $this->paramQuery('SELECT column_name FROM information_schema.columns WHERE table_name = ?', $table);
    	if ($res)
    	{
    		foreach ($res as $value)
    		{
    			$return[] = $value[0];
    		}
    	}

    	return $return;
    }

    /**
     * Возвращает версию MS SQL
     *
     * @return string Описание версии MS SQL.
     */
    public function getVersion()
    {
    	if ($this->mVersionInfo != '')
    		return $this->mVersionInfo;

    	return 'Error';
    }

	/**
     * Возвращает количество SQL-запросов
     *
     * @return int Количество SQL-запросов.
     */
    public function getSqlCount()
    {
    	return $this->mAllQueryNum;
    }

    /**
     * Возвращает список таблиц текущей БД
     *
     * @return array Список таблиц.
     */
    public function showTables()
    {
        $total = 0;
        $result = $this->query(array("SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE = 'BASE TABLE'"), $total);

        $tables = array();

        if ($result)
        {
            foreach ($result as $value)
            {
                $tables[] = $value[0];
            }
        }

        return $tables;
    }
}

?>