<?php
/**
 * Реализация класса работы с закладками
 *
 * Во многих диалоговых окнах системы используются закладки, как загружаемые сразу, так и подгружаемые через AJAX по мере
 * необходимости.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

/**
 * Класс для работы с закладками
 *
 * Во многих диалоговых окнах системы используются закладки, как загружаемые сразу, так и подгружаемые через AJAX по мере
 * необходимости. Данный класс реализует такой механизм. Принцип работы с классом следующий:
 * <code>
 * // статические закладки
 * $tabStr1 = '<b>Содержимое закладки 1</b>';
 * $tabStr2 = '<b>Содержимое закладки 2</b>';
 *
 * $tabs = new sbTabs('action.php', '', 'post', '');
 * $tabs->setTop('<script src="javascript.js"></script>');
 * $tabs->setBottom('<input type="submit" value="Сохранить">');
 *
 * $tabs->addTab('Закладка 1', $tabStr1);
 * $tabs->addTab('Закладка 2', $tabStr2);
 * $tabs->show();
 * </code>
 *
 * <code>
 * // динамически-подгружаемые закладки
 * $tabId = (isset($_GET['tab_id']) ? intval($_GET['tab_id']) : -1);
 *
 * if ($tabId == 0)
 *     $tabStr1 = '<b>Содержимое закладки 1</b>';
 * else
 *     $tabStr1 = '';
 *
 * if ($tabId == 1)
 *     $tabStr2 = '<b>Содержимое закладки 2</b>';
 * else
 *     $tabStr2 = '';
 *
 * $tabs = new sbTabs('action.php', '', 'post', '');
 * $tabs->setTop('<script src="javascript.js"></script>');
 * $tabs->setBottom('<input type="submit" value="Сохранить">');
 * $tabs->setAutoLoading($_SERVER['PHP_SELF']);
 *
 * $tabs->addTab('Закладка 1', $tabStr1);
 * $tabs->addTab('Закладка 2', $tabStr2);
 *
 * $tabs->show();
 * </code>
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_Layout
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbTabs
{
	/**
	 * Массив, содержащий информацию о закладках
	 *
	 * <code>
	 * $mTabs[0] = array();
	 * // название закладки
	 * $mTabs[0]['title'] = ...;
	 * // содержимое закладки
	 * $mTabs[0]['content'] = ...;
	 * // показывать (true) или не показывать (false) закладку
	 * $mTabs[0]['show'] = true;
	 * </code>
	 *
	 * @var array
	 */
	private $mTabs = array();

	/**
	 * action формы отправки данных
	 *
	 * @var string
	 */
	private $mFormAction = '';

	/**
	 * target формы отправки данных
	 *
	 * Задается в конструкторе класса.
	 *
	 * @var string
	 */
	private $mFormTarget = '';

	/**
	 * method формы отправки данных
	 *
	 * Задается в конструкторе класса.
	 *
	 * @var string
	 */
	private $mFormMethod = '';

	/**
	 * JavaScript-функция, вызываемая перед отправкой формы
	 *
	 * Обычно используется для проверки введенных пользователем данных, если возвращает false, то отправка данных не происходит.
	 * Задается в конструкторе класса.
	 *
	 * @var string
	 */
	private $mFormOnSubmit = '';

	/**
	 * JavaScript-функция, вызываемая после загрузки содержимого закладки
	 *
	 * @var string
	 */
	public $mTabsOnLoad = '';
	
	/**
	 * JavaScript-функция, вызываемая после загрузки и изменения размеров контейнера с закладками
	 *
	 * @var string
	 */
	public $mOnLoad = '';

	/**
	 * Дополнительные атрибуты формы отправки данных
	 *
	 * Здесь можно указать, например, стили формы. Задается в конструкторе класса.
	 *
	 * @var string
	 */
	private $mFormDopStr = '';

	/**
	 * HTML-код, выводимый над формой отправки данных
	 *
	 * Обычно содержит JavaScript-функции, необходимые для работы формы.
	 * Устанавливается методом setTop.
	 *
	 * @var string
	 */
	private $mTop = '';

    /**
     * HTML-код, выводимый внизу закладок
     *
     * Обычно содержит кнопки отправки формы и закрытия диалогового окна.
     * Устанавливается методов setBottom.
     *
     * @var string
     */
	private $mBottom = '';

    /**
     * Событие и параметры, вызываемое для подгрузки закладок через AJAX
     *
     * Данное событие должно возвращать HTML-код подгружаемой закладки. ID закладки передается событию
     * через $_GET['tab_id'].
     *
     * @var string
     */
	private $mAutoLoadingURL = '';

	/**
	 * Конструктор класса
	 *
	 * @param string $form_action action формы отправки данных. Если не указан, то форма выведена не будет.
	 * @param string $form_target target формы отправки данных
	 * @param string $form_method method формы отправки данных
	 * @param string $form_on_submit JavaScript-функция, вызываемая перед отправкой формы
	 * @param string $form_dop_str Дополнительные атрибуты формы отправки данных.Здесь можно указать, например, стили формы.
	 *                             Задается в конструкторе класса.
	 */
	public function __construct($form_action='', $form_target='thisDialog', $form_method='post', $form_on_submit='checkValues()', $form_dop_str='')
	{
		$this->mFormAction   = $form_action;
		if ($form_target == 'thisDialog')
        {
        	if (browser_get_agent() != 'IE')
        		$form_target = '_self';	
        }
        
		$this->mFormTarget   = $form_target;
		$this->mFormMethod   = $form_method;
		$this->mFormOnSubmit = $form_on_submit;
		$this->mFormDopStr   = $form_dop_str;
	}

	/**
	 * Устанавливает событие и параметры, вызываемое для подгрузки закладок через AJAX
	 *
	 * Данное событие должно возвращать HTML-код подгружаемой закладки. ID закладки передается событию
     * через $_GET['tab_id'].
	 *
	 * @param string $url Событие и параметры.
	 */
	public function setAutoLoading($url)
	{
	   $this->mAutoLoadingURL = $url;
	}

	/**
     * Устанавливает HTML-код, выводимый внизу закладок
     *
     * Обычно данный код содержит кнопки отправки формы и закрытия диалогового окна.
     *
     * @param string HTML-код
     */
	public function setBottom($str)
	{
	    $this->mBottom = $str;
	}

	/**
	 * Устанавливает HTML-код, выводимый над формой отправки данных
	 *
	 * Обычно данный код содержит JavaScript-функции, необходимые для работы формы.
	 *
	 * @param string HTML-код
	 */
	public function setTop($str)
	{
	    $this->mTop = $str;
	}

	/**
	 * Добавляет закладку
	 *
	 * @param string $title Название закладки.
	 * @param string $content Содержимое закладки. При подгрузке через AJAX указывается содержимое только для подгружаемой закладки.
	 * @param string $show Показывать (true) или не показывать (false) закладку.
	 */
	public function addTab($title, $content='', $show = true)
	{
	    $i = count($this->mTabs);
		$this->mTabs[$i] = array();
		$this->mTabs[$i]['title'] = $title;
		$this->mTabs[$i]['content'] = $content;
		$this->mTabs[$i]['show'] = $show;
	}

	/**
	 * Выводит закладки и их содержимое
	 *
	 * Eсли параметр $echo установлен в false, то вывод в браузер не осуществляется.
	 *
	 * @param boolean $echo Если true, то зыкладки выводятся в браузер пользователя.
	 * @return string HTML-код закладок.
	 */
	function show($echo = true)
	{
	    if (count($this->mTabs) == 0)
		{
		    // закладок нет
		    return '';
		}

		if ($this->mAutoLoadingURL != '' && isset($_GET['tab_id']))
		{
		    // если идет подгрузка, то возвращаем только содержимое подгружаемой закладки
		    $tab_id = intval($_GET['tab_id']);
		    if ($echo)
		    {
		        echo $this->mTabs[$tab_id]['content'];
		        return $this->mTabs[$tab_id]['content'];
		    }
		    else
		    {
		        return $this->mTabs[$tab_id]['content'];
		    }
		}

		$tabidx = (isset($_REQUEST['tabidx']) ? intval($_REQUEST['tabidx']) : 0);

		$result = '<script src="'.SB_CMS_JSCRIPT_URL.'/sbTabs.js.php"></script>
			'.($this->mTop != '' ? '<div id="sb_tabs_top">'.$this->mTop.'</div>' : '');

		if ($this->mFormAction != '')
		{
            $result .= '<form style="height:100%;" '.($this->mFormOnSubmit != '' ? 'onsubmit="return '.$this->mFormOnSubmit.'"' : '').' '.($this->mFormTarget != '' ? 'target="'.$this->mFormTarget.'"' : '').' action="'.$this->mFormAction.'" method="'.$this->mFormMethod.'"'.($this->mFormDopStr != '' ? ' '.$this->mFormDopStr:'').'>';
		}

		$result .= '<div id="sb_tabs">
		    <table cellpadding="0" cellspacing="0" id="sb_tabs_table" align="center">
            <tr>
                <td valign="top" align="left" height="1" id="sb_tabs_tabs">
                    <script>
                    '.($this->mOnLoad != '' ? 'sbTabsLoadFunc = '.$this->mOnLoad.';' : '').'
                    '.($this->mTabsOnLoad != '' ? 'window.sbTabsTabLoadFunc = '.$this->mTabsOnLoad.';' : '').'
                    
                    document.body.style.overflow = "hidden";
                    sbGetE("event_content").style.overflow = "hidden";
                    
                    window.sbTabClick = function(i)
                    {
                        sbGetE("tabidx").value=i;
                    }
                    
                    window.tabs = new sbTabs("tabs");
                    window.tabs.imgPath = "'.SB_CMS_IMG_URL.'/tabs/";
                    window.tabs.blankImage = "'.SB_CMS_IMG_URL.'/blank.gif";
                    window.tabs.selectedItem = '.$tabidx.';
                    window.tabs.clickFunc = sbTabClick;
                    window.tabs.loadingText = "'.str_replace('"', '\\"', sb_show_message(KERNEL_LOADING, true, 'loading', true)).'";
                    window.tabs.setAutoLoading("'.$this->mAutoLoadingURL.'");
                    ';

        for ($i = 0; $i < count($this->mTabs); $i++)
        {
            $result .= 'window.tabs.addTab("'.str_replace('"', '\\"', $this->mTabs[$i]['title']).'", "sb_tab'.$i.'", "", '.($this->mTabs[$i]['show'] ? 'true' : 'false').'); ';
        }

        $result .= 'window.tabs.show(tabs);
                    </script>
                </td>
            </tr>
		    <tr>
                <td id="sb_tabs_con_td">
                <div id="sb_tabs_con" style="overflow: hidden;">';
        for ($i = 0; $i < count($this->mTabs); $i++)
		{
			$result .= '<div id="sb_tab'.$i.'">'.($this->mAutoLoadingURL == '' ? $this->mTabs[$i]['content'] : '').'</div>';
		}

        $result .= '</div>'.($this->mBottom != '' ? '<div id="sb_tabs_bottom">'.$this->mBottom.'</div>':'').'</td>
            </tr>
            </table></div>
            <input type="hidden" name="tabidx" id="tabidx" value="0">';

        if ($this->mFormAction != '')
            $result .= '</form>';

        if ($echo)
        {
            echo $result;
            return $result;
        }
        else
        {
            return $result;
        }
	}
}
?>