<?php
/**
 * Реализация универсального класса, отвечающего за работу с деревом разделов, построенным по принципу left-right.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_DB
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

/**
 * Универсальный класс, отвечающий за работу с деревом разделов, построенным по принципу left-right.
 *
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_DB
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbTree
{
    /**
     * Идентификатор дерева раздела
     *
     * В одной таблице может хранится несколько деревьев.
     *
     * @var string
     */
    private $mIdent = '';

    /**
     * Новый идентификатор дерева раздела
     *
     * Указывается, когда мы хотим перетащить ветки одного дерева в другое.
     *
     * @var string
     */
    private $mNewIdent = '';
    
    /**
     * Название таблицы, хранящей дерево
     *
     * @var string
     */
    private $mTable = 'sb_categs';

    /**
     * Название поля, хранящего атрибут <i>right</i> дерева
     *
     * @var string
     */
    private $mRightFld = 'cat_right';

    /**
     * Название поля, хранящего атрибут <i>left</i> дерева
     *
     * @var string
     */
    private $mLeftFld = 'cat_left';

    /**
     * Название поля, хранящего атрибут <i>level</i> дерева
     *
     * @var string
     */
    private $mLevelFld = 'cat_level';

    /**
     * Название автоинкриментного поля, хранящего ID раздела
     *
     * @var string
     */
    private $mIdFld = 'cat_id';

    /**
     * Название поля, хранящего идентификатор раздела
     *
     * @var string
     */
    private $mIdentFld = 'cat_ident';

    /**
     * Конструктор класса
     *
     * @param string $ident Идентификатор дерева разделов. В одной таблице может хранится несколько деревьев.
     */
    public function __construct($ident, $new_ident = '')
    {
        $this->mIdent = $ident;
        $this->mNewIdent = $new_ident;
    }

    /**
     * Установка названий таблицы и полей дерева
     *
     * @param string $table Название таблицы, хранящей дерево.
     * @param string $right Название поля, хранящего атрибут <i>right</i> дерева.
     * @param string $left Название поля, хранящего атрибут <i>left</i> дерева.
     * @param string $level Название поля, хранящего атрибут <i>level</i> дерева.
     * @param string $id Название автоинкриментного поля, хранящего ID раздела.
     * @param string $ident Название поля, хранящего идентификатор раздела.
     */
    public function setSettings($table, $right, $left, $level, $id, $ident)
    {
        $this->mTable = $table;
        $this->mRightFld = $right;
        $this->mLeftFld = $left;
        $this->mLevelFld = $level;
        $this->mIdFld = $id;
        $this->mIdentFld = $ident;
    }

    /**
     * Добавляет новый подраздел
     *
     * @param int $par_id ID родительского раздела.
     * @param array $fields Ассоциативный массив, хранящий значения других полей дерева,
     *                      за исключением перечисленных в методе setSettings. Ключ массива - название поля.
     *
     * @return int ID нового раздела.
     */
    public function insertNode($par_id, $fields)
    {
        if ($par_id != 0)
        {
            $res = sql_query('SELECT '.$this->mRightFld.', '.$this->mLevelFld.' FROM '.$this->mTable.' WHERE '.$this->mIdFld.'=?d', $par_id);
            if (!$res)
                return false;
    
            list($right, $level) = $res[0];
    
            sql_query('LOCK TABLES '.$this->mTable.' WRITE');
            sql_query('UPDATE '.$this->mTable.' SET '.$this->mRightFld.'='.$this->mRightFld.' + 2 WHERE '.$this->mIdentFld.'=? AND '.$this->mRightFld.' >= ?d', $this->mIdent, $right);
            sql_query('UPDATE '.$this->mTable.' SET '.$this->mLeftFld.'='.$this->mLeftFld.' + 2 WHERE '.$this->mIdentFld.'=? AND '.$this->mLeftFld.' > ?d', $this->mIdent, $right);
    
            $level++;
        }
        else 
        {
            $right = 0;
            $level = 0;
        }
        
        sql_query('INSERT INTO '.$this->mTable.' (?#, ?#, ?#, ?#, ?#) VALUES (?a, ?, ?d, ?d, ?d)', array_keys($fields), $this->mIdentFld, $this->mLeftFld, $this->mRightFld, $this->mLevelFld, array_values($fields), $this->mIdent, $right, ($right + 1), $level);

        $id = sql_insert_id();
        sql_query('UNLOCK TABLES');

        return $id;
    }

    /**
     * Удаляет раздел и все его подразделы
     *
     * @param int $id ID удаляемого раздела.
     *
     * @return bool TRUE, если раздел удален успешно, и FALSE в ином случае.
     */
    public function removeNode($id)
    {
        $res = sql_query('SELECT '.$this->mRightFld.', '.$this->mLeftFld.' FROM '.$this->mTable.' WHERE '.$this->mIdFld.'=?d', $id);
        if (!$res)
            return false;

        list($right, $left) = $res[0];

        sql_query('LOCK TABLES '.$this->mTable.' WRITE');
        $res = sql_query('SELECT COUNT(*) FROM '.$this->mTable.' WHERE '.$this->mLeftFld.' >= ?d AND '.$this->mRightFld.' <= ?d AND '.$this->mIdentFld.' = ?', $left, $right, $this->mIdent);
        list($num) = $res[0];

        $decr = $num * 2;
        sql_query('DELETE FROM '.$this->mTable.' WHERE '.$this->mLeftFld.' >= ?d AND '.$this->mRightFld.' <= ?d AND '.$this->mIdentFld.' = ?', $left, $right, $this->mIdent);
        sql_query('UPDATE '.$this->mTable.' SET '.$this->mRightFld.' = '.$this->mRightFld.' - '.$decr.' WHERE '.$this->mIdentFld.' = ? AND '.$this->mRightFld.' >= ?d', $this->mIdent, $right);
        sql_query('UPDATE '.$this->mTable.' SET '.$this->mLeftFld.' = '.$this->mLeftFld.' - '.$decr.' WHERE '.$this->mIdentFld.' = ? AND '.$this->mLeftFld.' > ?d', $this->mIdent, $right);
        sql_query('UNLOCK TABLES');

        return true;
    }

    /**
     * Вставляет вырезанный или скопированный раздел
     *
     * @param  int $par_id ID раздела, в который производится вставка скопированного или вырезанного раздела.
     * @param  int $paste_id ID скопированного или вырезанного раздела.
     * @param  string $action CUT - раздел был вырезан, COPY - раздел был скопирован.
     * @param  int $before_id ID раздела, находящегося в иерархии дерева непосредственно под разделом, в который производится вставка.
     * @return bool TRUE, если раздел вставлен успешно, FALSE - в ином случае.
     */
    public function pasteNode($par_id, $paste_id, $action='cut', $before_id=-1)
    {
        sql_query('LOCK TABLES '.$this->mTable.' WRITE');
        if ($action == 'cut')
        {
            $res = sql_query('SELECT '.$this->mRightFld.', '.$this->mLeftFld.', '.$this->mLevelFld.' FROM '.$this->mTable.' WHERE '.$this->mIdFld.' = ?d', $paste_id);
            if (!$res)
            {
                sql_query('UNLOCK TABLES');
                return false;
            }

            list($old_right, $old_left, $old_level) = $res[0];

            $res = sql_query('SELECT '.$this->mIdFld.', '.$this->mLevelFld.' FROM '.$this->mTable.' WHERE '.$this->mIdentFld.' = ? AND '.$this->mLeftFld.' >= ?d AND '.$this->mRightFld.' <= ?d ORDER BY '.$this->mLeftFld.' ASC', $this->mIdent, $old_left, $old_right);
            if (!$res)
            {
                sql_query('UNLOCK TABLES');
                return false;
            }

            $num = count($res);
            if ($old_level == 0)
            {
                $num--;
                $old_level = 1;
            }

            // удаляем вырезаему ветку
            sql_query('UPDATE '.$this->mTable.' SET '.$this->mRightFld.' = '.$this->mRightFld.' - '.($num * 2).' WHERE '.$this->mIdentFld.' = ? AND '.$this->mRightFld.' >= ?d', $this->mIdent, $old_right);
            sql_query('UPDATE '.$this->mTable.' SET '.$this->mLeftFld.' = '.$this->mLeftFld.' - '.($num * 2).' WHERE '.$this->mIdentFld.' = ? AND '.$this->mLeftFld.' > ?d', $this->mIdent, $old_right);

            if ($before_id > 0)
            {
                $result = sql_query('SELECT '.$this->mRightFld.', '.$this->mLeftFld.', '.$this->mLevelFld.' FROM '.$this->mTable.' WHERE '.$this->mIdFld.' = ?d', $before_id);
                list($new_right, $new_left, $new_level) = $result[0];

                $levelIncr = $old_level - $new_level;
                $cat_left = $new_left - 2;
                $cat_right = -1;
                $cur_level = $new_level;

                // раздвигаем ветки дерева, чтобы вместить вставляемые
                sql_query('UPDATE '.$this->mTable.' SET '.$this->mRightFld.' = '.$this->mRightFld.' + '.($num * 2).' WHERE '.$this->mIdentFld.' = ? AND '.$this->mRightFld.' > ?d', $this->mNewIdent != '' ? $this->mNewIdent : $this->mIdent, $new_right);
                sql_query('UPDATE '.$this->mTable.' SET '.$this->mLeftFld.' = '.$this->mLeftFld.' + '.($num * 2).' WHERE '.$this->mIdentFld.' = ? AND '.$this->mLeftFld.' > ?d', $this->mNewIdent != '' ? $this->mNewIdent : $this->mIdent, $new_right);
                sql_query('UPDATE '.$this->mTable.' SET '.$this->mLeftFld.' = '.$this->mLeftFld.' + '.($num * 2).', '.$this->mRightFld.' = '.$this->mRightFld.' + '.($num * 2).' WHERE '.$this->mIdentFld.' = ? AND '.$this->mLeftFld.' >= ?d AND '.$this->mRightFld.' <= ?d', $this->mNewIdent != '' ? $this->mNewIdent : $this->mIdent, $new_left, $new_right);
            }
            else
            {
                $result = sql_query('SELECT '.$this->mRightFld.', '.$this->mLeftFld.', '.$this->mLevelFld.' FROM '.$this->mTable.' WHERE '.$this->mIdFld.' = ?d', $par_id);
                list($new_right, $new_left, $new_level) = $result[0];

                $levelIncr = $old_level - $new_level - 1;
                $cat_left = $new_right - 2;
                $cat_right = -1;
                $cur_level = $new_level + 1;

                // раздвигаем ветки дерева, чтобы вместить вставляемые
                sql_query('UPDATE '.$this->mTable.' SET '.$this->mRightFld.' = '.$this->mRightFld.' + '.($num * 2).' WHERE '.$this->mIdentFld.' = ? AND '.$this->mRightFld.' >= ?d', $this->mNewIdent != '' ? $this->mNewIdent : $this->mIdent, $new_right);
                sql_query('UPDATE '.$this->mTable.' SET '.$this->mLeftFld.' = '.$this->mLeftFld.' + '.($num * 2).' WHERE '.$this->mIdentFld.' = ? AND '.$this->mLeftFld.' > ?d', $this->mNewIdent != '' ? $this->mNewIdent : $this->mIdent, $new_right);
            }

            $i = 0;
            $old_cat_id = -1;
            $cat_ids = array();
			$cat_levels = array();
			
            foreach ($res as $value)
            {
                list($cat_id, $cat_level) = $value;
                if ($cat_level == 0)
                {
                    continue;
                }

                $cat_level = $cat_level - $levelIncr;
                if ($cat_level > $cur_level)
                {
                    $cat_ids[] = $old_cat_id;
                    $cat_levels[] = $cur_level;
                    
                    $cur_level = $cat_level;
                    $cat_left++;
                    $cat_right++;
                }
                elseif ($cat_level < $cur_level)
                {
                	$prev_level = array_pop($cat_levels);
                    $prev_id = array_pop($cat_ids);
                        
                    while (!is_null($prev_level) && $prev_level >= $cat_level)
                    {
                        $cat_right++;
                        sql_query('UPDATE '.$this->mTable.' SET '.$this->mRightFld.' = '.$cat_right.', '.$this->mIdentFld.' = \''.($this->mNewIdent != '' ? $this->mNewIdent : $this->mIdent).'\' WHERE '.$this->mIdFld.' = '.$prev_id);
                        
                        $prev_level = array_pop($cat_levels);
                        $prev_id = array_pop($cat_ids);
                    }
                        
                    if (!is_null($prev_level))
                    {
                        $cat_levels[] = $prev_level;
                        $cat_ids[] = $prev_id;
                    }
                    
                    $cur_level = $cat_level;
                    $cat_left = $cat_right + 1;
                    $cat_right += 2;
                }
                else
                {
                    $cat_left += 2;
                    $cat_right = $cat_left + 1;
                }

                sql_query('UPDATE '.$this->mTable.' SET '.$this->mLeftFld.' = '.$cat_left.', '.$this->mRightFld.' = '.$cat_right.', '.$this->mLevelFld.' = '.$cat_level.', '.$this->mIdentFld.' = '.$GLOBALS['sbSql']->escape($this->mNewIdent != '' ? $this->mNewIdent : $this->mIdent).' WHERE '.$this->mIdFld.' = '.$cat_id);
                $old_cat_id = $cat_id;
                $i++;
            }

            while (($cat_id = array_pop($cat_ids)) != false)
            {
                $cat_right++;
                sql_query('UPDATE '.$this->mTable.' SET '.$this->mRightFld.' = '.$cat_right.', '.$this->mIdentFld.' = '.$GLOBALS['sbSql']->escape($this->mNewIdent != '' ? $this->mNewIdent : $this->mIdent).' WHERE '.$this->mIdFld.' = '.$cat_id);
            }
            sql_query('UNLOCK TABLES');
            return true;
        }
        elseif ($action == 'copy')
        {
            $res = sql_query('SELECT '.$this->mRightFld.', '.$this->mLeftFld.', '.$this->mLevelFld.' FROM '.$this->mTable.' WHERE '.$this->mIdFld.' = ?d', $paste_id);
            if (!$res)
            {
                sql_query('UNLOCK TABLES');
                return false;
            }
            list($old_right, $old_left, $old_level) = $res[0];

            $res = sql_query('SELECT '.$this->mRightFld.', '.$this->mLeftFld.', '.$this->mLevelFld.' FROM '.$this->mTable.' WHERE '.$this->mIdFld.' = ?d', $par_id);
            if (!$res)
            {
                sql_query('UNLOCK TABLES');
                return false;
            }
            list($new_right, $new_left, $new_level) = $res[0];

            $columns = $GLOBALS['sbSql']->getFields($this->mTable);
            
            $res = sql_query('SELECT '.implode(',', $columns).' FROM '.$this->mTable.' WHERE '.$this->mIdentFld.' = '.$GLOBALS['sbSql']->escape($this->mIdent).' AND '.$this->mLeftFld.' >= '.$old_left.' AND '.$this->mRightFld.' <= '.$old_right.' ORDER BY '.$this->mLeftFld.' ASC');

            if ($res)
            {
                $num = count($res);
					
                if ($old_level == 0)
                {
                    if ($num == 1)
                    {
                        sql_query('UNLOCK TABLES');
                        return false;
                    }
                    $num--;
                    $old_level = 1;
                }

                $levelIncr = $old_level - $new_level - 1;

                // раздвигаем ветки дерева, чтобы вместить вставляемые
                sql_query('UPDATE '.$this->mTable.' SET '.$this->mRightFld.' = '.$this->mRightFld.' + '.($num * 2).' WHERE '.$this->mIdentFld.' = ? AND '.$this->mRightFld.' >= ?d', $this->mNewIdent != '' ? $this->mNewIdent : $this->mIdent, $new_right);
                sql_query('UPDATE '.$this->mTable.' SET '.$this->mLeftFld.' = '.$this->mLeftFld.' + '.($num * 2).' WHERE '.$this->mIdentFld.' = ? AND '.$this->mLeftFld.' > ?d', $this->mNewIdent != '' ? $this->mNewIdent : $this->mIdent, $new_right);

                $i = 0;
                $cur_level = $new_level + 1;
                $cat_left = $new_right - 2;
                $cat_right = -1;
                $cat_ids = array();
                $cat_levels = array();
                $return_res = '';

                foreach ($res as $value)
                {
                	$tmp = array();
                	foreach ($value as $k => $v)
                	{
                		$tmp[$columns[$k]] = $v;	
                	}
                	$value = $tmp;
                	
                    $cat_level = $value[$this->mLevelFld];
                    $old_cat_id = $value[$this->mIdFld];

                    if ($cat_level == 0)
                    {
                        continue;
                    }

                    $cat_level = $cat_level - $levelIncr;
                    if ($cat_level > $cur_level)
                    {
                        $cat_ids[] = $cat_id;
                        $cat_levels[] = $cur_level;
                        
                        $cur_level = $cat_level;
                        $cat_left++;
                        $cat_right++;
                    }
                    elseif ($cat_level < $cur_level)
                    {
                    	$prev_level = array_pop($cat_levels);
                        $prev_id = array_pop($cat_ids);
                        
                        while (!is_null($prev_level) && $prev_level >= $cat_level)
                        {
                        	$cat_right++;
                        	sql_query('UPDATE '.$this->mTable.' SET '.$this->mRightFld.' = '.$cat_right.' WHERE '.$this->mIdFld.' = '.$prev_id);
                        	
                        	$prev_level = array_pop($cat_levels);
                        	$prev_id = array_pop($cat_ids);
                        }
                        
                        if (!is_null($prev_level))
                        {
                        	$cat_levels[] = $prev_level;
                        	$cat_ids[] = $prev_id;
                        }
                        
                        $cur_level = $cat_level;
                        $cat_left = $cat_right + 1;
                        $cat_right += 2;
                    }
                    else
                    {
                        $cat_left += 2;
                        $cat_right = $cat_left + 1;
                    }

                    unset($value[$this->mIdFld]);

                    $value[$this->mLeftFld] = $cat_left;
                    $value[$this->mRightFld] = $cat_right;
                    $value[$this->mLevelFld] = $cat_level;

                    if ($this->mNewIdent != '' && $this->mNewIdent != $this->mIdent)
                    {
                        $value[$this->mIdentFld] = $this->mNewIdent;
                    }
                    // Если $this->mIdent - число - это идентификатор раздела для меню
                    if(!is_numeric($this->mIdent))
                    	$value['cat_url'] = sb_check_chpu(0, $value['cat_url'], $value['cat_title'], 'sb_categs', 'cat_url', 'cat_id');
                    sql_query('INSERT INTO '.$this->mTable.' (?#) VALUES (?a)', array_keys($value), array_values($value));
                    $cat_id = sql_insert_id();
                    $return_res .= $old_cat_id.'::'.$cat_id.'|';
                    $i++;
                }
                
                while (($cat_id = array_pop($cat_ids)) != false)
                {
                    $cat_right++;
                    sql_query('UPDATE '.$this->mTable.' SET '.$this->mRightFld.' = '.$cat_right.' WHERE '.$this->mIdFld.' = '.$cat_id);
                }
                
                if ($return_res != '')
                {
                    $return_res = substr($return_res, 0, -1);
                    sql_query('UNLOCK TABLES');
                    return $return_res;
                }
                else
                {
                    sql_query('UNLOCK TABLES');
                    return false;
                }
            }
            else
            {
                sql_query('UNLOCK TABLES');
                return false;
            }
        }

        sql_query('UNLOCK TABLES');
        return false;
    }
}
?>