<?php
/**
 * Реализация класса работы с файловой системой сервера
 *
 * Данный класс работает с файловой системой сервера либо локально, либо через FTP в зависимости от настроек системы (config.inc.php).
 * После обявления класса вызывается конструктор класса и ссылка на класс сохраняется в глобальной переменной $GLOBALS['sbVfs'].
 *
 * @see config.inc.php
 * @see $GLOBALS['sbVfs']
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_FileSystem
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */

define ('SB_VFS_SCANDIR_ORDER_NAME_ASC', 1);
define ('SB_VFS_SCANDIR_ORDER_NAME_DESC', 2);
define ('SB_VFS_SCANDIR_ORDER_SIZE_ASC', 3);
define ('SB_VFS_SCANDIR_ORDER_SIZE_DESC', 4);
define ('SB_VFS_SCANDIR_ORDER_TIME_ASC', 5);
define ('SB_VFS_SCANDIR_ORDER_TIME_DESC', 6);

/**
 * Класс работы с файловой системой сервера
 *
 * Данный класс работает с файловой системой сервера либо локально, либо через FTP в зависимости от настроек системы (config.inc.php).
 * После обявления класса вызывается конструктор класса и ссылка на класс сохраняется в глобальной переменной $GLOBALS['sbVfs'].
 *
 * @see config.inc.php
 * @see $GLOBALS['sbVfs']
 * @author Казбек Елекоев <elekoev@binn.ru>
 * @version 4.0
 * @package SB_FileSystem
 * @copyright Copyright (c) 2007, OOO "СИБИЭС Групп"
 */
class sbVfs
{
    /**
     * Права на создаваемые файлы
     *
     * Восьмиричное число. Например, для установки прав 755, присвойте данной переменной значение octdec(755);
     *
     * @var octal
     */
    public  $mFileMode;

    /**
     * Права на создаваемые директории
     *
     * Восьмиричное число. Например, для установки прав 755, присвойте данной переменной значение octdec(755);
     *
     * @var octal
     */
    public  $mDirMode;

    /**
     * Дескриптор последней открытой c помощью метода opendir() директории
     *
     * @var resource
     */
    public  $mDirHandle = false;

    /**
     * Дескриптор последнего открытого c помощью метода fopen() файла
     *
     * @var resource
     */
    public  $mFileHandle = false;

    /**
     * Ведется работа с локальной файловой системой или с удаленной
     *
     * @var bool
     */
    public  $mLocal = false;

    /**
     * Домен, для которого используется соединение
     *
     * @var string
     */
    public  $mDomain = '';

    /**
     * Хост и порт для FTP
     *
     * Например <samp>ftp.sb40.ru:21</samp>. Порт указывать необязательно. По умолчанию работа идет черз порт 21.
     * Для локальной работы с файловой системой необходимо указать <samp>local</samp>.
     *
     * @var string
     */
    private $mFtpHost;

    /**
     * Абсолютный путь до рутовой директории FTP-аккаунта
     *
     * Указывается полностью. Например <samp>C:/Internet/www/sbuilder/4.0</samp>
     * для Windows-серверов или <samp>/home/sbuilder</samp> для Unix-серверов.
     * Не содержит / в конце.
     *
     * @var string
     */
    private $mFtpBasedir;

    /**
     * Имя пользователя для FTP
     *
     * Указывается имя пользователя для подключения к FTP-аккаунту.
     *
     * @var string
     */
    private $mFtpUser;

    /**
     * Пароль для FTP
     *
     * Указывается пароль для подключения к FTP-аккаунту.
     *
     * @var string
     */
    private $mFtpPassword;

    /**
     * Массив, хранящий параметры считанных методом scandir() файлов и каталогов
     *
     * Только при использовании расширения PHP cURL.
     *
     * @var array
     */
    private $mParams = array();

    /**
     * Массив, хранящий список считанных методом scandir() файлов и каталогов
     *
     * @var array
     */
    private $mList = array();

    /**
     * Конструктор класса
     *
     * Переменные класса mFileMode и mDirMode получают значения octdec(755).
     *
     * @see $mFileMode
     * @see $mDirMode
     *
     * @param string $ftp_host Хост и порт для FTP (local для локальной работы)
     * @param string $ftp_basedir Абсолютный путь до рутовой директории FTP-аккаунта
     * @param string $ftp_user Имя пользователя для FTP
     * @param string $ftp_password Пароль для FTP
     */
    public function __construct($ftp_host, $ftp_basedir, $ftp_user = array(), $ftp_password = array())
    {
        $this->mFileMode = octdec(644);
        $this->mDirMode = octdec(755);

        $this->mFtpHost = $ftp_host;
        $this->mFtpBasedir = $ftp_basedir;
        $this->mFtpUser = $ftp_user;
        $this->mFtpPassword = $ftp_password;

        $this->mDomain = defined('SB_KEY_DOMAIN') ? SB_KEY_DOMAIN : SB_COOKIE_DOMAIN;

        if (!isset($GLOBALS['sb_domains'][$this->mDomain]))
        {
        	foreach ($GLOBALS['sb_domains'] as $key => $value)
   			{
   				if(isset($value['pointers']) && in_array($this->mDomain, $value['pointers']))
   				{
       			 	$this->mDomain = $key;
       			 	break;
   				}
    		}
        }
    }

    /**
     * Открывает директорию для последующих операций с ней
     *
     * @param string $path Путь к директории, которую следует открыть.
     * @return resource Дескриптор открытой директории или FALSE.
     */
    public function opendir($path)
    {
        /**
         * Преобразуем путь к нужному нам виду
         */
        $path = $this->get_path($path);

        if (!$path)
        	return false;

        if ($this->mFtpHost[$this->mDomain] != 'local')
        {
            $opts = array('ftp' => array('overwrite' => true));
            $context = stream_context_create($opts);

            $this->mDirHandle = @opendir($path, $context);
        }
        else
        {
            $this->mDirHandle = @opendir($path);
        }

        return $this->mDirHandle;
    }

    /**
     * Закрывает директорию
     *
     * Если передан параметр $handle и он является дескриптором открытой директории, то закрывается эта директория.
     * Иначе используется переменная класса mDirHandle.
     *
     * @param resource $handle Дескриптор директории.
     * @return bool TRUE в случае успеха и FALSE в случае неудачи.
     */
    public function closedir($handle = false)
    {
        if (!is_resource($handle))
            $handle = $this->mDirHandle;

        $this->mDirHandle = false;

        return @closedir($handle);
    }

    /**
     * Возвращает следующее имя файла или директории
     *
     * Если параметр $handle является дескриптором открытой директории, то чтение идет из этой директории.
     * Иначе используется переменная класса mDirHandle.
     *
     * @param resource $handle Дескриптор открытой директории.
     * @return string Имя файла или директории или FALSE в случае неудачи.
     */
    public function readdir($handle = false)
    {
        if (!is_resource($handle))
            $handle = $this->mDirHandle;

        $filename = @readdir($handle);
        while ($filename == '.' || $filename == '..')
            $filename = readdir($handle);

        return $filename;
    }

    /**
     * Копирует файл или директорию
     *
     * Если в качестве параметра $src передается имя файла, то параметр $dest_real также обязательно должен быть
     * именем файла. С помощью данного метода можно также переименовывать файлы. Если в качестве параметра $src
     * передается имя директории, то параметр $dest_real также обязательно должен быть именем директории. В этом случае
     * в директории $dest_real будет создана поддиректория.
     *
     * @param string $src  Путь к копируемому файлу или директории.
     * @param string $dest_real Путь к новому файлу (копии) или директории.
     * @param bool $overwrite Перезаписывать файл или нет (TRUE - перезаписывать, FALSE - нет
     *
     * @return bool TRUE в случае успешного копирования, FALSE в случае неудачи
     */
    public function copy($src, $dest_real, $overwrite=false)
    {
        /**
         * Копируется файл или директория
         */
        $src_is_d = $this->is_dir($src); // TRUE если директория

        if ($src_is_d)
        {
            /**
             * Если копируем директорию
             */
            $src = str_replace('\\', '/', rtrim($src, '/\\'));
            $dest = rtrim($dest_real, '/\\');

            /**
             * Определяем имя последней директории в пути $src
             */
            $last_pos = strrchr($src, '/');
            if ($last_pos)
                $last_dir = substr($last_pos, 1);
            else
                $last_dir = $src;

            /**
             * Добавляем имя последней директории в пути $dest_real (создаем поддиректорию, в которую и будет
             * производиться копирования.
             */
            $dest .= '/'.$last_dir;

            /**
             * Копируем директорию
             */
            $res = $this->copy_dir($src, $dest, $overwrite);

            return $res;
        }

        /**
         * Преобразуем пути к нужному нам виду
         */
        $src = $this->get_path($src);
        $dest = $this->get_path($dest_real);

        if (!$src || !$dest)
        	return false;

        if (!$overwrite && $this->exists($dest_real))
        {
            /**
             * Если файл уже существует и не включен режим перезаписи, то возвращаем FALSE
             */
            return false;
        }

        if (!$this->exists(dirname($dest_real)))
        {
            /**
             * Если директория, в которую производится копирование, не существует,
             * то создаем ее
             */
            $this->mkdir(dirname($dest_real));
        }

        if ($overwrite && $this->mFtpHost[$this->mDomain] != 'local' && $this->exists($dest_real) && !$this->is_dir($dest_real))
        {
            /**
             * Если включен режим перезаписи и мы работаем через FTP и файл, в который производится копирование
             * существует, то удаляем этот файл.
             */
            $this->delete($dest_real);
        }

        /**
         * Копируем файл
         */
        $res = @copy($src, $dest);

        if ($res && $this->mFtpHost[$this->mDomain] == 'local')
        {
            /**
             * Для локального режима выставляем права на файл. Для FTP файл получит права пользователя FTP.
             */
            @chmod($dest, $this->mFileMode);
        }

        return $res;
    }

    /**
     * Копирует одну директорию в другую.
     *
     * Может вызываться рекурсивно.
     *
     * @param string $src  Путь к директории, которую копируем.
     * @param string $dest Путь к директории, в которую копируем.
     * @param bool   $overwrite Записывать поверх или нет.
     *
     * @return bool TRUE, если копирование прошло успешно и FALSE в противном случае.
     */
    private function copy_dir($src, $dest, $overwrite=false)
    {
        if (!$this->is_dir($dest))
        {
            /**
             * Если директории, в которую производится копирование, не существует, то создаем ее
             */
            if (!$this->mkdir($dest))
            {
                /**
                 * Если создать не удалось, то возвращаем FALSE
                 */
                return false;
            }
        }
        elseif (!$overwrite)
        {
            return false;
        }

        /**
         * Считываем содержимое директории, которую копируем
         */
        $dir_contents = $this->scandir($src);

        if (is_array($dir_contents))
        {
            foreach ($dir_contents as $item)
            {
                /**
                 * Производим копирование каждого файла и директории из считанного списка
                 */
                $src_file = $src.'/'.$item;
                $dest_file = $dest.'/'.$item;

                if ($item != '.' && $item != '..' && $this->is_dir($src_file))
                {
                    /**
                     * Если копируем директорию, то вызываем рекурсию
                     */
                    $res = $this->copy_dir($src_file, $dest_file, $overwrite);
                    if (!$res)
                    {
                        /**
                         * Если скопировать не удалось, то возвращаем FALSE
                         */
                        return false;
                    }
                }
                elseif ($item != '.' && $item != '..')
                {
                    /**
                     * Копирование файла
                     */
                    $exists = $this->exists($dest_file);
                    if (!$overwrite && $exists)
                    {
                        /**
                         * Если файл существует и запрещена перезапись, то возвращаем FALSE
                         */
                        return false;
                    }

                    if ($exists && $this->mFtpHost[$this->mDomain] != 'local')
                    {
                        /**
                         * Если файл существует и работа идет через FTP, то
                         * удаляем его
                         */
                        $this->delete($dest_file);
                    }

                    $src_file_real = $this->get_path($src_file);
                    $dest_file_real = $this->get_path($dest_file);

                    if (!$src_file_real || !$dest_file_real)
        				return false;

                    /**
                     * Копируем файл
                     */
                    $res = copy($src_file_real, $dest_file_real);

                    if (!$res && $this->filesize($src_file) != 0)
                    {
                        /**
                         * Иногда файлы с нулевым размером могут приводить к ошибке, игнорируем их
                         */
                        return false;
                    }

                    if ($this->mFtpHost[$this->mDomain] == 'local')
                    {
                        /**
                         * При локальной работе меняем атрибуты файла
                         */
                        @chmod($dest_file_real, $this->mFileMode);
                    }
                }
            }
        }
        else
        {
            return false;
        }

        return true;
    }

    /**
     * Проверяет, существует ли файл или директория
     *
     * @param string $path  Путь к файлу или директории.
     *
     * @return bool TRUE, если файл существует, FALSE в ином случае.
     */
    public function exists($path)
    {
        $path = str_replace('\\', '/', trim($path, '/\\'));
        $last_pos = strrchr($path, '/');
        if ($last_pos)
        {
            /**
             * Если есть путь
             */
            $file = substr($last_pos, 1);
            $path = substr($path, 0, strrpos($path, '/'));
        }
        else
        {
            /**
             * Файл или директория располагаются в корневой директории
             */
            $file = $path;
            $path = '/';
        }

        if (!isset($this->mFtpHost[$this->mDomain]))
        	return false;

        if ($this->mFtpHost[$this->mDomain] == 'local')
        {
            /**
             * При локальной работе просто вызываем file_exists()
             */
            $path = $this->get_path($path);

            if (!$path)
        		return false;

            return @file_exists($path.'/'.$file);
        }

        /**
         * При работе через FTP считываем содержимое директории и проверяем,
         * есть ли переданное имя файла или директории в списке
         */
        $files = $this->scandir($path);
        if (!$files)
            return false;
        elseif ($file == '')
            return true;

        return @array_search($file, $files) !== false;
    }

    /**
     * Создает полностью путь
     *
     * @param string $path  Путь, который надо создать.
     * @return bool TRUE, если путь создан корректно, FALSE в ином случае.
     */
    public function mkdir($path)
    {
        if ($this->exists($path))
            return true;

        $path = $this->get_path($path);

        if (!$path)
        	return false;

        if ($this->mFtpHost[$this->mDomain] == 'local' && strtoupper(substr(PHP_OS, 0, 3)) === 'WIN')
        {
            /**
             * При локальной работе в системе Windows заменяем прямые слэши на обратные
             * (глюк PHP)
             */
            $path = str_replace('/', '\\', $path);
        }

        /**
         * Создаем путь (только для PHP 5.0)
         */
        $oldmask = @umask(0);
        $result = @mkdir($path, $this->mDirMode, true);
        @umask($oldmask);

        return $result;
    }

    /**
     * Переименовывает директорию или файл
     *
     * @param string $old_path  Путь к директории или файлу.
     * @param string $new_path  Путь к переименованной директории или файлу.
     * @return bool TRUE, если переименование прошло успешно, FALSE в ином случае.
     */
    public function rename($old_path, $new_path)
    {
        $old_path = $this->get_path($old_path);
        $new_path = $this->get_path($new_path);

        if (!$old_path || !$new_path)
        	return false;

        /**
         * Переименовываем директорию или файл (только для PHP 5.0)
         */
        return @rename($old_path, $new_path);
    }

    /**
     * Проверяет, является ли переданный параметр файлом
     *
     * @param string $path Путь к файлу.
     * @return bool TRUE, если файл и FALSE в ином случае.
     */
    public function is_file($path)
    {
    	if (!isset($this->mFtpHost[$this->mDomain]))
    		return false;

        if ($this->mFtpHost[$this->mDomain] == 'local')
        {
            $path = $this->get_path($path);

			if (!$path)
        		return false;

            return @is_file($path) || @is_link($path);
        }
        else
        {
            return $this->exists($path) && !$this->is_dir($path);
        }
    }

    /**
     * Проверяет, является ли переданный параметр символической ссылкой
     *
     * @param string $path Путь к ссылке.
     * @return bool TRUE, если ссылка и FALSE в ином случае.
     */
    public function is_link($path)
    {
    	$path = $this->get_path($path);

        if (!$path)
        	return false;

        if ($this->mFtpHost[$this->mDomain] == 'local')
        {
            return @is_link($path);
        }
        else
        {
            /**
             * При работе через FTP
             */
            if (function_exists('curl_init'))
            {
                /**
                 * Если доступно расширение PHP cURL, то проверяем массив класса mParams, который
                 * заполняется методом scandir()
                 */
                if (array_key_exists($path, $this->mParams))
                {
                    return $this->mParams[$path][0];
                }
                else
                {
                    return @is_link($path);
                }
            }
            else
            {
                return @is_link($path);
            }
        }
    }

    /**
     * Проверяет, является ли переданный параметр директорией
     *
     * @param string $path Путь к директории.
     * @return bool TRUE, если директория и FALSE в ином случае.
     */
    public function is_dir($path)
    {
        if ($path == '/')
            return true;

        $path = $this->get_path($path);

        if (!$path)
        	return false;

        if ($this->mFtpHost[$this->mDomain] == 'local')
        {
            return @is_dir($path) && !@is_link($path);
        }
        else
        {
            /**
             * При работе через FTP
             */
            if (function_exists('curl_init'))
            {
                /**
                 * Если доступно расширение PHP cURL, то проверяем массив класса mParams, который
                 * заполняется методом scandir()
                 */
                if (array_key_exists($path, $this->mParams))
                {
                    return $this->mParams[$path][0];
                }
                else
                {
                    return @is_dir($path) && !@is_link($path);
                }
            }
            else
            {
                return @is_dir($path) && !@is_link($path);
            }
        }
    }

    /**
     * Удаляет файл или директорию
     *
     * @param string $path  Путь к удаляемому файлу или директории.
     *
     * @return bool TRUE - если удаление прошло успешно и FALSE в ином случае
     */
    public function delete($path)
    {
    	$path = preg_replace('![/]+!', '/', str_replace('\\', '/', $path));

    	if (trim($path) == '' || trim($path) == '/' || rtrim($path, '/') == $this->mFtpBasedir[$this->mDomain])
    		return false;

        $is_d = $this->is_dir($path);
        if ($is_d)
        {
            /**
             * Если удаляется директория, то запускаем рекурсию
             */
            return $this->delete_dir($path);
        }
        else
        {
        	$path = $this->get_path($path);

        	if (!$path)
        		return false;

            return @unlink($path);
        }
    }

    /**
     * Удаляет директорию и все ее содержимое
     *
     * Может вызываться рекурсивно.
     *
     * @param string $path Путь к директории.
     * @return bool TRUE, если директория удалена успешно, FALSE в ином случае.
     */
    private function delete_dir($path)
    {
        /**
         * Считываем содержимое удаляемой директории
         */
        $dir_contents = $this->scandir($path);
        $path = rtrim($path, '/\\');

        if ($dir_contents)
        {
            foreach ($dir_contents as $item)
            {
                if ($this->is_dir($path.'/'.$item) && $item != '.' && $item != '..')
                {
                    /**
                     * Если удаляем директорию, то запускаем рекурсию
                     */
                    $this->delete_dir($path.'/'.$item);
                }
                elseif ($item != '.' && $item != '..')
                {
                	if ($this->exists($path.'/'.$item))
                        @unlink($this->get_path($path.'/'.$item));
                }
            }
        }

        /**
         * Удаляем исходную директорию
         */
        if ($this->exists($path))
            return @rmdir($this->get_path($path));
        else
            return true;
    }

    /**
     * Считывает содержимое директории
     *
     * Возвращает массив, содержащий список директорий и файлов, расположенных в указанной директории.
     *
     * @param string $path Путь к директории.
     * @param int $order Режим сортировки списка файлов.
     *
     * @return array Список директорий и файлов, расположенных в указанной директории.
     */
    public function scandir($path, $sort = SB_VFS_SCANDIR_ORDER_NAME_ASC)
    {
        $path = $this->get_path($path);

        if (!$path)
        	return false;

        if (array_key_exists($path.'_'.$sort, $this->mList))
        {
            /**
             * Если до этого подобный список формировался, то используем его
             */
            return $this->mList[$path.'_'.$sort];
        }

        $result = array();
        if ($this->mFtpHost[$this->mDomain] != 'local' && function_exists('curl_init'))
        {
            /**
             * При работе через FTP и при наличии расширения PHP cURL, используем это расширение,
             * работает гораздо быстрее
             */
            $ch = curl_init($path.'/');

            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_FTP_USE_EPSV, 1);
            curl_setopt($ch, CURLOPT_TIMEOUT, 5);
            curl_setopt($ch, CURLOPT_CRLF, 1);

            $result = trim(curl_exec($ch));
            curl_close($ch);
            if ($result === false)
                return false;

            if (!is_string($result) || strlen($result) < 10)
            {
                /**
                 * Директория пустая, результат в этом случае содержит 1
                 */
                return array();
            }

            $res = explode("\r\n", $result);
            $result = array();

            foreach ($res as $value)
            {
                /**
                 * Формируем список файлов и директорий и их параметров
                 */
                $value = preg_replace('/\s+/', ' ', $value);
                $params = explode(' ', $value);

                // файл или директория
                $is_dir = substr($params[0], 0, 1) == 'd';
                // имя файла
                $file = $params[8];
                array_splice($params, 8);
                array_unshift($params, $is_dir);
                $params[1] = substr($params[1], 1);

                // кэшируем результаты запроса
                $this->mParams[$path.'/'.$file] = $params;
                $result[] = $file;
            }
        }
        else
        {
	        if ($sort != SB_VFS_SCANDIR_ORDER_NAME_ASC && $sort != SB_VFS_SCANDIR_ORDER_NAME_DESC)
	        {
	        	$dir = @opendir($path);
	        	if ($dir)
                {
                    while (false !== ($file = @readdir($dir)))
                    {
                        if (($file != '.') && ($file != '..') && @is_file($path.'/'.$file))
                        {
                            $result[$file] = array();
                            $result[$file]['size'] = @filesize($path.'/'.$file);
                            $result[$file]['time'] = @filectime($path.'/'.$file);
                        }
                    }

	                function sort_size_asc($a, $b)
		            {
		                if ($GLOBALS['sb_vfs_result'][$a]['size'] == $GLOBALS['sb_vfs_result'][$b]['size'])
		                    return 0;

		                return ($GLOBALS['sb_vfs_result'][$a]['size'] > $GLOBALS['sb_vfs_result'][$b]['size'] ? 1: -1);
		            }

                	function sort_size_desc($a, $b)
		            {
		                if ($GLOBALS['sb_vfs_result'][$a]['size'] == $GLOBALS['sb_vfs_result'][$b]['size'])
		                    return 0;

		                return ($GLOBALS['sb_vfs_result'][$a]['size'] < $GLOBALS['sb_vfs_result'][$b]['size'] ? 1: -1);
		            }

                	function sort_time_asc($a, $b)
		            {
		                if ($GLOBALS['sb_vfs_result'][$a]['time'] == $GLOBALS['sb_vfs_result'][$b]['time'])
		                    return 0;

		                return ($GLOBALS['sb_vfs_result'][$a]['time'] > $GLOBALS['sb_vfs_result'][$b]['time'] ? 1: -1);
		            }

                	function sort_time_desc($a, $b)
		            {
		                if ($GLOBALS['sb_vfs_result'][$a]['time'] == $GLOBALS['sb_vfs_result'][$b]['time'])
		                    return 0;

		                return ($GLOBALS['sb_vfs_result'][$a]['time'] < $GLOBALS['sb_vfs_result'][$b]['time'] ? 1: -1);
		            }

		            $GLOBALS['sb_vfs_result'] = &$result;

		            if ($sort == SB_VFS_SCANDIR_ORDER_SIZE_ASC)
		            	@uksort($result, 'sort_size_asc');
		            elseif ($sort == SB_VFS_SCANDIR_ORDER_SIZE_DESC)
		            	@uksort($result, 'sort_size_desc');
		            elseif ($sort == SB_VFS_SCANDIR_ORDER_TIME_ASC)
		            	@uksort($result, 'sort_time_asc');
		            elseif ($sort == SB_VFS_SCANDIR_ORDER_TIME_DESC)
		            	@uksort($result, 'sort_time_desc');

		            $result = array_keys($result);
		            unset($GLOBALS['sb_vfs_result']);
                }
	        }
	        else
	        {
	            if ($sort == SB_VFS_SCANDIR_ORDER_NAME_DESC)
	        	{
	            	$result = @scandir($path, 1);
	        	}
	        	else
	        	{
	        		$result = @scandir($path);
	        	}
	        }
        }

        // кэшируем результаты запроса
        $this->mList[$path.'_'.$sort] = $result;

        return $result;
    }

    /**
     * Открывает файл на чтение или запись
     *
     * @param string $path Путь к файлу.
     * @param string $mode Режим открытия файла (r - чтение, w - запись).
     * @return resource Дескриптор файла или FALSE, если файл открыть не удалось.
     */
    public function fopen($path, $mode)
    {
        $path = $this->get_path($path);

        if (!$path)
        	return false;

        if (!strpos($mode, 'b'))
        {
            /**
             * Всегда прибавляем к режиму открытия файлов флаг <i>b</i>, чтобы не было
             * проблем с бинарными файлами.
             */
            $mode .= 'b';
        }

        $mask = sbPlugins::getSetting('sb_file_rights');
        $mask = 777 - $mask;
        $oldmask = @umask(octdec($mask));

        if ($this->mFtpHost[$this->mDomain] != 'local')
        {
            $opts = array('ftp' => array('overwrite' => true));
            $context = @stream_context_create($opts);

            $this->mFileHandle = @fopen($path, $mode, false, $context);
        }
        else
        {
            $this->mFileHandle = @fopen($path, $mode);
        }

        @umask($oldmask);

        return $this->mFileHandle;
    }

    /**
     * Закрывает файл
     *
     * Если параметр $handle не является дескриптором открытого файла, то используется переменная класса mFileHandle.
     *
     * @param resource $handle Дескриптор файла. Если не передан, используется дескриптор, возвращенный последним вызовом метода fopen.
     * @return bool TRUE в случае успеха и FALSE в противном случае.
     */
    public function fclose($handle=false)
    {
        if (!is_resource($handle))
            $handle = $this->mFileHandle;

        $this->mFileHandle = false;

        $res = @fclose($handle);

        return $res;
    }

    /**
     * Определяет, находится ли внутренний указатель файла в конце файла
     *
     * Если параметр $handle не является дескриптором открытого файла, то используется переменная класса mFileHandle.
     *
     * @param resource $handle Дескриптор файла. Если не передан, используется дескриптор, возвращенный последним вызовом метода fopen.
     *
     * @return bool TRUE, если указатель в конце файла и FALSE в ином случае.
     */
    public function feof($handle=false)
    {
        if (!is_resource($handle))
            $handle = $this->mFileHandle;

        return @feof($handle);
    }

    /**
     * Считывает следующий символ из файла
     *
     * Если параметр $handle не является дескриптором открытого файла, то используется переменная класса mFileHandle.
     *
     * @param resource $handle Дескриптор файла. Если не передан, используется дескриптор, возвращенный последним вызовом метода fopen.
     *
     * @return char Считанный символ или FALSE, если указатель в конце файла.
     */
    public function fgetc($handle=false)
    {
        if (!is_resource($handle))
            $handle = $this->mFileHandle;

        return @fgetc($handle);
    }

    /**
     * Считывает строку из файла и парсит ее как CSV-формат
     *
     * Если параметр $handle не является дескриптором открытого файла, то используется переменная класса mFileHandle.
     *
     * @param resource $handle Дескриптор файла. Если не передан, используется дескриптор, возвращенный последним вызовом метода fopen.
     * @param int $length Максимальная длина считываемой строки.
     * @param char $del Разделитель полей CSV-формата.
     * @return array Массив считанных полей или FALSE в случае возникновения ошибки.
     */
    public function fgetcsv($handle=false, $length=1024, $del=';')
    {
        if (!is_resource($handle))
            $handle = $this->mFileHandle;

        return @fgetcsv($handle, $length, $del);
    }

    /**
     * Считывает строку из файла
     *
     * Если параметр $handle не является дескриптором открытого файла, то используется переменная класса mFileHandle.
     *
     * @param int $length Максимальное кол-во символов в считываемой строке.
     * @param resource $handle Дескриптор файла. Если не передан, используется дескриптор, возвращенный последним вызовом метода fopen.
     * @return string Считанная строка или FALSE в случае возникновения ошибки.
     */
    public function fgets($length=-1, $handle=false)
    {
        if (!is_resource($handle))
            $handle = $this->mFileHandle;

        if ($length != -1)
            return @fgets($handle, $length);
        else
            return @fgets($handle);
    }

    /**
     * Возвращает содержимое всего файла в виде строки
     *
     * @param string $path Путь к файлу.
     * @return string Содержимое файла или FALSE в случае возникновения ошибки.
     */
    public function file_get_contents($path)
    {
        $path = $this->get_path($path);

        if (!$path)
        	return false;

        return @file_get_contents($path);
    }

    /**
     * Заменяет содержимое файла переданными данными
     *
     * Если файл не существует, он будет создан.
     *
     * @param string $path Путь к файлу.
     * @param mixed $data Строка или массив с данными, которые следует записать в файл.
     * @return int Кол-во записанных в файл байт или FALSE в случае ошибки
     */
    public function file_put_contents($path, $data)
    {
        $path = $this->get_path($path);

        if (!$path)
        	return false;

		$mask = sbPlugins::getSetting('sb_file_rights');
        $mask = 777 - $mask;

        $oldmask = @umask(octdec($mask));

        if ($this->mFtpHost[$this->mDomain] != 'local')
        {
            $opts = array('ftp' => array('overwrite' => true));
            $context = @stream_context_create($opts);

			$res = @file_put_contents($path, $data, 0, $context);
        }
        else
        {
            $res = @file_put_contents($path, $data);
        }

        @umask($oldmask);
        return $res;
    }

    /**
     * Считывает содержимое файла в массив
     *
     * @param string $path Путь к файлу.
     * @return array Массив строк из файла или FALSE в случае ошибки.
     */
    public function file($path)
    {
        $path = $this->get_path($path);

        if (!$path)
        	return false;

        return @file($path);
    }

    /**
     * Возвращает размер файла в байтах
     *
     * @param string $path Путь к файлу.
     * @return int Размер файла в байтах или FALSE в случае ошибки.
     */
    public function filesize($path)
    {
        $real_path = $this->get_path($path);

        if (!$real_path)
        	return false;

        $size = @filesize($real_path);
        if ($size && $this->exists($path))
        {
            /**
             * Если не удалось получить размер файла, пытаемся вытащить его из кэша
             */
            if (isset($this->mParams[$real_path]))
            {
                $size = $this->mParams[$real_path][5];
            }
        }

        return $size;
    }

    /**
     * Возвращает время последнего изменения файла
     *
     * @param string $path Путь к файлу.
     * @return int Время последнего изменения файла (UNIX TIMESTAMP) или FALSE в случае ошибки.
     */
    public function filectime($path)
    {
        $real_path = $this->get_path($path);

        if (!$real_path)
        	return false;

        $time = @filectime($real_path);
        if ($time <= 0 && $this->exists($path) && isset($this->mParams[$real_path]))
        {
            return $this->mParams[$real_path][6].' '.$this->mParams[$real_path][7].' '.$this->mParams[$real_path][8];
        }
        return $time;
    }

    /**
     * Выводит содержимое файла в браузер пользователя
     *
     * Если параметр $handle не является дескриптором открытого файла, то используется переменная класса mFileHandle.
     *
     * @param resource $handle Дескриптор файла. Если не передан, используется дескриптор, возвращенный последним вызовом метода fopen.
     * @return int Кол-во выведенных символов или FALSE в случае ошибки.
     */
    public function fpassthru($handle=false)
    {
        if (!is_resource($handle))
            $handle = $this->mFileHandle;

        return @fpassthru($handle);
    }

    /**
     * Меняет атрибуты доступа к файлу
     *
     * Только при работе с файловой системой локально.
     *
     * @param string $path Путь к файлу
     * @param int $mode Десятичное число, задающее права доступа (например 755).
     * @return bool TRUE, если удалось сменить атрибуты, и FALSE в ином случае.
     */
    public function chmod($path, $mode)
    {
    	if (!isset($this->mFtpHost[$this->mDomain]))
    		return false;

        if ($this->mFtpHost[$this->mDomain] != 'local')
            return false;

        $path = $this->get_path($path);

        if (!$path)
        	return false;

        return @chmod($path, octdec($mode));
    }

    /**
     * Считывает заданное кол-во байтов из файла
     *
     * Если параметр $handle не является дескриптором открытого файла, то используется переменная класса mFileHandle.
     *
     * @param int $length Кол-во символов, которое необходимо считать.
     * @param resource $handle Дескриптор файла. Если не передан, используется дескриптор, возвращенный последним вызовом метода fopen.
     *
     * @return string Считанная строка или FALSE в случае возникновения ошибки.
     */
    public function fread($length, $handle=false)
    {
        if (!is_resource($handle))
            $handle = $this->mFileHandle;

        return @fread($handle, $length);
    }

    /**
     * Устанавливает внутренний указатель файла в нужную позицию
     *
     * Только при работе с файловой системой локально.
     *
     * @param int $offset Позиция указателя, относительно начала файла.
     * @param resource $handle Дескриптор файла. Если не передан, используется дескриптор, возвращенный последним вызовом метода fopen.
     *
     * @return int 0 в случае успешного выполнения и -1 в случае ошибки.
     */
    public function fseek($offset, $handle=false)
    {
        if ($this->mFtpHost[$this->mDomain] != 'local')
            return -1;

        if (!is_resource($handle))
            $handle = $this->mFileHandle;

        return @fseek($handle, $offset);
    }

    /**
     * Возвращает пощицию внутреннего указателя файла
     *
     * @param resource $handle Дескриптор файла. Если не передан, используется дескриптор, возвращенный последним вызовом метода fopen.
     *
     * @return int Позиция внутреннего указателя или FALSE в случае возникновения ошибки.
     */
    public function ftell($handle=false)
    {
        if (!is_resource($handle))
            $handle = $this->mFileHandle;

        return @ftell($handle);
    }

    /**
     * Записывает строку в файл
     *
     * @param string $str Строка, которую необходимо записать.
     * @param resource $handle Дескриптор файла. Если не передан, используется дескриптор, возвращенный последним вызовом метода fopen.
     * @return int Кол-во записанных байт или FALSE в случае возникновения ошибки.
     */
    public function fwrite($str, $handle=false)
    {
        if (!is_resource($handle))
            $handle = $this->mFileHandle;

        return @fwrite($handle, $str);
    }

    /**
     * Проверяет, существует ли файл и доступен ли он для чтения
     *
     * Только при работе с файловой системой локально.
     *
     * @param string $path Путь к файлу.
     * @return bool TRUE если файл существует и доступен для чтения и FALSE в ином случае.
     */
    public function is_readable($path)
    {
    	if (!isset($this->mFtpHost[$this->mDomain]))
    		return false;

        if ($this->mFtpHost[$this->mDomain] != 'local')
            return true;

        $path = $this->get_path($path);

        if (!$path)
        	return false;

        return @is_readable($path);
    }

    /**
     * Проверяет, существует ли файл и доступен ли он для записи
     *
     * Только при работе с файловой системой локально.
     *
     * @param string $path Путь к файлу.
     * @return bool TRUE если файл существует и доступен для записи и FALSE в ином случае.
     */
    public function is_writable($path)
    {
        if ($this->mFtpHost[$this->mDomain] != 'local')
            return true;

        $path = $this->get_path($path);

        if (!$path)
        	return false;

        return @is_writable($path);
    }

    /**
     * Возвращает реальный путь к файлу или директории
     *
     * Если используется доступ через FTP, то возвращается URI FTP.
     *
     * @param string $path Путь к файлу или директории.
     * @return string Реальный путь.
     */
    private function get_path($path)
    {
    	if (!isset($this->mFtpHost[$this->mDomain]))
    		return false;

        $path = str_replace('\\', '/', trim($path, '/\\'));

        if (!$this->mLocal)
        {
            $path = $this->mFtpBasedir[$this->mDomain].($path != '' ? '/'.$path : '');
            if ($this->mFtpHost[$this->mDomain] != 'local')
            {
                $path = 'ftp://'.(!defined('SB_USE_ENCRYPT') || SB_USE_ENCRYPT ? sb_decrypt($this->mFtpUser[$this->mDomain], SB_HASH) : $this->mFtpUser[$this->mDomain]).':'.(!defined('SB_USE_ENCRYPT') || SB_USE_ENCRYPT ? sb_decrypt($this->mFtpPassword[$this->mDomain], SB_HASH) : $this->mFtpPassword[$this->mDomain]).'@'.$this->mFtpHost[$this->mDomain].$path;
            }
        }
        elseif ($path != '' && strtoupper(substr(PHP_OS, 0, 3)) !== 'WIN')
        {
            $path = '/'.$path;
        }

        return $path;
    }

    /**
     * Перемещает загруженный файл из темповой директории в указанное место
     *
     * @param string $tmp_name Имя темпового файла.
     * @param string $file_name Путь к файлу с именем файла, куда сохраняем темповый файл.
     *
     * @return bool TRUE, если файл перемещен успешно, FALSE в ином случае.
     */
    public function move_uploaded_file($tmp_name, $file_name)
    {
        $real_file_name = $this->get_path($file_name);

        if (!$real_file_name)
        	return false;

        if ($this->mFtpHost[$this->mDomain] == 'local')
        {
            // при локальной работе достаточно вызвать move_uploaded_file
            if (@move_uploaded_file($tmp_name, $real_file_name))
            {
                @chmod($real_file_name, $this->mFileMode);
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            // если работа идет через FTP, то сначала загружаем файл в темповую директорию,
            // потом копируем его через FTP.
            if (!@is_dir(SB_CMS_TMP_PATH))
            {
                /**
                 * Если темповая директория не существует, то создаем ее
                 */
                $oldmask = @umask(0);
                @mkdir(SB_CMS_TMP_PATH, $this->mDirMode, true);
                @umask($oldmask);
            }


            $src = SB_CMS_TMP_PATH.'/'.time().'_uploaded.tmp';
            if (@move_uploaded_file($tmp_name, $src))
            {
                if (!$this->exists(dirname($file_name)))
                {
                    /**
                     * Если директория, в которую производится копирование, не существует,
                     * то создаем ее
                     */
                    $this->mkdir(dirname($file_name));
                }

                if ($this->exists($file_name) && !$this->is_dir($file_name))
                {
                    /**
                     * Если файл, в который производится копирование
                     * существует, то удаляем этот файл.
                     */
                    $this->delete($file_name);
                }

                /**
                 * Копируем файл
                 */
                $res = @copy($src, $real_file_name);
                @unlink($src);

                return $res;
            }
            else
            {
                return false;
            }
        }
    }

    /**
     * Возвращает информацию об изображении
     *
     * @param string $path Путь к файлу с изображением.
     * @return mixed Информация об изображении или FALSE в случае ошибки.
     */
    public function getimagesize($path)
    {
    	if (!$this->is_file($path))
    		return false;

        $path = $this->get_path($path);

        if (!$path)
        	return false;

        return @getimagesize($path);
    }

    /**
     * Возвращает идентификатор изображения, сформированного из файла
     *
     * @param string $path Путь к файлу.
     * @return mixed Идентификатор изображения или FALSE в случае ошибки.
     */
    public function imagecreatefromjpeg($path)
    {
        $path = $this->get_path($path);
    	if (!$path)
        	return false;

        return imagecreatefromjpeg($path);
    }

    /**
     * Возвращает идентификатор изображения, сформированного из файла
     *
     * @param string $path Путь к файлу.
     * @return mixed Идентификатор изображения или FALSE в случае ошибки.
     */
    public function imagecreatefrompng($path)
    {
        $path = $this->get_path($path);
		if (!$path)
        	return false;

        return @imagecreatefrompng($path);
    }

    /**
     * Возвращает идентификатор изображения, сформированного из файла
     *
     * @param string $path Путь к файлу.
     * @return mixed Идентификатор изображения или FALSE в случае ошибки.
     */
    public function imagecreatefromgif($path)
    {
        $path = $this->get_path($path);
        if (!$path)
        	return false;

        if (function_exists('imagecreatefromgif'))
        {
            return @imagecreatefromgif($path);
        }
        else
        {
            return false;
        }
    }

    /**
     * Генерирует файл с изображением по дескриптору изображения
     *
     * @param handle $img Дескриптор изображения.
     * @param string $path Путь и имя генерируемого файла.
     * @param int $quality Качество файла (от 0 до 100).
     * @return bool TRUE, если файл сгенерирован, и FALSE в ином случае.
     */
    public function imagejpeg($img, $path, $quality=100)
    {
    	if (!$path)
    	{
    		$real_path = null;
    	}
    	else
    	{
	        $real_path = $this->get_path($path);

	        if (!$real_path)
	        	return false;
    	}

        if ($this->mFtpHost[$this->mDomain] == 'local')
        {
            return @imagejpeg($img, $real_path, $quality);
        }
        else
        {
            // если работа идет через FTP, то сначала генерируем файл в темповую директорию,
            // потом копируем его через FTP.
            if (!is_dir(SB_CMS_TMP_PATH))
            {
                /**
                 * Если темповая директория не существует, то создаем ее
                 */
                $oldmask = @umask(0);
                @mkdir(SB_CMS_TMP_PATH, $this->mDirMode, true);
                @umask($oldmask);
            }

            $src = SB_CMS_TMP_PATH.'/'.time().'_image.tmp';
            if (@imagejpeg($img, $src, $quality))
            {
                if (!$this->exists(dirname($path)))
                {
                    /**
                     * Если директория, в которую производится копирование, не существует,
                     * то создаем ее
                     */
                    $this->mkdir(dirname($path));
                }

                if ($this->exists($path) && !$this->is_dir($path))
                {
                    /**
                     * Если файл, в который производится копирование
                     * существует, то удаляем этот файл.
                     */
                    $this->delete($path);
                }

                /**
                 * Копируем файл
                 */
                $res = @copy($src, $real_path);
                @unlink($src);

                return $res;
            }
            else
            {
                return false;
            }
        }
    }

    /**
     * Генерирует файл с изображением по дескриптору изображения
     *
     * @param handle $img Дескриптор изображения.
     * @param string $path Путь и имя генерируемого файла.
     * @param int $quality Сжатие файла (от 0 до 9).
     * @return bool TRUE, если файл сгенерирован, и FALSE в ином случае.
     */
    public function imagepng($img, $path, $quality=0)
    {
    	if (!$path)
    	{
    		$real_path = null;
    	}
    	else
    	{
	        $real_path = $this->get_path($path);

	        if (!$real_path)
	        	return false;
    	}

        if ($this->mFtpHost[$this->mDomain] == 'local')
        {
            return @imagepng($img, $real_path, $quality);
        }
        else
        {
            // если работа идет через FTP, то сначала генерируем файл в темповую директорию,
            // потом копируем его через FTP.
            if (!is_dir(SB_CMS_TMP_PATH))
            {
                /**
                 * Если темповая директория не существует, то создаем ее
                 */
                $oldmask = @umask(0);
                @mkdir(SB_CMS_TMP_PATH, $this->mDirMode, true);
                @umask($oldmask);
            }

            $src = SB_CMS_TMP_PATH.'/'.time().'_image.tmp';
            if (@imagepng($img, $src, $quality))
            {
                if (!$this->exists(dirname($path)))
                {
                    /**
                     * Если директория, в которую производится копирование, не существует,
                     * то создаем ее
                     */
                    $this->mkdir(dirname($path));
                }

                if ($this->exists($path) && !$this->is_dir($path))
                {
                    /**
                     * Если файл, в который производится копирование
                     * существует, то удаляем этот файл.
                     */
                    $this->delete($path);
                }

                /**
                 * Копируем файл
                 */
                $res = @copy($src, $real_path);
                @unlink($src);

                return $res;
            }
            else
            {
                return false;
            }
        }
    }

    /**
     * Генерирует файл с изображением по дескриптору изображения
     *
     * @param handle $img Дескриптор изображения.
     * @param string $path Путь и имя генерируемого файла.
     * @return bool TRUE, если файл сгенерирован, и FALSE в ином случае.
     */
    public function imagegif($img, $path)
    {
    	if (!$path)
    	{
    		$real_path = null;
    	}
    	else
    	{
	        $real_path = $this->get_path($path);

	        if (!$real_path)
	        	return false;
    	}

        if ($this->mFtpHost[$this->mDomain] == 'local')
        {
            return @imagegif($img, $real_path);
        }
        else
        {
            // если работа идет через FTP, то сначала генерируем файл в темповую директорию,
            // потом копируем его через FTP.
            if (!is_dir(SB_CMS_TMP_PATH))
            {
                /**
                 * Если темповая директория не существует, то создаем ее
                 */
                $oldmask = @umask(0);
                @mkdir(SB_CMS_TMP_PATH, $this->mDirMode, true);
                @umask($oldmask);
            }

            $src = SB_CMS_TMP_PATH.'/'.time().'_image.tmp';
            if (@imagegif($img, $src))
            {
                if (!$this->exists(dirname($path)))
                {
                    /**
                     * Если директория, в которую производится копирование, не существует,
                     * то создаем ее
                     */
                    $this->mkdir(dirname($path));
                }

                if ($this->exists($path) && !$this->is_dir($path))
                {
                    /**
                     * Если файл, в который производится копирование
                     * существует, то удаляем этот файл.
                     */
                    $this->delete($path);
                }

                /**
                 * Копируем файл
                 */
                $res = @copy($src, $real_path);
                @unlink($src);

                return $res;
            }
            else
            {
                return false;
            }
        }
    }

    /**
     * Возвращает атрибуты файла или директории
     *
     * @param string $path Путь к файлу или директории.
     * @return mixed Число, соотв. атрибутам файла или директории, или FALSE в случае ошибки.
     */
    public function fileperms($path)
    {
        $path = $this->get_path($path);

        if (!$path)
        	return false;

        return @fileperms($path);
    }
}

$hosts = array();
$dirs = array();
$users = array();
$passwords = array();

foreach ($GLOBALS['sb_domains'] as $key => $value)
{
    $hosts[$key] = $value['ftp_host'];

    $basedir = str_replace('\\', '/', rtrim($value['basedir'], '/\\'));
    if (isset($value['ftp_basedir']))
    {
        $ftp_basedir = str_replace('\\', '/', rtrim($value['ftp_basedir'], '/\\'));
    }
    else
    {
        $ftp_basedir = '';
    }

    $dirs[$key] = $value['ftp_host'] != 'local' ? sb_substr($basedir, sb_strlen($ftp_basedir)) : $basedir;
    $users[$key] = isset($value['ftp_user']) ? $value['ftp_user'] : '';
    $passwords[$key] = isset($value['ftp_password']) ? $value['ftp_password'] : '';

    unset($GLOBALS['sb_domains'][$key]['ftp_host']);
    unset($GLOBALS['sb_domains'][$key]['ftp_user']);
    unset($GLOBALS['sb_domains'][$key]['ftp_password']);
}

$GLOBALS['sbVfs'] = new sbVfs($hosts, $dirs, $users, $passwords);

unset($hosts);
unset($dirs);
unset($users);
unset($passwords);
unset($basedir);
unset($ftp_basedir);
unset($key);
unset($value);
?>